; ===============================================================
; Archive Routines
; ===============================================================

; ---------------------------------------------------------------
; ArcSprite
; Copies an archived large sprite into RAM, then displays
; Input: A = x coordinate
; 	  L = y coordinate
; 	  IX = sprite location in ROM
; ---------------------------------------------------------------

ArcSprite:
.relocate SwapZone
	di
	push HL				; save L, y coord
	push AF				; save A, x coord
	in A, (6)				; A = current page in mem bank A
	push AF				; save A, page
	push IX				; HL = IX
	pop HL
	call GetROMSet			; HL = ROM offset, B = ROM page, set ROM page
	
	ld DE, Buffer
	call IncSet				; A = height byte, inc HL
	ld (DE), A				; (DE), A, C = height byte
	inc DE					; inc DE to Buffer + 1
	ld C, A
	call IncSet				; A = width byte, inc HL
	ld (DE), A				; (DE) = height byte
	push BC				; save B, ROM page
	push HL				; save HL, pt to sprite content
	ld H, C
	ld L, A
	call RMultHL				; height * width * 2 --> HL, IX (size of sprite)
	add HL, HL
	push HL
	pop IX
	pop HL					; get HL, pt to sprite content
	pop BC					; get B, ROM page
	
	ld DE, Buffer + 2
ArcSpriteLp:
	call IncSet
	ld (DE), A
	inc DE
	dec IX
	ld A, IXL				; Repeat until IX == 0
	or IXH					; can't do or IXL (assembles to or IXH)
	jr nz, ArcSpriteLp
	
	pop AF					; get A = current page in mem bank A
	out (6), A				; set ROM page
	pop AF
	pop HL
	ld IX, Buffer
	RCall( cgsLargeSprite )
	ei
	ret

RMultHL:			; subroutine - multiplies H and L, ~13 bytes
	ld E, L
	ld L, 0		; HL = zero
	ld D, L		; DE = zero extended L
	ld B, 8		; B = 8
RMultHLLp:
	add HL, HL
	jr nc, RMultHLSkip
	add HL, DE
RMultHLSkip:
	djnz RMultHLLp
	ret
.endrelocate

; ---------------------------------------------------------------
; ArcSprite8
; Copies archived sprite to RAM, then displays
; Input: A = x coordinate
; 	  L = y coordinate
; 	  IX = sprite location in ROM
; ---------------------------------------------------------------

ArcSprite8:
.relocate SwapZone
	push HL
	push AF
	in A, (6)				; A = current page in mem bank A
	push AF
	push IX				; IX --> HL
	pop HL
	call GetROMSet			; HL = ROM offset, B = ROM page, set ROM page
	
	ld DE, Buffer				; height --> A, (DE)
	call IncSet				; A = height, inc HL
	ld (DE), A
	add A, A				; A * 2
	ld C, A				; C = A
	
	inc DE					; inc to core of sprite
ArcSprite8Lp:
	call IncSet				; A = byte, inc HL
	ld (DE), A				; put A in buffer
	inc DE					; inc buffer pointer
	dec C
	jr nz, ArcSprite8Lp
	
	pop AF					; get A = current page
	out (6), A				; set ROM page
	pop AF
	pop HL
	ld IX, Buffer
	RCall( cgsPutSprite )
	ret
.endrelocate

; ---------------------------------------------------------------
; ArcPKMN / ArcFront
; Displays front / back pic of pkmn, stored in ROM
; Input: A = pkmn number
;        H = x coordinate
;        L = y coordinate
;        FrontFlag = 1 for front sprites, 0 for back sprites
; ---------------------------------------------------------------

ArcPKMN:
ArcFront:
.relocate SwapZone
	push AF
	push HL
	ld BC, 24 * 256 + 25
	RCall( cDrawRectWhite3 )		; draw white background
	ld A, (PKMNPicFrom)			; A = ROM page of PKMNPICF
	bit FrontFlag, (IY + Asm_Flag2)
	jr nz, ArcPKMNChkF
ArcPKMNChkB:
	ld A, (PKMNPicBrom)
	
ArcPKMNChkF:
	or A
	pop HL
	pop BC
	ld A, B
	ret z					; ret if prgm not found
	
	push HL				; save coords
	ld H, A
	ld L, 3 * 25 * 2
	call MultHL				; HL = offset of pic, relative to PICF program
	in A, (6)				; A = current page in mem bank A
	push AF				; save A
	bit FrontFlag, (IY + Asm_Flag2)
	jr nz, ArcPKMNF
ArcPKMNB:
	call GetROMB				; HL = offset in memory, B = ROM page
	jr ArcPKMNCont
	
ArcPKMNF:
	call GetROMF				; HL = offset in memory, B = ROM page
	
ArcPKMNCont:
	call LoadCIndPagedSet		; set ROM page
	ld DE, Buffer				; DE = ptr to Buffer
	ld A, 25
	ld (DE), A				; height = 25
	inc DE
	ld A, 3
	ld (DE), A				; width = 24 = 3 * 8
	inc DE
	
	ld C, 3 * 25 * 2			; C = number of bytes to copy
ArcPKMNLp:
	call IncSet				; A = byte from ROM, inc HL
	ld (DE), A				; store byte in buffer
	inc DE					; increment buffer pointer
	dec C
	jr nz, ArcPKMNLp			; loop until A = 0
	
	pop AF					; get A, page in mem bank A
	out (6), A				; set ROM page
	pop HL					; get coords
	
	ld A, H
	ld IX, Buffer
	RCall( cgsLargeSprite )		; display sprite
	ret
.endrelocate
	
; ---------------------------------------------------------------
; ArcBack
; Displays back pic of pkmn, stored in ROM
; Input: A = pkmn number
; ---------------------------------------------------------------

ArcBack:
.relocate SwapZone
	res FrontFlag, (IY + Asm_Flag2)
	RCall( cArcPKMN )
	set FrontFlag, (IY + Asm_Flag2)
	ret
.endrelocate

; ---------------------------------------------------------------
; ArcText:
; Copies archived text to RAM, then displays
; Input: HL = text (ROM)
;	  (PenCol)
;	  (PenRow)
; ---------------------------------------------------------------
; TODO: space optimized RArcText routine
ArcText:
.relocate SwapZone
	xor A
	ld (TextSize), A			; no size limit
	RCall( cArcTextDc )			; decode archived text, returns HL = pt to decoded text
	b_call(_VPutS)			; display string
	ret
.endrelocate

; ---------------------------------------------------------------
; ArcTextN:
; Copies archived text to RAM, then displays; has max length
; Input: B = max length (chars)
;	  HL = text (ROM)
;	  (PenCol)
;	  (PenRow)
; ---------------------------------------------------------------

ArcTextN:
.relocate SwapZone
	ld A, B
	ld (TextSize), A			; set size limit to B
	push BC				; save B, max length
	RCall( cArcTextDc)			; decode archived text, returns HL = pt to decoded text
	pop BC
	RCall( cPutSN )			; display string at buffer, w/ B = max length
	ret
.endrelocate

; ---------------------------------------------------------------
; ArcTextDc
; Decodes archived text
; Input: HL = offset of text
;        (TextSize) = size of text, 0 if no limit
; Output: HL = Buffer = decoded text
; ---------------------------------------------------------------

ArcTextDc:					; ~ 160 bytes
.relocate SwapZone
	in A, (6)				; A = current page in mem bank A
	push AF				; save A
	call GetROMSet			; HL = ROM offset, B = ROM page, set ROM page
	
	ld IX, Buffer				; copy decoded text to buffer
	xor A
	ld C, A				; C = 0, size 
	ld D, A
	ld E, A				; DE = 0, huff code
	
ArcTextLp:
	ld A, (HL)				; A = encoded char
	inc HL					; inc HL to next char
	
	bit 7, H
	jr z, ArcTextNotOver
	push AF
	in A, (6)				; A = current page in mem bank A
	ld B, A				; B = current page in mem bank A
	inc B					; B = next page
	call LoadCIndPagedSet		; set ROM page
	ld HL, $4000				; HL = $4000, start address of ROM page bank
	pop AF
	
ArcTextNotOver:
	ld B, 8				; 8 bits in byte
	
ArcTextDcLp:
	sla A					; shift A left, carry <- MSB of A
	rl E					; shift E left, LSB of E <- carry
	rl D					; shift D left, LSB of D <- carry
	inc C					; inc size
	
	push HL
	call ArcTextChk
	pop HL
	jr z, ArcTextDcEnd			; return if done
	
	djnz ArcTextDcLp			; loop until B = 0
	jr ArcTextLp				; get next encoded char
	
ArcTextDcEnd:
	xor A
	ld (IX), A				; null terminate buffer
	inc IX
	pop AF					; get A = page in mem bank A
	out (6), A				; set ROM page
	ld HL, Buffer
	ret
	
ArcTextChk:					; subroutine - chks if huffcode matches, ret z if done, input: C = len, DE = code
	push BC				; save B, C
	push AF				; save A
	ld A, C				; A = len
	cp 3					; if len < 3
	jr c, ArcTextChkRetNZ		; then ret w/ nz, 0 entries in table
	
	push DE				; save DE
	sub 3					; A = len - 3
	ld B, A				; B = len - 3
	or A
	jr z, ArcTextChkLpSkip
	
ArcTextChkLpB:
	srl D
	rr E					; DE = DE >> 1
	djnz ArcTextChkLpB			; loop until B = 0
	
ArcTextChkLpSkip:
	ld C, E				; C = leftmost 3 bits of huffcode in DE
	pop DE					; get DE
	
	ld B, 0
	sla C
	sla C					; BC = C * 4
	ld HL, htTables
	add HL, BC				; HL = pt to htOffset, htTable for huffcode
	
	ld C, A				; restore C from A
	push HL				; save pt to htOffset, htTable
	call LdHLIndRAM			; HL = pt to htOffset for huffcode
	ld B, 0
	add HL, BC
	ld A, (HL)				; A = htOffset[BC]
	ld C, A				; C = A
	inc HL
	ld A, (HL)				; A = htOffset[BC + 1]
	pop HL					; get pt to htOffset, htTable
	sub C					; A = number of entries
	jr z, ArcTextChkRetNZ		; ret w/ nz if 0 entries
	
	ld B, A				; B = number of entries
	inc HL
	inc HL
	call LdHLIndRAM			; HL = pt to htTable for huffcode
	ld A, C				; A = C
	add A, A
	add A, C				; A = A * 3
	
	push BC
	ld B, 0
	ld C, A
	add HL, BC				; HL = pt to row in table
	pop BC
	
ArcTextChkLp:
	push HL				; save HL, pt to table
	call LdHLIndRAM			; HL = huff code
	call RCpHLDE				; compare HL and DE
	pop HL					; get HL, pt to table
	inc HL
	inc HL					; HL += 2, skip huff code, flags preserved
	jr nz, ArcTextChkNeq			; skip if not equal
	
	ld A, (HL)				; A = decoded char
	ld (IX), A				; put A in buffer
	inc IX					; increment buffer pointer
	ld HL, TextSize
	ld B, (HL)				; A = textsize limit
	sla B					; B = B << 1
	jr z, ArcTextChkSkip			; if (TextSize) is 0, then skip
	dec (HL)				; dec (TextSize)
	jr z, ArcTextChk0			; if (TextSize) is 0, then end
	
ArcTextChkSkip:
	or A					; if A = 0, null
	
ArcTextChk0:
	pop BC					; pop and ld don't affect flags
	ld A, B				; get A
	pop BC					; get B, C
	ret z					; then end, ret z
	
	ld C, 0				; C = 0, size
	ld D, C
	ld E, C				; DE = 0, huff code
	inc H
	ret					; ret w/ nz
	
ArcTextChkNeq:
	inc HL					; inc HL to next huff code
	djnz ArcTextChkLp			; loop until B = 0
	
ArcTextChkRetNZ:
	xor A
	inc A
	pop BC
	ld A, B				; get A
	pop BC					; get B, C
	ret					; ret w/ nz
.endrelocate

ArcTextDc2:					; calls ArcTextDc, copies result to Buffer + 100, returns HL = pt to Buffer + 100
.relocate SwapZone
	xor A
	ld (TextSize), A			; no text size limit
	RCall( cArcTextDc )			; returns HL = pt to start of decoded text, IX = pt to end of decoded text
	
	push HL				; save HL, pt to text
	ex DE, HL				; DE = pt to start of text
	push IX
	pop HL					; HL = pt to end of text
	or A
	sbc HL, DE				; HL = size of text
	push HL
	pop BC					; BC = size of text
	pop HL					; get HL, pt to text
	
	ld DE, Buffer + 100
	push DE
	ldir					; copy text to Buffer + 100
	pop HL					; HL = Buffer + 100
	ret
.endrelocate