; ===============================================================
; Attack Effect Routines
; ===============================================================

; ---------------------------------------------------------------
; Stat raising move effects
; ---------------------------------------------------------------

AE_StatRaise:
.relocate SwapZone
AE_StatRaiseAtk1:				; ~ 128 bytes
	; raise attack by 1 stage
	ld B, stAtk
	jr AttackEffect_StatRaiseC1
	
AE_StatRaiseDef1:
	; raise defense by 1 stage
	ld B, stDef
	jr AttackEffect_StatRaiseC1
	
AE_StatRaiseEva1:
	; raise evasion by 1 stage
	ld B, stEva
	jr AttackEffect_StatRaiseC1
	
AE_StatRaiseAtk2:
	; raise attack by 2 stages
	ld B, stAtk
	jr AttackEffect_StatRaiseC2
	
AE_StatRaiseDef2:
	; raise defense by 2 stages
	ld B, stDef
	jr AttackEffect_StatRaiseC2
	
AE_StatRaiseSpDef2:
	; raise sp defense by 2 stages
	ld B, stSDef
	jr AttackEffect_StatRaiseC2
	
AE_StatRaiseSpd2:
	; raise speed by 2 stages
	ld B, stSpd
	jr AttackEffect_StatRaiseC2
	
AE_StatRaiseAtkDef:
	; raise atk and def by 1 stage
	ld A, stAtk
	ld B, stDef
	jr AttackEffect_StatRaiseAB
	
AE_StatRaiseSpAtkDef:
	; raise sp atk and def by 1 stage
	ld A, stSAtk
	ld B, stSDef
	jr AttackEffect_StatRaiseAB
	
AE_StatRaiseDefSp:
	; raise def and sp def by 1 stage
	ld A, stDef
	ld B, stSDef
	jr AttackEffect_StatRaiseAB
	
AE_StatRaiseAtkSpd:
	; raise atk and spd by 1 stage
	ld A, stAtk
	ld B, stSpd
	jr AttackEffect_StatRaiseAB
	
AE_StatRaiseAll:
	; raise all stats by 1 stage (qAllStatsRaise)
	ld B, 1
AE_StatRaiseAllStatsLp:			; loop over stats (1 - 5)
	push HL
	push BC
	call AttackEffect_StatRaiseC1
	pop BC
	pop HL
	inc B
	ld A, B
	cp 6
	jr c, AE_StatRaiseAllStatsLp
	ret
	
AttackEffect_StatRaiseAB: 			; raise [stat1] and [stat2] by 1 stage, HL = pt, A = [stat1], B = [stat2]
	push BC
	ld B, A
	push HL
	call AttackEffect_StatRaiseC1
	pop HL
	pop BC					; B = [stat2], C = 1, fall through to StatRaiseC1
	;jp AttackEffect_StatRaiseC1
	
AttackEffect_StatRaiseC1:			; raise [stat] by 1 stage, HL = pt to atk boost, B = [stat], boost = 1
	ld C, 1
	jr AttackEffect_StatRaiseC
	
AttackEffect_StatRaiseC2:			; raise [stat] by 2 stages, HL = pt to atk boost, B = [stat], boost = 2
	ld C, 2
	
AttackEffect_StatRaiseC:			; raise [stat] by C stages, HL = pt to atk boost, B = [stat], C = boost (1 | 2)
	ld D, 0
	ld E, B
	dec E
	add HL, DE				; pointer to [stat] = HL + B - 1
	ld A, (HL)				; A = [stat] boost
	cp 6					; if A == 6, then stat at max
	jr z, AttackEffect_StatRaiseMax
	add A, C				; else, A += C (1 | 2)
	cp 7
	jr nz, AtackEffect_StatRaiseCont
	ld A, 6				; if A == 7, then set A to 6
AtackEffect_StatRaiseCont:
	ld (HL), A
	ld HL, TextStat			; display 'stat rose' msg
	ld (HL), B
	ld A, C
	cp 2
	jr z, AttackEffect_StatRaiseCSharp
	rTextboxRet(tPKMNsStatRose)
	
AttackEffect_StatRaiseCSharp:
	rTextboxRet(tPKMNsStatSharplyRose)	
	
AttackEffect_StatRaiseMax:
	ld HL, TextStat			; display 'stat won't go higher' msg
	ld (HL), B
	rTextboxRet(tPKMNStatWontGoHigher)
.endrelocate
	
; ---------------------------------------------------------------
; Stat lowering move effects
; ---------------------------------------------------------------

AE_StatLower:
.relocate SwapZone
AE_StatLowerDef1:				; ~140 bytes
	; lower defense by 1 stage
	ld B, stDef
	jr AttackEffect_StatLowerC1
	
AE_StatLowerSpDef1:
	; lower sp defense by 1 stage
	ld B, stSDef
	jr AttackEffect_StatLowerC1
	
AE_StatLowerSpAtk1:
	; lower sp attack by 1 stage
	ld B, stSAtk
	jr AttackEffect_StatLowerC1
	
AE_StatLowerSpd1:
	; lower speed by 1 stage
	ld B, stSpd
	jr AttackEffect_StatLowerC1
	
AE_StatLowerAcc1:
	; lower accuracy by 1 stage, but not if keen eye
	push HL
	ld BC, PKMN_AtkBoost - PKMN_Abil
	or A
	sbc HL, BC
	ld A, (HL)				; A = target pkmn ability
	pop HL
	
	cp aKeenEye
	jr nz, AE_StatLowerAcc
	
	ld (TextAbil), A			; set (TextAbil) to aKeenEye
	ld A, stAcc
	ld (TextStat), A			; se (TextStat) to stAcc
	rTextboxRet(tAbilPrevStatLoss)	; disp msg, return
	
AE_StatLowerAcc:
	ld B, stAcc
	jr AttackEffect_StatLowerC1
	
AE_StatLowerEva1:
	; lower evasion by 1 stage
	ld B, stEva
	jr AttackEffect_StatLowerC1
	
AE_StatLowerAtk2:
	; lower attack by 2 stages
	ld B, stAtk
	jr AttackEffect_StatLowerC2
	
AE_StatLowerDef2:
	; lower defense by 2 stages
	ld B, stDef
	jr AttackEffect_StatLowerC2
	
AE_StatLowerSpDef2:
	; lower sp defense by 2 stages
	ld B, stSDef
	jr AttackEffect_StatLowerC2
	
AE_StatLowerSpd2:
	; lower speed by 2 stages
	ld B, stSpd
	jr AttackEffect_StatLowerC2
	
AE_StatLowerSpAtk2:
	; lower user sp attack by 2 stages
	ld B, stSAtk
	jr AttackEffect_StatLowerC2
	
AE_StatLowerAtkDef:
	; lower user atk and def by 1 stage (qUserAtkDefLower)
	ld B, stAtk
	push HL
	call AttackEffect_StatLowerC1
	pop HL
	ld B, stDef
	;jr AttackEffect_StatLowerC1
	
AttackEffect_StatLowerC1:
	ld C, 1
	jr AttackEffect_StatLowerC
	
AttackEffect_StatLowerC2:
	ld C, 2
	;jr AttackEffect_StatLowerC
	
AttackEffect_StatLowerC:			; lower [stat] by 1 stage, HL = pt to atk boost, B = [stat], C = neg boost (1 | 2)
	ld D, 0
	ld E, B
	dec E
	add HL, DE				; pointer to [stat] = HL + B - 1
	ld A, (HL)				; A = [stat] boost
	cp 7					; if 0 <= A < 7, then can lower stat
	jr c, AttackEffect_StatLowerCOK
	cp -6 + 1				; if A <= -6, then stat at min
	jr c, AttackEffect_StatRaiseMin
AttackEffect_StatLowerCOK:
	sub C					; else, A -= C (1 | 2)
	cp -7
	jr nz, AttackEffect_StatLowerContinue
	ld A, -6				; if A == -7, set A to -6
AttackEffect_StatLowerContinue:
	ld (HL), A
	ld HL, TextStat			; display 'stat fell' msg
	ld (HL), B
	ld A, C
	cp 2
	jr z, AttackEffect_StatLowerCHarsh
	rTextboxRet(tPKMNsStatFell)
	
AttackEffect_StatLowerCHarsh:
	rTextboxRet(tPKMNsStatHarshlyFell)
	
AttackEffect_StatRaiseMin:
	ld HL, TextStat			; display 'stat won't go lower' msg
	ld (HL), B
	rTextboxRet(tPKMNStatWontGoLower)
	
AE_StatLowerAtk1:
AttackEffect_StatLowerAtk:
	; lower attack by 1 stage
	ld B, stAtk
	jr AttackEffect_StatLowerC1
.endrelocate
	
; ---------------------------------------------------------------
; Status inducing move effects
; ---------------------------------------------------------------

AE_Status1:
.relocate SwapZone
AE_StatusBurn:				; ~220 bytes to BadPoison
	; burns target
	; lose 1/8 HP/turn, phys dmg x1/2, fire PKMN immune
	ld A, (HL)
	or A
	jr z, AttackEffect_StatusDoBurn	; burn if no statuses
	cp stBRN
	jp nz, AttackEffect_ButItFailed
	rTextboxRet(tPKMNAlreayHasABurn)	; disp msg if already burned
	
AttackEffect_StatusDoBurn:			; HL = pt to target status
	push HL
	ld BC, PKMN_Status
	or A
	sbc HL, BC				; HL = pt to target data
	ld E, eFire
	call PKMNIsType			; check if pkmn is fire type, preserves HL
	ex DE, HL				; DE = pt to target data
	pop HL
	ret z					; return if target is fire pkmn
	
	ld B, aWaterVeil
	ld C, stBurns
	call AttackEffect_StatusChkAbil	; check ability, saves HL, DE
	jr nz, AttackEffect_StatusBurnNotWaterVeil; fail if target has water veil
	
	rTextboxRet(tPKMNAbilPrevStat)	; ret w/ 'prevent' message
	
AttackEffect_StatusBurnNotWaterVeil:
	push DE
	ld (HL), stBRN
	rTextbox(tPKMNWasBurned)
	ld A, stBRN
	push HL
	RCall( cDispStatus )
	pop HL
	pop DE
	
	push HL
	ld A, stBurn
	ld H, qBurn
	call AttackEffect_StatusSynchronize
	pop HL
	
	ld DE, tPKMNRawstBerry
	ld B, iRawstBerry
	jp AttackEffect_PKMNBerryCure
	
AE_StatusFreeze:
	; freezes target
	; 20% defrost / turn, ;fire atks defrost;, ;disables flash fire;, ice PKMN immune
	ld A, (HL)
	or A
	ret nz
	
AttackEffect_StatusFreeze:
	push HL
	ld HL, (TargetPKMN)
	ld E, eIce
	call PKMNIsType
	pop HL
	ret z					; return if target is ice pkmn
	
	ld (HL), stFRZ
	rTextbox(tPKMNWasFrozenSolid)
	ld A, stFRZ
	RCall( cDispStatus )
	ret
	
AE_StatusParalyze:
	; paralyzes target
	; paralyzes 25% of time, 25% speed
	ld A, (HL)
	or A
	jr z, AttackEffect_StatusDoParalyze
	cp stPAR
	jp nz, AttackEffect_ButItFailed
	rTextboxRet(tPKMNIsAlreadyParalyzed)
	
AttackEffect_StatusDoParalyze:		; HL = pt to status	
	ld (HL), stPAR
	rTextbox(tPKMNIsParalyzed)
	ld A, stPAR
	push HL
	RCall( cDispStatus )
	pop HL
	
	push HL
	ld A, stParalyz
	ld DE, (TargetPKMN)
	ld H, qParalyze
	call AttackEffect_StatusSynchronize
	pop HL
	
	ld DE, tPKMNCheriBerry
	ld B, iCheriBerry
	jp AttackEffect_PKMNBerryCure
	
AE_StatusPoison:
	; poisons target
	; lose 1/8 HP/turn, poison/steel type immune
	ld A, (HL)
	or A
	jr z, AttackEffect_StatusDoPoison
	cp stPSN
	ret nz ;jp nz, AttackEffect_ButItFailed
	rTextboxRet(tPKMNIsAlreadyPoisoned)
	
AttackEffect_StatusDoPoison:		; HL = pt to target status
	push HL				; save HL
	ld BC, PKMN_Status
	or A
	sbc HL, BC				; HL = pt to target data
	ld E, ePoison
	call PKMNIsType
	pop DE
	ret z
	push DE
	ld E, eSteel
	call PKMNIsType
	ex DE, HL				; DE = pt to target data
	pop HL
	ret z					; return if target is poison/steel pkmn
	
	call AttackEffect_StatusPoisonChkImmunity
	jr nz, AttackEffect_StatusPoisonNotImmunity; fail if target has immunity
	
	rTextboxRet(tPKMNAbilPrevStat)	; ret w/ 'prevent' message
	; pkmn, in case of poison point -> immunity
	
AttackEffect_StatusPoisonNotImmunity:
	push HL
	push DE
	ld (HL), stPSN
	rTextbox(tPKMNWasPoisoned)
	ld A, stPSN
	RCall( cDispStatus )
	pop DE
	pop HL
	jp AttackEffect_StatusPoisonSync
.endrelocate
	
AE_Status2:
.relocate SwapZone
AE_StatusBadPoison:				; ~201 bytes
	; badly poisons target
	; lose 1/16 HP/turn, dmg inc by 1/16, poison/steel type immune
	ld A, (HL)
	or A
	jr z, AttackEffect_StatusBadPoison	; inflict bad poison if no status
	cp stPSN				; else if poisoned
	jr z, AttackEffect_StatusAlreadyPoisoned
	cp stBADPSN				; or badly poisoned
	jr nz, AttackEffect_StatusButItFailed;ret nz
	
AttackEffect_StatusAlreadyPoisoned:	; then disp 'already poisoned' msg
	rTextboxRet(tPKMNIsAlreadyPoisoned)
	
AttackEffect_StatusBadPoison:		; HL = pt to target status
	push HL
	ld DE, PKMN_Status
	or A
	sbc HL, DE				; HL = pt to target pkmn data
	ld E, ePoison
	call PKMNIsType			; preserves HL
	pop DE
	jr z, AttackEffect_StatusButItFailed
	push DE
	ld E, eSteel
	call PKMNIsType			; preserves HL
	jr z, AttackEffect_StatusButItFailed; return if target is poison/steel pkmn
	ex DE, HL				; DE = pt to target data
	pop HL
	
	call AttackEffect_StatusPoisonChkImmunity
	jr nz, AttackEffect_StatusBadPoisonNotImmunity; fail if target has immunity
	
	rTextboxRet(tTrgtAbilPrevStat)	; ret w/ 'prevent' message
	
AttackEffect_StatusButItFailed:
	jp AttackEffect_ButItFailed 
	
AttackEffect_StatusBadPoisonNotImmunity:
	push HL
	push DE
	ld (HL), stBADPSN
	ld BC, PKMN_BadPSN - PKMN_Status
	add HL, BC
	ld (HL), 1				; set bad psn counter to 1 (1/16 HP/turn)
	rTextbox(tPKMNIsBadlyPoisoned)
	ld A, stBADPSN
	RCall( cDispStatus )
	pop DE
	pop HL
	
	jp AttackEffect_StatusPoisonSync
	
AE_StatusSleep:
	; puts target to sleep
	; sleep for 1-6 turns
	ld A, (HL)
	or A					; c flag reset
	jr z, AttackEffect_StatusSleep
	cp stSLP
	jr nz, AttackEffect_StatusButItFailed;ret nz
	rTextboxRet(tPKMNIsAlreadyAsleep)
	
AttackEffect_StatusSleep:			; HL = pt to pkmn status
	push HL
	sbc HL, BC				; subtract BC = PKMN_Status from HL
	ex DE, HL				; DE = pt to pkmn data
	pop HL
	
	ld B, aInsomnia
	ld C, stSleep
	call AttackEffect_StatusChkAbil
	jr nz, AttackEffect_StatusSleepNotInsomnia; fail if target has insomnia
	
	; TODO: wake up when gain insomnia ability
	rTextboxRet(tTrgtAbilPrevStat)	; ret w/ 'prevent' message
	
AttackEffect_StatusSleepNotInsomnia:
	push HL
	ld (HL), stSLP
	ld BC, PKMN_SLPCount - PKMN_Status
	add HL, BC
	push HL
	ld B, 6
	call iRandom				; 0 <= A < 6
	inc A					; 1 <= A <= 6
	inc A					; 2 <= A <= 7
	pop HL
	ld (HL), A
	rTextbox(tPKMNFellAsleep)
	ld A, stSLP
	RCall( cDispStatus )
	pop HL
	jp AttackEffect_StatusSleepChesto	; check if holding chesto berry, cures sleep
	
AE_StatusTri:
	; burns/paralyzes/freezes target
	; 1/3 chance of BRN/PAR/FRZ
	ld A, (HL)
	or A
	ret nz
	
	ld B, 100
	call iRandom				; 0 <= A < 100
	cp 33
	jr c, AE_StatusTriBurn
	cp 66
	jr c, AE_StatusTriFreeze
	
AE_StatusTriParalyze:
	ld A, qParalyze
	jr AE_StatusCall			; AE_StatusParalyze
	
AE_StatusTriBurn:
	ld A, qBurn
	jr AE_StatusCall			; AE_StatusBurn
	
AE_StatusTriFreeze:
	ld A, qFreeze
	
AE_StatusCall:
	jp AttackEffect_StatusCall		; AE_StatusFreeze
	
AE_StatusYawn:
	; induces sleep (qYawn)
	; trgt falls asleep next turn, unless status / ;safeguard;
	; ;if trgt falls asleep, then wakes up, yawn takes effect;
	ld A, (HL)
	or A
	ret nz
	
	call GetPKMNAbility
	cp aInsomnia
	jp z, AttackEffect_StatusButItFailed
	
	ld BC, PKMN_Yawn - PKMN_Status
	add HL, BC
	ld A, (HL)
	or A
	jp nz, AttackEffect_StatusButItFailed
	ld (HL), 2
	rTextboxRet(tPKMNMadeDrowsy)
.endrelocate
	
; ---------------------------------------------------------------
; Volatile status move effects
; ---------------------------------------------------------------

AE_Vol1:
.relocate SwapZone
AE_VolDisable:				; ~176 bytes to confuse
	; disables target
	; lasts 4-7 turns, 1 move, ;fails if move has no PP;, fails if user goes first
	; sleep talk can use disabled move
	ld BC, PKMN_LastMove
	add HL, BC
	ld A, (HL)
	ld B, A
	cp $FF					; if user goes first in first turn, then fail
	jp z, AttackEffect_ButItFailed ; jr z, AttackEffect_VolStatusDisableFail
	inc HL					; assumes PKMN_DisabledMove - PKMNLastMove = 1
	ld A, (HL)
	cp $FF					; if already disabled move, then fail
	jp nz, AttackEffect_ButItFailed ; jr nz, AttackEffect_VolStatusDisableFail
	ld (HL), B				; disabled move = last move
	inc HL
	push HL
	ld B, 4
	call iRandom				; 0 <= A < 4
	add A, 4				; 4 <= A < 8
	pop HL
	ld (HL), A				; disable lasts 4-7 turns
	rTextboxRet(tPKMNsMoveWasDisabled)
	
AE_VolSeed:
	; plants seeds on target
	; steals 1/8 HP per turn, doesn't affect GRASS pkmn, dmgs if liquid ooze
	push HL
	ld E, eGrass
	call PKMNIsType
	pop HL
	jr z, AttackEffect_VolStatusSeedFail	; fail if target is grass pkmn
	call GetPKMNSub
	jr nz, AttackEffect_VolStatusSeedFail	; fail if target has substitute
	ld BC, PKMN_Leech
	add HL, BC
	ld A, (HL)
	or A
	jr nz, AttackEffect_VolStatusSeedFail	; fail if already seeded
	ld (HL), 1					; leech seed status = true
	rTextboxRet(tPKMNWasSeeded)
	
AttackEffect_VolStatusSeedFail:
	rTextboxRet(tPKMNEvadedTheAttack)
	
AE_VolEncore:
	; encore
	; lasts 2-6 turns, can be combined w/ torment, ;ends when no PP;
	ld BC, PKMN_LastMove
	add HL, BC
	ld A, (HL)
	ld B, A
	cp $FF					; if user goes first, then fail
	jp z, AttackEffect_ButItFailed
	ld BC, PKMN_EncoreMove - PKMN_LastMove
	add HL, BC
	ld A, (HL)
	cp $FF					; if move already encored, then fail
	jp nz, AttackEffect_ButItFailed
	ld (HL), B				; encore move = last move
	inc HL
	push HL
	ld B, 5
	call iRandom				; 0 <= A < 5
	add A, 2				; 2 <= A < 7
	pop HL
	ld (HL), A				; disable lasts 4-7 turns
	rTextboxRet(tPKMNGotAnEncore)
	
AE_VolInfatuation:
	; infatuation
	; affects pkmn of opposite gender, attack 50% time, oblivious immune, remove on switch
	call GetPKMNAbility			; A = pkmn ability
	cp aOblivious
	jr nz, AttackEffect_VolStatusInfatuationNotOblivious
	
	ld (TextAbil), A			; set (TextAbil) to aOblivious
	ld A, stRomance
	ld (TextStat), A			; set (TextStat) to stRomance
	rTextboxRet(tPKMNAbilPrevStat)	; disp 'prev' message, return
	
AttackEffect_VolStatusInfatuationNotOblivious:
	ld BC, PKMN_Infatuation
	add HL, BC
	ld A, (HL)
	or A					; fail if already infatuated
	jp nz, AttackEffect_ButItFailed ; jr nz, AttackEffect_VolStatusInfatuateFail
	
	ex DE, HL				; preserve HL in DE
	ld HL, (TargetPKMN)
	RCall( cChkGender )			; A = opp gender
	or A					; fail if genderless
	jp z, AttackEffect_ButItFailed
	dec A					; B = 0 if male, 1 if female
	ld B, A
	push BC
	ld HL, (AtkPKMN)
	RCall( cChkGender )
	pop BC
	or A					; fail if genderles
	jp z, AttackEffect_ButItFailed
	dec A					; A = 0 if male, 1 if female
	xor B					; A xor B == 1, iff A and B are opposite
	jp z, AttackEffect_ButItFailed
	ex DE, HL
	ld (HL), 1				; infatuation = true
	rTextboxRet(tPKMNFellInLove)
.endrelocate
	
AE_Vol2:
.relocate SwapZone
AE_VolConfuse:				; HL = pt to detailed data, ~228 bytes
	; confusion
	; 50% chance to hurt self w/ 40-power typeless attack, lasts 1-4 atk turns, own tempo immune
	call IsSafeguard			; Both preserve HL
	jr c, AttackEffect_VolButItFailed	; //TODO: silence for status side effects
	
	call GetPKMNSub
	jr nz, AttackEffect_VolButItFailed	; fail if target has substitute
	
	call GetPKMNAbility			; A = pkmn ability
	cp aOwnTempo
	jr nz, AttackEffect_VolStatusNotTempo
	
	ld (TextAbil), A			; set (TextAbil) to aOwnTempo
	ld A, stConfusion
	ld (TextStat), A			; set (TextStat) to stConfusion
	rTextboxRet(tPKMNAbilPrevStat)	; disp 'prev' message, return
	
AttackEffect_VolStatusNotTempo:
	ld BC, PKMN_Confusion
	add HL, BC
	ld A, (HL)
	or A
	jr nz, AttackEffect_VolStatusConfused
	push HL
	ld B, 4
	call iRandom				; 0 <= A < 4
	inc A					; 1 <= A < 5
	inc A					; 2 <= A < 6
	pop HL
	ld (HL), A				; 2-5 => 1-4 turns
	rTextbox(tPKMNBecameConfused)
	
	ld BC, PKMN_Confusion - PKMN_Item
	or A					; reset c flag
	sbc HL, BC				; HL = pt to pkmn item
	ld A, (HL)				; A = item ID
	ld (TextItem), A			; set (TextItem) to A
	
	ld DE, tPKMNPersimBerry
	cp iPersimBerry			; if holding persim berry, cure
	jr z, AttackEffect_VolStatusConfCure
	ld DE, tPKMNLumBerry
	cp iLumBerry				; else if holding lum berry, cure
	jr z, AttackEffect_VolStatusConfCure
	ret					; else, return
	
AttackEffect_VolStatusConfCure:
	ld (HL), iNoItem			; item is depleted
	ld BC, PKMN_Confusion - PKMN_Item
	add HL, BC
	xor A					; A = 0
	ld (HL), A				; cure confusion
	ex DE, HL				; HL = pt to msg
	res SilenceFlag, (IY + Asm_Flag2)	; don't silence msg
	RCall( cTextbox )			; then disp msg
	ret
	
AttackEffect_VolStatusConfused:
	rTextboxRet(tPKMNIsAlreadyConfused)
	
AttackEffect_VolButItFailed:
	jp AttackEffect_ButItFailed
	
AE_VolFollow:
	; follow me
	; no effect b/c not double battle
	rTextboxRet(tPKMNBecameCenter)
	
AE_VolPrevEscape:
	; prevent escape
	; no escaping, except w/ baton pass (new pkmn can't escape) and roar
	call GetPKMNSub
	jr nz, AttackEffect_VolButItFailed	; fail if target has substitute
	
	ld HL, (AtkPKMN)	
	ld BC, PKMN_PrevEscape
	add HL, BC
	ld A, (HL)
	or A
	jr nz, AttackEffect_VolButItFailed	; fail if prev escape move already used
	
	ld (HL), 1
	rTextboxRet(tPKMNCantEscapeNow)
	
AE_VolPerish:
	; perish song
	; all pkmn faint in 3 turns, ;no effect on soundproof;, ;carried by baton pass;, removed on switch
	ld HL, (TargetPKMN)
	ld BC, PKMN_PerishCount
	add HL, BC
	ld (HL), 4

AttackEffect_VolStatusPerishAtk:
	ld HL, (AtkPKMN)
	ld BC, PKMN_PerishCount
	add HL, BC
	ld (HL), 4
	rTextboxRet(tAffectedPKMNFaint)
	
AE_VolProtect:
	; protect
	; protects user for 1 turn (except curse, future sight, perish song, psych up), 50% less chance each consecutive use
	; fails if user goes last
	call PKMNProtect			; determines if success, ret c if fail, HL = pt to counter if success
	ret c					; ret if fail
	dec HL					; Counter - Protect = 1
	ld (HL), 1				; set protect to true
	ret
	
AE_VolSwagger:
	; swagger
	; confuses target and boosts attack 2 stages
	push HL				; HL = PlayerPKMN | OppPKMN
	ld A, qConfuse
	call AttackEffect_Call		; AE_VolConfuse
	pop HL
	ld BC, PKMN_AtkBoost
	add HL, BC
	ld A, qAtkRaise2
	jp AttackEffect_Call			; AE_StatRaiseAtk2
	
AE_VolTorment:
	; torment (qTorment)
	; ;can't use same move twice;, ;struggles if choice band, disable, encore, taunt, no PP;
	ld BC, PKMN_Torment
	add HL, BC
	ld A, (TurnNum)
	ld (HL), A				; if 1st move in turn, then affects move in same turn, if 2nd move in turn, affects move in next turn
	rTextboxRet(tPKMNWasSubjectedToTorment)
	
AE_VolTrap:
	; trap (qTrap)
	; traps target for 2-5 turns, does 1/16 damage at EOT, prevents switching
	ld HL, (TargetPKMN)
	call GetPKMNTrap			; HL = pt to trap status, A = trap status
	ret nz					; return and do nothing if already trapped
	
	push HL
	ld B, 4
	call iRandom				; 0 <= A < 4
	inc A
	inc A
	inc A					; 3 <= A < 7, TODO: 37.5% 2 / 3 turns, 12.5% 4 / 5 turns ?
	pop HL
	ld (HL), A				; set trap status, turn number = A - 1
	ld A, (AtkMove)
	cp mSandTomb				; if move = sand tomb
	jr z, AttackEffect_MiscSandTomb	; then disp sand tomb message
	
AttackEffect_MiscWrap:			; else, disp wrap message
	rTextboxRet(tPKMNWasWrapped)
	
AttackEffect_MiscSandTomb:
	rTextboxRet(tPKMNWasTrappedBySandTomb)
.endrelocate
	
; ---------------------------------------------------------------
; Field related move effects
; ---------------------------------------------------------------

AE_Field:
.relocate SwapZone
AE_FieldHail:					; ~131 bytes
	; hail
	; 5 turns, 1/16 of non-ice HP/turn, ;solarbeam has 60 power;, ;qWeather is 1/4 HP;
	ld HL, Weather
	ld A, (HL)
	cp wHail
	jp z, AttackEffect_ButItFailed	; fail if already hailing
	ld (HL), wHail
	inc HL					; WeatherCount - Weather = 1
	ld (HL), 5				; lasts 5 turns
	rTextboxRet(tStartedToHail)
	
AE_FieldLightScreen:
	; light screen
	; halves sp damage for 5 turns, ;fades by brick break;, ;ignored in crit;
	ld A, (HL)
	or A
	jp nz, AttackEffect_ButItFailed
	ld (HL), 6				; UserLight = UserField, 6 => 5 turns
	rTextboxRet(tPKMNsLightScreenRaisedSPDEF)
	
AE_FieldRain:
	; rain dance
	; 5 turns, water moves 50% more power, fire moves 50% less power, solarbeam has 60 power, thunder never misses
	; qWeather is 1/4 HP, rain dish + swift swim
	ld HL, Weather
	ld A, (HL)
	cp wRain
	jp z, AttackEffect_ButItFailed	; fail if already raining
	ld (HL), wRain
	inc HL					; WeatherCount - Weather = 1
	ld (HL), 5				; lasts 5 turns
	rTextboxRet(tItStartedToRain)
	
AE_FieldReflect:
	; reflect
	; halves phys damage for 5 turns, ;fades by brick break;, ;ignored in crit;
	inc HL
	ld A, (HL)
	or A
	jp nz, AttackEffect_ButItFailed
	ld (HL), 6				; UserReflect = UserField + 1, 6 => 5 turns
	rTextboxRet(tPKMNsReflectRaisedDEFENSE)
	
AE_FieldSafeguard:
	; safeguard
	; 5 turns, blocks status (except for abilities, items, or self-inflicted)
	inc HL
	inc HL					; Safeguard - Field = 2
	ld A, (HL)
	or A
	jp nz, AttackEffect_ButItFailed
	ld (HL), 6				; UserSafeguard = UserField + 1, 6 => 5 turns
	rTextboxRet(tPKMNIsProtectedBySafeguard)
	
AE_FieldSandstorm:
	; sandstorm
	; 5 turns, non rock/ground/steel pkmn lose 1/16 HP/turn, solarbeam has 60 power, qWeather heals 1/4 HP, Sand Veil
	ld HL, Weather
	ld A, (HL)
	cp wSandstorm
	jp z, AttackEffect_ButItFailed	; fail if already sandstorm
	ld (HL), wSandstorm
	inc HL					; WeatherCount - Weather = 1
	ld (HL), 5				; lasts 5 turns
	rTextboxRet(tSandstormBrewed)
	
AE_FieldSpikes:
	; spikes
	; 3 layers, 1 => 1/8 HP, 2 => 1/6 HP, 3 => 1/4 HP on switch, flying/levitate immune, ;rapid spin removes;
	call FlipPKMNFlag			; pkmn flag set to target pkmn
	call GetLight				; get HL = pt to field of target pkmn
	inc HL
	inc HL
	inc HL
	ld A, (HL)				; UserSpikes - UserField = 3
	cp 3
	jp z, AttackEffect_ButItFailed	; fail if at max, 3
	inc A
	ld (HL), A
	rTextboxRet(tPKMNTeamHadSpikes)
	
AE_FieldSunny:
	; Sunny Day (qSunnyDay)
	; lasts 5 turns, fire moves have 50% more power, water moves have 50% less power, solarbeam is 1 turn
	;thunder has 50% accuracy, chlorophyll, qWeather heals 2/3 HP
	ld HL, Weather
	ld A, (HL)
	cp wSun
	jp z, AttackEffect_ButItFailed	; fail if already sunny
	ld (HL), wSun
	inc HL					; WeatherCount - Weather = 1
	ld (HL), 5				; lasts 5 turns
	rTextboxRet(tSunlightGotBright)
.endrelocate
	
; ---------------------------------------------------------------
; Stall move effects
; ---------------------------------------------------------------

AE_Stall:
.relocate SwapZone
AE_StallBounce:				; ~91 bytes
	; Bounce (qBounce)
	; invulnerable on 1st turn, may paralyze (30%), hit by sky uppercut / thunder (2x ?)
	ld A, (HL)
	or A
	jr z, AttackEffect_StallBounce1
	
	xor A
	ld (HL), A
	ld B, 100				; if 2nd turn
	call iRandom
	cp 30					; then 30% chance paralysis
	ret nc
	
	ld HL, (TargetPKMN)
	ld BC, PKMN_Status
	add HL, BC
	ld A, qParalyze
	jp AttackEffect_StatusCall		; AE_StatusParalyze
	
AttackEffect_StallBounce1:			; if 1st turn
	ld (HL), evBounce
	rTextbox(tPKMNSprangUp)
	jr AttackEffectStallDisappear
	
AE_StallDig:
	; Dig (qDig)
	; invulnerable on 1st turn, hit by earthquake ;(2x); / fissure, no weather dmg
	ld A, (HL)
	or A
	jr z, AttackEffect_StallDig1
	
AttackEffect_StallEnd:
	xor A
	ld (HL), A
	
	bit PKMNFlag, (IY + Asm_Flag1)
	jr nz, AttackEffectStallAppearOpp
AttackEffectStallAppearPlayer:		; opp is target, player sprite appears
	jp BattleGraphicsPlayerSprite
	
AttackEffectStallAppearOpp:			; player is target, opp sprite appears
	jp BattleGraphicsOppSprite
	
AttackEffect_StallDig1:
	ld (HL), evDig
	rTextbox(tPKMNDugAHole)
	jr AttackEffectStallDisappear
	
AE_StallDive:
	; Dive (qDive)
	; invulnerable on 1st turn, hit by surf ;(2x);, no weather dmg
	ld A, (HL)
	or A
	jr nz, AttackEffect_StallEnd
	
AttackEffect_StallDive1:
	ld (HL), evDive
	rTextbox(tPKMNHidUnderwater)
	jr AttackEffectStallDisappear
	
AE_StallFly:
	; Fly (qFly)
	; invulnerable on 1st turn, hit by sky uppercut / thunder 
	ld A, (HL)
	or A
	jr nz, AttackEffect_StallEnd	
	
AttackEffectStallFly1:
	ld (HL), evFly
	rTextbox(tPKMNFlewUpHigh)
	; jr AttackEffectStallDisappear
	
AttackEffectStallDisappear:			; subroutine - erases pkmn sprite
	bit PKMNFlag, (IY + Asm_Flag1)
	jr nz, AttackEffectStallDisappearOpp
AttackEffectStallDisappearPlayer:		; opp is target, player sprite disappears
	;jp BattleGraphicsPlayerClear
	
AttackEffectStallDisappearOpp:		; player is target, opp sprite disappears
	;jp BattleGraphicsOppClear
	ret  ;//TODO: draw white rect or fill buff w/ wfill to clear sprite
.endrelocate
	
; ---------------------------------------------------------------
; Misc move effects
; ---------------------------------------------------------------

AE_Misc1:
.relocate SwapZone
AE_MiscNeverMiss:				; ~232 bytes to charge
	; Never miss (qNeverMiss)
	; attack never misses, factored into CalcMiss
	ret
	
AE_MiscHighCrit:
	; High Crit ratio (qHighCrit)
	; high critical hit ratio, factored into CalcCrit
	ret
	
AE_MiscFlinch:
	; Flinch (qFlinch)
	; attack causes target to flinch
	ex DE, HL
	jp SetFlinch
	
AE_MiscHitTwice:
	; Hit twice (qHitTwice)
	; attack hits twice, factored into Attack routine
	ld A, (HitRepeat)
	ld B, A
	ld A, 2				; if counter at 0, then hit 2 time(s)
	sub B					; else if counter at 1, then hit 1 time(s)
	ld (TextNum), A
	
	rTextboxRet(tHitXTimes)
	
AE_MiscRecoil:
	; Recoil (qRecoil)
	; attacker hit with 1/3 recoil
	call GetPKMNAbility			; A = pkmn ability
	cp aRockHead				; if ability is rock head
	ret z					; then ret, no recoil
	
	ld HL, (Damage)
	ld D, 3
	RCall( cDivHL )
	ld H, 0				; HL = damage / 3
	ld A, L
	or A
	ret z					; ret if no recoil damage
	
	rTextbox(tPKMNIsHitWithRecoil)
	
	RCall( cDoDamage )
	ret nc					; if user faints
	ld A, 1
	ld (PKMNFNT), A			; set PKMNFNT
	ret					; ret w/ c set
	
AE_MiscPriority:
	; Priority (qPriority)
	; attacker hits first, factored into CalcPriority
	ret
	
AE_MiscOHKO:
	; OHKO (qOHKO)
	; hits target w/ OHKO, subtracts max HP, factored into CalcDamage
	rTextboxRet(tItsAOHKO)		; disp OHKO message
	
AE_MiscReverse:
	; Reversal / Flail (qReverse)
	; more power when less HP, factored into CalcPow
	ret
	
AE_MiscFuture:
	; Future Sight (qFuture)
	; hits after end of 3 turns, typeless, no crits, no stack
	bit PKMNFlag, (IY + Asm_Flag1)
	jr z, AttackEffect_FutureUser
AttackEffect_FutureOpp:			; user turn, opp is hit
	ld HL, OppFutureTurn
	jr AttackEffect_FutureCont
	
AttackEffect_FutureUser:			; opp turn, user is hit
	ld HL, UserFutureTurn
	
AttackEffect_FutureCont:			; HL = pointer to future turn count
	jp AttackEffect_FutureDo
	
AE_MiscDrain:
	; Drain (qDrain)
	; attack drains 1/2 target's HP
	ex DE, HL				; HL = pt to target data
	call GetPKMNAbility			; A = target ability, HL, DE preserved
	cp aLiquidOoze
	jr nz, AttackEffect_MiscDrainNotOoze
	
	rTextbox(tSuckedUpLiquidOoze)	; disp liquid ooze message
	call GetHalfDamage			; HL = damage / 2
	RCall( cDoDamage )			; damage user by half of damage done
	ret
	
AttackEffect_MiscDrainNotOoze:
	rTextbox(tPKMNHadItsEnergyDrained)	; always drains, even if at full HP
	jp HealHalfDamage			; heals user by half of damage done
	
AE_MiscBlazeKick:
	; Blaze kick (qBlazeKick)
	; high critical hit ratio (factored into CalcCrit), 10% chance of burn
	jp AttackEffect_MiscBlaze
	
AE_MiscBlizzard:
	; Blizzard (qBlizzard)
	; may freeze (10%), hits through protect/detect in hail (30%)
	jp AttackEffect_MiscBlizzard

AE_MiscEarthquake:
	; Earthquake (qEarthquake)
	; hits underground foes w/ 2x damage
	ret
	
AE_MiscFakeOut:
	; Fake Out (qFakeOut)
	; priority (factored into CalcPriority), flinches target, only used on first turn (sleep/frozen doesn't reset)
	call GetLastMove			; returns nz, if no last move
	jp nz, AttackEffect_ButItFailed	; if last move is not 'none', then attack fails
	ex DE, HL
	call SetFlinch
	ret
	
AE_MiscFocusPunch:
	; Focus Punch (qFocusPunch)
	; user flinches if hit (not w/ pain split)
	ret
	
AE_MiscHiJumpKick:
	; High Jump Kick (qJumpKick)
	; attack hurts user min(1/2 damage, 1/2 target max HP) if it misses / protect / detect / ghost
	ld A, (HitMiss)			; A = whether attack missed
	or A
	jr z, AE_MiscHiJumpKickAct		; activated if attack missed
	
	ld A, (DamageEff)
	cp 2
	jr z, AE_MiscHiJumpKickAct		; activated if move has no effect against target (ghost)
	
	ld A, (TrgtProtect)
	or A
	ret z					; activated if target used protect
	
AE_MiscHiJumpKickAct:
	push DE
	ld A, $11				; A = $11, or 17
	ld (TypeEff), A			; set (TypeEff) to $11 signal, attack is always effective
	di
	RCall( cCalcDamage )			; HL = calculated damage
	ei
	pop DE
	
	ex DE, HL				; HL = pt to target, DE = calculated damage
	call GetPKMNMaxHP			; HL = max HP of target pkmn, DE preserved
	call CpHLDE				; compare HL and DE
	jr c, AE_MiscHiJumpKickLT		; if HL >= DE
	ex DE, HL				; then set HL <- DE
	
AE_MiscHiJumpKickLT:
	srl H
	rr L					; HL /= 2
	rTextbox(tPKMNKeptGoingAndCrashed)
	RCall( cDoDamage )			; damage user by min(1/2 damage, 1/2 target max HP)
	ret nc
	ld A, 1
	ld (PKMNFNT), A			; if ret c, then set (PKMNFNT) to 1
	ret
.endrelocate
	
AE_Misc2:
.relocate SwapZone
AE_MiscCharge:				; ~225 bytes to baton pass
	; Charge (qCharge)
	; user cannot attack or switch afterwards
	jp SetCharge
	
AE_MiscHeal:
	; Heal (qHeal)
	call ChkFullHP			; check if full HP, preserves HL, returns flag
	jp nc, AttackEffect_ButItFailed	; fail if current HP >= max HP
	
	call HealHalfHP
	rTextboxRet(tPKMNRegainedHealth)
	
AE_MiscWeatherHeal:
	; Weather heal (qWeatherHeal)
	; heals 2/3 HP in sun, 1/4 HP in other weather, 1/2 else (moonlight, synthesis)
	call ChkFullHP			; check if full HP, preserves HL, returns flag
	jp nc, AttackEffect_ButItFailed	; fails if current HP >= max HP
	
	ld A, (Weather)			; A = weather status
	cp wSun
	jr z, AttackEffect_WeatherHealSun	; heal 2/3 HP in sun
	or A					; cp wNone
	jr z, AttackEffect_WeatherHealNone	; heal 1/2 HP in no weather
	call GetFourthMaxHP			; else heal 1/4 HP (hail, sandstorm, rain)
	jr AttackEffect_WeatherHealCont
	
AttackEffect_WeatherHealNone:
	call GetHalfMaxHP
	jr AttackEffect_WeatherHealCont
	
AttackEffect_WeatherHealSun:
	call GetTwoThirdsMaxHP
	
AttackEffect_WeatherHealCont:
	RCall( cDoHeal )
	rTextboxRet(tPKMNRegainedHealth)	; disp message at end
	
AE_MiscWish:
	; Wish (qWish)
	; heals 1/2 HP at end of 2nd turn
	bit PKMNFlag, (IY + Asm_Flag1)
	jr nz, AttackEffect_MiscWishPlayer
	ld HL, OppWish
	jr AttackEffect_MiscWishCont
	
AttackEffect_MiscWishPlayer:
	ld HL, UserWish
	
AttackEffect_MiscWishCont:
	ld A, (HL)				; A = wish status
	or A
	jp nz, AttackEffect_ButItFailed	; fail if wish already set
	ld (HL), 2				; else, trigger wish at end of 2nd turn
	rTextboxRet(tPKMNMadeAWish)		; disp msg
	
AE_MiscCure:
	; Cure (qCure)
	; cures user of status problems (burn, poison, paralysis)
	ld BC, PKMN_Status
	add HL, BC
	ld A, (HL)
	cp stBRN
	jr z, AttackEffect_MiscCure
	cp stPSN
	jr z, AttackEffect_MiscCure
	cp stPAR
	jr z, AttackEffect_MiscCure
	jp AttackEffect_ButItFailed		; only works on burn, poison, paralysis
	
AttackEffect_MiscCure:			; HL = pt to pkmn status
	xor A
	ld (HL), A				; (HL) = stNone
	RCall( cDispStatus )			; A = 0 to clear status
	rTextboxRet(tPKMNCuredProblem)
	
AE_MiscCureAll:
	; Cure all (qCureAll) - Aromatherapy
	; cures user and all party pkmn of status effects
	xor A					; A = 0, stNone
	ld BC, PKMN_Status
	add HL, BC				; HL = pt to user pkmn status
	ld (HL), A				; set user pkmn status to none
	ld HL, TeamPKMN1 + PKMN_Status
	bit PKMNFlag, (IY + Asm_Flag1)	; if user pkmn is player
	jr nz, AE_MiscCureAllPlayer		; then skip, HL set to player status
	
AE_MiscCureAllOpp:
	ld HL, OppPKMN1 + PKMN_Status	; else, set HL to pt to opp status
	
AE_MiscCureAllPlayer:
	ld DE, BYTES_PER_TPKMN		; DE = size of team pkmn, in bytes
	ld BC, 3 * 256			; B = number of pkmn on team, C = 0
	ld A, stFNT
AE_MiscCureAllLp:
	cp (HL)
	jr z, AE_MiscCureAllSkip		; don't cure if FNT
	ld (HL), C				; else, set status to stNone
	
AE_MiscCureAllSkip:
	add HL, DE				; move HL to next pkmn
	djnz AE_MiscCureAllLp		; loop until B = 0
	
	xor A
	RCall( cDispStatus )			; clear status of user pkmn
	rTextboxRet(tSoothingAroma)		; disp status and ret
	
AE_MiscLevelDmg:
	; Level damage (qLevelDmg)
	; damage = level of user, factored into CalcDamage
	ret
	
AE_MiscDuration:
	; Duration (qDuration)
	; ;lasts 2-3 turns, user is confused at end;
	ld BC, PKMN_Duration
	add HL, BC				; HL = pt to duration status of pkmn
	ld A, (HL)				; A = duration status of pkmn
	or A					; if A != 0, clear c flag
	jr nz, AE_MiscDurationDec		; then dec duration counter
	
	ld A, R				; A = rand var
	and %00000001				; bitmask to get bit 1, A = 0 | 1
	add A, 2				; A = 2 | 3
	ld (HL), A
	
AE_MiscDurationDec:
	dec (HL)				; dec duration counter by 1, c flag preserved
	ret nz					; ret if counter != 0
	
	rTextbox(tPKMNBecameConfusedOutOfFatigue)
	
	sbc HL, BC				; HL = pt to user pkmn data
	ld A, qConfuse
	jp AttackEffect_CallSilent		; call AE_VolConfusion
	
AE_MiscSkyAttack:
	; Sky attack (qSkyAttack)
	; charges for 1 turn, 30% flinch
	ld B, 100
	push DE				; save DE, pt to target pkmn
	call iRandom				; 0 <= A < 100
	pop HL					; restore HL, pt to target pkmn
	cp 30					; if A >= 30
	ret nc					; then ret, flinch not activated
	jp SetFlinch				; else, set flinch status of target
	
AE_MiscSkyUppercut:
	; Sky uppercut (qSkyUppercut)
	; hits pmkn in bounce and fly, factored into CalcMiss
	ret
	
AE_MiscSmellingsalt:
	; Smellingsalt (qSmellingsalt)
	; cures target and deals 2x damage if target paralyzed
	ex DE, HL				; HL = pt to target pkmn
	ld BC, PKMN_Status
	add HL, BC
	ld A, (HL)				; A = target pkmn status
	cp stPAR
	ret nz					; ret if not paralyzed
	xor A					; A = 0, stNone
	ld (HL), A				; else, set target status to none
	call FlipPKMNFlag			; pkmn flag set to target
	RCall( cDispStatus )			; clear opp status
	rTextboxRet(tPKMNCuredProblem)	; disp msg and ret
	; TODO: disp 'target was cured of its paralysis!'
	
AE_MiscSolarbeam:
	; Solarbeam (qSolarbeam)
	; charges on turn 1, skips charging if sunny, x1/2 power in hail/rain/sandstorm
	ret
	
AE_MiscSurf:
	; Surf (qSurf)
	; hits underwater targets w/ 2x damage
	ret
	
AE_MiscThunder:
	; Thunder (qThunder)
	; may paralyze (30%), 1/2 accurate in sun
	ret
.endrelocate
	
; ---------------------------------------------------------------
; Move specific move effects
; ---------------------------------------------------------------

AE_Spec1:
.relocate SwapZone
AE_SpecBatonPass:				; ~224 bytes to knock off
	; Baton Pass (qBatonPass)
	; switches pkmn, passes stat changes and volatile stats
	; prev escape, confusion, curse, ingrain, leech seed, perish count, substitute
	call ChkParty				; return c if party pkmn all FNT
	jp c, AttackEffect_ButItFailed	; fail if all party pkmn FNT
	bit PKMNFlag, (IY + Asm_Flag1)
	jr nz, AttackEffect_BatonPassPlayer
AttackEffect_BatonPassOpp:
	or A
	jr nz, AttackEffect_BatonPassOppN0	; if opp has both party pkmn not FNT
	ld A, 5				; then switch to 2nd pkmn, by default
AttackEffect_BatonPassOppN0:
	call CopyOpp
	ld DE, OppPKMN
	call AttackEffect_BatonPassSwitch
	jp Turn_SwitchOppNoInit
	
AttackEffect_BatonPassPlayer:
	set SwitchFlag, (IY + Asm_Flag2)	; can switch out from qPrevEscape, using baton pass
	call RParty
	res SwitchFlag, (IY + Asm_Flag2)
	
	ld A, (PartySel)
	cp 3					; if no switch
	jr z, AttackEffect_BatonPassPlayer	; then go back and do again
	
	inc A					; 1 <= A <= 3
	call Battle_Redraw			; saves AF, redraws UI + graphics
	call CopyPlayer			; copies PlayerPKMN to TeamPKMN1, AF not affected
	ld DE, PlayerPKMN
	call AttackEffect_BatonPassSwitch
	jp Turn_SwitchPlayerNoInit
	
AE_SpecBellyDrum:
	; Belly Drum (qBellyDrum)
	; reduces user's HP by half of max, but sets attack boost to +6, fail if HP < 50% or atk at +6
	call ChkHalfHP			; ret c if HP < 1/2 max, preserves HL
	jp c, AttackEffect_ButItFailed	; fail if HP < 1/2 max
	ld BC, PKMN_AtkBoost
	add HL, BC				; HL = pt to atk pkmn atk boost
	ld A, (HL)				; A = atk pkmn atk boost
	ld D, 6
	cp D
	jp z, AttackEffect_ButItFailed	; fail if atk boost = 6
	ld (HL), D				; else, set atk boost to +6
	or A					; reset c flag
	sbc HL, BC				; HL = pt to atk pkmn
	call GetHalfMaxHP			; HL = 1/2 max HP of atk pkmn
	RCall( cDoDamage )			; deduct 1/2 of max HP from atk pkmn
	rTextboxRet(tPKMNCutItsOwnHP)	; disp 'maximize atk' msg, return
	
AE_SpecBrickBreak:
	; Brick Break (qBrickBreak)
	; shatter reflect / light screen, even if miss/not affect, factored into Attack_BrickBreak
	ret
	
AE_SpecCounter:
	; Counter (qCounter)
	; if last attack is physical, then do 2x damage. no affect on ghosts.
	; ;fail if hit sub;, fail if not second attack
	ld C, 1				; set C parameter to 1, for physical
	call DoReflectDmg
	ret nc
	ld A, 1				; if c is set
	ld (PKMNFNT), A			; then set PKMNFNT
	ret
	
AE_SpecCurse:
	; Curse (qCurse)
	; if ghost, ;;then curse;;, else raise atk/def and lower spd
	ld BC, PKMN_AtkBoost
	add HL, BC
	push HL
	ld A, qAtkRaise
	call AttackEffect_Call		; raise atk 1 stage
	pop HL
	push HL
	ld A, qDefRaise
	call AttackEffect_Call		; raise def 1 stage
	pop HL
	ld A, qSpdLower
	jp AttackEffect_Call			; lower spd 1 stage
	
AE_SpecDefCurl:
	; Defense Curl (qDefCurl)
	; boosts defense by 1 stage, doubles power of rollout while user is in battle
	ld BC, PKMN_DefCurl
	add HL, BC
	ld (HL), 1				; set defense curl flag
	
	ld BC, PKMN_DefCurl - PKMN_AtkBoost
	or A
	sbc HL, BC				; HL = pt to pkmn atk boost
	ld B, stDef
	ld C, 1				; set parameters to boost defense by 1 stage
	jp AttackEffect_StatRaiseC		; B = stat, C = stages
	
AE_SpecDestinyBond:
	; Destiny bond (qDestinyBond)
	; if user faints from direct attack after destiny bond is used, then attacker also faints, ;fails against bonemerang;
	ld A, (TurnNum)			; A = 1 if attack is first in turn, 2 if second
	ld BC, PKMN_Destiny
	add HL, BC
	ld (HL), A				; destiny bond works on current turn if A = 1, next turn if A = 2
	rTextboxRet(tPKMNIsTryingToTakeItsFoe)
	; TODO: pkmn defeated by destiny bond faints first
	
AE_SpecDreamEater:
	; Dream Eater (qDreamEater)
	; fails if target is not asleep, 50% damage restored as HP
	rTextbox(tPKMNDreamWasEaten)
	jp HealHalfDamage			; heals user by half of damage done
	
AE_SpecEndure:
	; Endure (qEndure)
	call PKMNProtect		; determine if success, c if fail, HL = pt to counter if success
	ret c					; ret if fail
	inc HL					; Endure - Counter = 1
	ld (HL), 1				; set endure
	rTextboxRet(tPKMNBracedItself)	; disp msg
	
AE_SpecExplosion:
	; Explosion (qExplosion)
	; user destroys itself (factored into Attack), damage w/ 1/2 target's defense (factored into CalcDef), ;fails if Damp;, no focus band
	ret
	
AE_SpecFacade:
	; Facade (qFacade)
	; base power doubles w/ burn, paralysis and poison (factored into CalcPow)
	ret
	
AE_SpecFrustration:
	; Frustration (qFrustration)
	; deals more damage w/ less happiness, factored into CalcPow
	ret
	
AE_SpecGrudge:
	; Grudge (qGrudge)
	; target loses all move PP if user faints, fails against substitute
	call GetPKMNSub
	jr nz, AE_SpecFailNZ			; fail if target has sub
	
	ld A, (TurnNum)			; A = 1 if attack is first in turn, 2 if second
	ld BC, PKMN_Grudge
	add HL, BC
	ld (HL), A				; grudge works on current turn if A = 1, next turn if A = 2
	rTextboxRet(tPKMNWantsGrudge)
	
AE_SpecIngrain:
	; Ingrain (qIngrain)
	; ingrains user, gain 1/16 max HP end of every turn, prev switching
	ld BC, PKMN_Ingrain
	add HL, BC
	ld A, (HL)
	or A
	
AE_SpecFailNZ:
	jp nz, AttackEffect_ButItFailed	; fail if ingrain already set
	
	ld (HL), 1				; set ingrain status to 1
	rTextboxRet(tPKMNPlantedItsRoots)	; disp msg
.endrelocate

AE_Spec2:
.relocate SwapZone
AE_SpecKnockOff:				; ~188 bytes to Rest
	; Knock Off (qKnockOff)
	; knocks off target pkmn's item, focus band triggers first, fails against sticky hold / substitute
	ex DE, HL
	call AttackEffect_ChkStickyHold	; check if target has sticky hold, returns z if target has sticky hold
	ret z
	
	call GetPKMNSub
	jp nz, AttackEffect_ButItFailed	; fail if target has substitute
	ex DE, HL
	
AttackEffect_SpecKnockOffNotHold:
	ld BC, PKMN_Item
	add HL, BC
	ld A, (HL)				; A = target item
	cp iNoItem
	ret z					; return if target has no item
	ld (HL), iNoItem			; set item to None
	ld (TextItem), A			; set (TextItem) for textbox
	rTextboxRet(tPKMNKnockedOff)
	
AE_SpecMemento:
	; Memento (qMemento)
	; if user faints, target attack and sp attack fall 2 stages, fails if target has substitute
	push DE
	call FlipPKMNFlag
	call Attack_ExplosionSub		; sets HP of user pkmn to 0, set Fainted flag, disp empty HP bar
	pop HL
	ld BC, PKMN_AtkBoost
	add HL, BC				; HL = pt to atk boost of target pkmn
	call AttackEffect_StatLowerClearBody; disp msg if clear body, saves A, HL, ret z if clear body
	ret z
	
	ld A, qAtkLower2
	push HL
	call AttackEffect_StatLowerCont2	; lower atk by 2 stages, factor in clear body + substitute
	pop HL
	ld A, qUserSpAtkLower2
	jp AttackEffect_StatLowerCont2	; lower sp atk by 2 stages, factor in clear body + substitute
	
AE_SpecMetronome:
	; Metronome (qMetronome)
	; attacks with random move, except for Counter, Destiny Bond, Detect / Protect / Endure, Focus Punch, Follow Me, Mirror Coat, 
	; Sleep Talk, Struggle, Trick...
	ret
	
AE_SpecMirrorCoat:
	; Mirror Coat (qMirrorCoat)
	; if last attack is special, then do 2x damage. ;no affect on dark type;
	; ;fail if hit sub;, fail if not second attack
	ld C, 2			; set C parameter to 2, for special
	call DoReflectDmg
	ret nc
	ld A, 1			; if c is set
	ld (PKMNFNT), A		; then set PKMNFNT
	ret
	
AE_SpecNightmare:
	; Nightmare (qNightmare)
	; reduces target's HP by 1/4 at end of turn, if asleep
	ex DE, HL			; HL = pt to target pkmn
	call GetPKMNSub
	jr nz, AE_SpecFailNZ2	; fail if target has substitute
	call GetPKMNStatus		; A = pkmn status, HL preserved
	cp stSLP			; fail if not asleep
	
AE_SpecFailNZ2:
	jp nz, AttackEffect_ButItFailed
	ld BC, PKMN_Nightmare
	add HL, BC			; HL = pt to nightmare status
	ld (HL), 1			; set nightmare status to 1
	rTextboxRet(tPKMNFellIntoANightmare)
	
AE_SpecPainSplit:
	; Pain Split (qPainSplit)
	; splits HP of user and target equally, fails if target has substitute
	ex DE, HL
	call GetPKMNSub		; A = substitute status of target
	ex DE, HL
	jr nz, AE_SpecFailNZ2	; fail if target has substitute
	
	ld BC, PKMN_HP		; BC = HP offset
	add HL, BC			; HL = pt to attacker HP
	ex DE, HL			; HL = pt to target, DE = pt to atk pkmn HP
	add HL, BC			; HL = pt to target HP
	call LdHLIndRAM		; HL = target HP value
	ex DE, HL			; HL = pt to atk pkmn HP, DE = target HP value
	call LdHLIndRAM		; HL = atk pkmn HP value
	push DE			; save DE, target pkmn HP value
	push HL			; save HL, atk pkmn HP value
	add HL, DE			; HL = HL + DE, atk pkmn HP + target pkmn HP
	srl H
	rr L				; HL = HL >> 1, HL / 2, average of atk + target HP
	ex DE, HL			; DE = HL
	pop HL				; restore HL, pt to atk pkmn HP
	call AE_SpecPainSplitSub	; apply pain split to atk pkmn
	call FlipPKMNFlag		; pkmn flag set to target pkmn
	pop HL				; restore HL, pt to target pkmn HP
	call AE_SpecPainSplitSub	; apply pain split to target pkmn
	res HealFlag, (IY + Asm_Flag2); restore heal flag
	rTextboxRet(tBattlersSharedPain)
	
AE_SpecPainSplitSub:			; subroutine - apply pain split to pkmn, input: HL = pkmn HP value, DE = avg HP, PKMNFlag, preserves DE
	push DE
	res HealFlag, (IY + Asm_Flag2); restore heal flag
	call CpHLDE			; if HL >= DE, then damage
	jr nc, AE_SpecPainSplitSubDam
	ex DE, HL			; else, heal
	set HealFlag, (IY + Asm_Flag2)
	
AE_SpecPainSplitSubDam:
	or A
	sbc HL, DE			; HL = HL - DE
	RCall( cDoDamage )		; do damage or heal
	pop DE
	ret
	
AE_SpecPyschUp:
	; Psych Up (qPsychUp)
	; copies targets stat changes (atk, def, satk, sdef, spd, acc, eva)
	ld BC, PKMN_AtkBoost		; BC = atk boost offset
	add HL, BC			; HL = pt to attacker atk boost
	push HL			; save HL
	ex DE, HL			; HL = pt to taget
	add HL, BC			; HL = pt to target atk boost
	pop DE				; DE = pt to attacker atk boost
	ld BC, PKMN_EvaBoost - PKMN_AtkBoost	; BC = num bytes to copy
	ldir				; copy stats from target to attacker
	rTextboxRet(tPKMNCopiedStatChanges)
	
AE_SpecPursuit:
	; Pursuit (qPursuit)
	; deals 2x damage if target switches out
	ret
.endrelocate
	
AE_Spec3:
.relocate SwapZone
AE_SpecRest:						; ~245 bytes to end
	; Rest (qRest)
	; user goes to sleep for 2 turns, heals all HP and status
	; fails if full HP or already asleep or can't sleep ;(insomnia, vital sprit);
	call ChkFullHP				; preserves HL
	jp nc, AttackEffect_ButItFailed
	ld BC, PKMN_Status
	add HL, BC
	ld A, (HL)					; A = pkmn status
	cp stSLP
	jp z, AttackEffect_ButItFailed		; fail if already asleep
	ld A, stSLP
	ld (HL), A					; else set status to sleep
	push HL
	RCall( cDispStatus )				; display status
	pop HL
	push HL					; save pt to status
	ld BC, PKMN_SLPCount - PKMN_Status
	add HL, BC
	ld (HL), 3					; sleep for 3 turns, including this turn
	ld BC, PKMN_SLPCount - PKMN_MaxHP
	or A
	sbc HL, BC
	call LdHLIndRAM				; HL = pkmn max HP
	RCall( cDoHeal )				; heal pkmn to full HP
	rTextbox(tPKMNSleptAndBecameHealthy)
	pop HL
	jp AttackEffect_StatusSleepChesto		; input: HL = pt to pkmn status
	
AE_SpecRevenge:
	; Revenge (qRevenge)
	; deals more damage if user takes damage, factored into CalcPow
	ret
	
AE_SpecSwitch:
	; Switch (qSwitch)
	; forces target to switch, fails if no pkmn left, fails against soundproof (roar) / suction cups / ingrain
	ld HL, (TargetPKMN)
	call GetPKMNIngrain				; A = target pkmn ingrain status, HL preserved
	jr z, AttackEffect_SwitchNoIngrain		; jump if no ingrain
	
	rTextboxRet(tPKMNAnchoredItself)		; else, fail w/ msg
	
AttackEffect_SwitchNoIngrain:
	call GetPKMNAbility				; A = pkmn ability, HL preserved
	cp aSuctionCups
	jr nz, AttackEffect_SwitchNoSuction	; jump if no suction cups
	
	ld (TextAbil), A				; set (TextAbil) to aSuctionCups
	rTextboxRet(tAnchorsItself)			; else, fail w/ msg
	
AttackEffect_SwitchNoSuction:
	call FlipPKMNFlag				; check target, not user
	call ChkParty					; check party, returns 
	call FlipPKMNFlag				; preserves c flag (bit, res/set, ret)
	jp c, AttackEffect_ButItFailed		; fail if c set, fail signal
	
	call AttackEffect_SwitchSub			; determine pkmn to switch to, perform switch
	
	ld A, 1
	ld (PKMNSwitch), A				; set PKMNSwitch flag
	
	jp AE_SpecRecoilFNT				; if pkmn faints, set PKMNFNT (last pkmn cannot FNT from spikes)
	; TODO: glitch when roar -> opp FNT from spikes -> opp sends out Skarmory -> Skarm takes spikes damage
	
AE_SpecRollout:
	; Rollout (qRollout)
	; doubles power with each consecutive hit, locks user for 5 turns or until interrupted, defense curl doubles power
	ld BC, PKMN_Rollout
	add HL, BC
	ld A, (HL)					; A = rollout counter
	inc (HL)					; inc rollout counter, A not changed
	cp 5
	jr nz, AttackEffect_SpecRolloutNot5	; if rollout counter at 5
	ld (HL), 0					; then reset to 0
	
AttackEffect_SpecRolloutNot5:
	ret ;;;
	
AE_SpecSkillSwap:
	; Skill Swap (qSkillSwap)
	; ;swaps skill with target;, ;wonder guard cannot be swapped;
	call GetPKMNAbility				; A = atk pkmn ability (or traced abililty), HL, DE preserved
	push AF					; save A, atk pkmn ability
	ex DE, HL					; swap DE, HL
	call GetPKMNAbility				; A = target pkmn ability
	set 7, A					; sets most significant bit of A
	ex DE, HL					; swap DE, HL
	ld BC, PKMN_SkillSwap			; BC = skill swap offset
	add HL, BC					; HL = pt to atk pkmn skill swap
	ld (HL), A					; set atk skill swap to target pkmn ability ID | 0x80
	ex DE, HL					; swap DE, HL
	add HL, BC					; HL = pt to target pkmn skill swap
	pop AF						; restore A, atk pkmn ability
	ld (HL), A					; set target skill swap to atk pkmn ability ID | 0x80
	rTextboxRet(tPKMNSwappedAbilities)		; ret w/ msg
	
AE_SpecSleepTalk:
	; Sleep Talk (qSleepTalk)
	; attack w/ random move when asleep, cannot use Sleep Talk, ;Fly, Focus Punch, Uproar ... ;
	; PP not deducted for move used, rollout not boosted, rest fails, moves w/ no PP fail; factored into Attack
	ret
	
AE_SpecSubstitute:
	; Substitute (qSubstitute)
	; uses 1/4 HP to produce substitute, passed by baton pass, blocks status, leech seed, mean look, confusion, stat lower, lose item
	; intimidate, ghost curse, dream eater, grudge, nightmare, pain split, ;partial trap;, ;damage to sub cannot be countered;, ;smellingsalt;
	call ChkFourthHP				; ret c if current HP < 1/4 max HP, ret z if equal, DE = 1/4 max HP
	jr z, AE_SpecSubstituteWeak
	jr c, AE_SpecSubstituteWeak
	ld BC, PKMN_Substitute
	add HL, BC					; HL = pt to sub status
	ld A, (HL)
	or A
	jr nz, AE_SpecSubstituteAlready
	
	ld (HL), E					; store DE in (HL), little-endian, assumes DE < 256
	
	ex DE, HL					; HL = 1/4 max HP
	RCall( cDoDamage )				; do damage of 1/4 max HP
	
	call DispPKMNSprite				; display front / back sprite, depending on PKMNFlag
	
	rTextboxRet(tPKMNMadeASubstitute)		; disp msg, return
	; TODO: sub has current stats of pkmn
	
AE_SpecSubstituteWeak:
	rTextboxRet(tTooWeakToMakeSub)		; disp msg, return
	
AE_SpecSubstituteAlready:
	rTextboxRet(tPKMNAlreadyHasASubstitute)	; disp msg, return
	
AE_SpecTrick:
	; Trick (qTrick)
	; swap item with target, fails if not holding item, target has substitute or sticky hold
	ex DE, HL
	call AttackEffect_ChkStickyHold		; check if target has sticky hold, returns z if target has sticky hold, preserves HL, DE if nz
	ret z
	
	call GetPKMNSub
	jp nz, AttackEffect_ButItFailed		; fail if target has substitute
	ex DE, HL
	
	ld BC, PKMN_Item
	add HL, BC					; HL = pt to atk pkmn item
	ld A, (HL)					; A = atk pkmn item
	ex DE, HL
	add HL, BC					; HL = pt to target pkmn item
	ld B, (HL)					; B = target pkmn item
	ld C, A					; C = atk pkmn item
	ld A, iNoItem
	cp B						; if B = iNoItem
	jr z, AttackEffect_SpecTrickFail		; then trick fails
	cp C						; if C = iNoItem
AttackEffect_SpecTrickFail:
	jp z, AttackEffect_ButItFailed		; then trick fails
	ld (HL), C					; else, set target pkmn item to atk pkmn item
	ex DE, HL					; HL = pt to atk pkmn item
	ld (HL), B					; set atk pkmn item to target pkmn item
	rTextboxRet(tSwitchedItemsOpp)
	
AE_SpecRecoil2:
	; Recoil2
	; struggle, user takes 1/2 damage as recoil
	call GetHalfDamage				; HL = (Damage) / 2
	ld A, H
	or L
	ret z						; ret if no recoil damage
	
	rTextbox(tPKMNIsHitWithRecoil)
	
	RCall( cDoDamage )
	
AE_SpecRecoilFNT:
	ret nc						; if user faints
	ld A, 1
	ld (PKMNFNT), A				; set PKMNFNT
	ret
.endrelocate
