; ===============================================================
; Routines
; ===============================================================

Routines_Start:

; ---------------------------------------------------------------
; RProgram
; ---------------------------------------------------------------

RProgram:
.relocate SwapZone
	call Init		; initialize flags, variables, interrupt, get save file
	
Start:
	call MainMenu		; bring up main menu, return selected option
	or A			; If option = Play (0)
	jr z, Play		; Then, goto Play
	dec A			; If option = Record (1)
	jr z, Record		; Then, goto Record
	dec A			; If option = Info (2)
	jr z, About		; Then, goto Info
	
	call Quit		; Else, quit
	
Record:
	call Results		; display score results
	jr Start		; go back to start
	
About:
	call AboutDisp	; display about info
	jr Start		; go back to start
	
Play:
	call SelectRound	; select round (1 - 5)
	cp $FF			; if A = $FF
	jr z, Start		; then go back to start
	
ResearchOpp:
	call Research		; bring up research menu, return selected option
	or A			; If option = Rent (0)
	jr z, Select		; Then, goto select team
	cp 2			; If option = Swap (2)
	jr z, Swap		; then swap
	cp 3			; If option = Continue (3)
	jr z, BattleStart	; then go to battle
	
	ld (SaveVar), A	; set savevar to 1
	jr Start		; Else, return (option = 1 | 4)
	
Swap:
	call SwapTeam		; swap pkmn w/ opponent
	jr BattleStart	; skip over selectteam
	
Select:
	call SelectTeam	; rent pkmn
	
BattleStart:
	call Relocate		; RAM = Battle
	call Battle		; start battle
	call Relocate		; RAM = Overworld
	
	ld A, (BattleWin)
	inc A			; if A = $FF
	jr z, BattleEnd	; end if opp wins
	inc A			; if A = $FE
	jr z, BattleEnd	; end if forfeit
	
	ld A, (MatchNum)
	dec A			; if A = 1
	jr z, Start		; return to menu if new round
	
	jr ResearchOpp	; loop back to research
	
BattleEnd:
	call HighScore	; check if beat high score
	jr Start		; return to menu
.endrelocate

; ---------------------------------------------------------------
; StringLen:
; Returns the pixel length of a string, in small font
; Input: HL = Pointer to string (Zero terminated)
;	  B = Max length
; Output: C = length
; ---------------------------------------------------------------

StringLen:
.relocate SwapZone
	xor A
	ld C, A
StringLenLp:
	bit ROMFlag, (IY + Asm_FLag1)
	jr nz, StringLenROM
	ld A, (HL)		; RAM
	jr StringLenRAM
StringLenROM:
	call GetByte	; ROM
StringLenRAM:
	or A
	ret z
	push HL
	push BC
	RCall( cCharLen )
	pop BC
	pop HL
	add A, C
	ld C, A
	inc HL
	djnz StringLenLp
	ret
.endrelocate

; ---------------------------------------------------------------
; CharLen
; Returns the pixel length of a character, in small font
; Input: A = Character
; Output: A = length
; ---------------------------------------------------------------

CharLen:
.relocate SwapZone
	ld H, 0
	ld L, A
	add HL, HL		; HL * 8
	add HL, HL
	add HL, HL
	b_call(_SFont_Len)
	ld A, B
	ret
.endrelocate
	
; ---------------------------------------------------------------
; RightText:
; Moves PenCol for right justification; assumes 4 pix/char
; Input: A = lenth of string (chars)
; Output: A = (PenCol) = x position of text (pix)
; ---------------------------------------------------------------

RightText:
.relocate SwapZone
	sla A			; 95 - (A * 4)
	sla A
	ld B, A
	ld A, 95
	sub B
	ld (PenCol), A
	ret
.endrelocate
	
; ---------------------------------------------------------------
; PutSN
; Puts string to buffer, with max char length
; Input: HL = pt to string
;        B = max char length
;        (PenCol), (PenRow)
; ---------------------------------------------------------------

PutSN:
.relocate SwapZone
	xor A
	cp B
	ret z			; ret if B = 0
PutSNLp:
	ld A, (HL)		; A = char to display
	or A			; if A = 0, null char
	ret z			; then ret
	b_call(_VPutMap)	; put one character, preserves BC, HL
	inc HL			; move HL to next char
	djnz PutSNLp		; loop until B = 0
	ret
.endrelocate
	
; ---------------------------------------------------------------
; SwapOP1OP2:
; Swaps OP registers
; ---------------------------------------------------------------

SwapOP1OP2:
.relocate SwapZone
	ld HL, OP1
	ld DE, OP2
	;ld BC, 11
	;call SwapRAM
	;ret
	ld B, 11
	jp SwapRAMB
.endrelocate
	
; ---------------------------------------------------------------
; GetInput:
; Uses ports to get key input
; Output: A = keycode (GetCSC)
; ---------------------------------------------------------------

GetInput:
.relocate SwapZone
	halt
	;halt
	;halt
	
gs_GetK2:
	ld b,7			; loop 7 times
gs_GetK_loop:
	ld a,7
	sub b			; A = 7 - B
	ld hl,gs_keygroups
	ld d,0 \ ld e,a
	add hl,de		; HL = keygroup table base + A
	ld a,(hl)
	ld c,a			; C = A = key
	
	ld a,0ffh           ;resets the keypad.
	out (1),a           ;'Required syntax' :)
	ld a,c
	out (1),a
	in a,(1)            ;'Required Syntax'
	
	ld HL, gs_keyvalues	; HL = pt to table base
	ld DE, 8		; 8 values to compare to
	
gs_GetK_lp2:
	cp (HL)		; if A == (HL)
	jr z, gs_GetK_val	; then end
	inc HL			; else inc HL to next value
	dec E			; dec E
	jr nz, gs_GetK_lp2	; loop until E = 0
	
	djnz gs_GetK_loop

	xor a
	ld (gs_keymem),a
	ret

gs_GetK_val:
	ld hl,gs_keygroup1
	dec DE			; DE--, change from (1 - 8) to (0 - 7)
	add hl,de		; HL = key group base + DE (0 - 7)
	ld a,7
	sub b			; A = 7 - B
	add a,a \ add a,a \ add a,a
	ld d,0 \ ld e,a	; DE = A * 8
	add hl,de		; HL = HL + DE
	ld a,(hl)		; A = key
	
	ld HL, gs_keymem
	cp (HL)		; if A = (key mem)
	ret z			; then end
	ld (HL), A		; else (key mem) = A

gs_GetK_end:
	ret
.endrelocate

; ---------------------------------------------------------------
; Wait
; Waits for keypress, then returns
; ---------------------------------------------------------------

Wait:						; subroutine - waits until 2nd | enter is pressed
.relocate SwapZone
WaitLp:
	RCall( cGetInput )
	or A
	jr z, WaitLp
	cp gk2nd				; return if 2nd | Alpha | Clear | Enter is pressed
	ret z
	cp gkAlpha
	ret z
	cp gkClear
	ret z
	cp gkEnter
	ret z
	;cp gkMode				; quit if Mode pressed (BattleQuit)
	;jp z, Quit
	jr WaitLp
.endrelocate

; ---------------------------------------------------------------
; WaitYes
; Waits for keypress, toggles between yes / no options
; Output: c = option selected, nc = draw
;         A = (SelectTeamOptNum) = current option
; ---------------------------------------------------------------

WaitYes:
.relocate SwapZone
WaitYesLp:
	RCall( cGetInput ) ;b_call(_GetCSC)
	or A
	jr z, WaitYesLp
	
	cp gkEnter
	jr z, WaitYesDone		; ret w/ c if done
	cp gk2nd
	jr z, WaitYesDone		; ret w/ c if done
	cp gkAlpha
	jr z, WaitYesNo
	cp gkClear
	jr z, WaitYesNo
	;cp gkMode
	;jp z, Quit	(BattleQuit)
	cp gkUp
	jr z, WaitYesUp
	cp gkDown
	jr z, WaitYesDown
	jr WaitYesLp

WaitYesDone:
	ld A, (SelectTeamOptNum)
	scf
	ret				; ret w/ c, option selected
	
WaitYesNo:
	xor A
	inc A
	ld (SelectTeamOptNum), A	; set option to 1
	scf
	ret				; ret w/ c
	
WaitYesUp:				; up and down do the same thing
WaitYesDown:
	ld A, (SelectTeamOptNum)	; A = opt num
	xor 1				; invert A
	ld (SelectTeamOptNum), A	; set opt num
	ret				; ret w/ nc 
.endrelocate

; ---------------------------------------------------------------
; CopySave
; Copies data to PKMNSaveData
; ---------------------------------------------------------------

CopySave:						; subroutine - copies data to PKMNSaveData
.relocate SwapZone
	ld HL, TeamData
	ld DE, PKMNSaveTeam
	ld BC, 18
	ldir						; copy team data, opp team data
	
	ld HL, PrevOppTeam
	ld DE, PKMNSavePrev
	ld BC, 9
	ldir						; copy previous opp team data
	
	ld A, (BattleNum)				; A = battle number LSB
	ld (DE), A
	inc DE
	ld A, (BattleNum + 1)			; A = battle number MSB
	ld (DE), A					; copy battle number to (DE), little endian
	inc DE
	
	ld A, (RoundNum)
	ld (DE), A					; copy round number
	inc DE
	ld A, (MatchNum)
	ld (DE), A					; copy match number
	;inc DE
	
	ret
.endrelocate
	
; ---------------------------------------------------------------
; SaveData:
; Saves high score data in PKMNSAVE appvar
; ---------------------------------------------------------------

SaveData:
.relocate SwapZone
	call SaveDataFind				; look for app var
	jr nc, SaveDataExists			; if found, then store data
	
	ld HL, SAVESIZE
	b_call(_CreateAppVar)			; else if not found, then create new appvar
	jr SaveDataRAM
	
SaveDataExists:
	xor A
	or B						; if B = 0
	jr z, SaveDataRAM				; then appvar in RAM
	b_call(_Arc_Unarc)				; unarchive appvar
	call SaveDataFind
	
SaveDataRAM:
	inc DE						; skip first two bytes
	inc DE
	ld A, SAVEID					; set token ID byte
	ld (DE), A
	inc DE						; skip token ID byte
	ld HL, PKMNSaveData
	ld BC, SAVESIZE - 3				; BC = size of appvar - 3
	ldir						; copy data
	
	call SaveDataFind
	b_call(_Arc_Unarc)				; automatically archive appvar	
	ret						; return
	
SaveDataFind:
	ld HL, PKMNSAVEName				; HL = pt to appvar name //TODO: integrate w/ InitGetSaveFind?
	rst rMov9ToOP1				; OP1 = data type + var name
	b_call(_ChkFindSym)				; searches symbol table for PKMNSAVE appvar, returns c if not found
	ret
.endrelocate

; ---------------------------------------------------------------
; RSwapRAM
; Swap two blocks of RAM using stack
; Input: HL = Pt to RAM block 1
;        DE = Pt to RAM block 2
;        BC = Amount of RAM to swap
; ---------------------------------------------------------------

RSwapRAM:
.relocate SwapZone
	di						; disable interrupts, don't draw to screen
	ld (Buffer), HL				; save HL, pt to RAM block 1
	ld (Buffer + 2), DE				; save DE, pt to RAM block 2
	push BC
	pop HL						; HL = BC, amount of RAM to swap
	
RSwapRAMLp1:
	or A						; reset c flag, prepare for sbc
	ld BC, 120					; BC = 120
	push HL					; save HL, amount of RAM left
	sbc HL, BC					; HL = HL - BC
	jr nc, RSwapRAMLp2				; skip if no overflow
	pop BC						; else, BC = HL, amount of RAM to swap = amount of RAM left
	push BC
	xor A
	ld H, A
	ld L, A					; HL = 0
	
RSwapRAMLp2:
	pop DE						; DE = HL, amount of RAM to swap
	push HL					; save HL
	ld HL, (Buffer)				; HL = pt to RAM block 1
	ld DE, Buffer + 4				; DE = pt to free RAM
	push DE					; save pt to free RAM
	push HL					; save pt to RAM block 1
	push BC					; save BC ($00C8)
	ldir						; copy BC bytes from RAM block 1 to free RAM
	pop BC						; restore BC ($00C8)
	pop DE						; DE = pt to RAM block 1
	ld HL, (Buffer + 2)				; HL = pt to RAM block 2
	push HL					; save pt to RAM block 2
	push BC					; save BC ($00C8)
	ldir						; copy BC bytes from RAM block 2 to RAM block 1
	pop BC						; restore BC ($00C8)
	ld (Buffer), DE				; save new index of RAM block 1
	pop DE						; DE = pt to RAM block 2
	pop HL						; HL = pt to free RAM
	ldir						; copy BC bytes from free RAM to RAM block 2
	ld (Buffer + 2), DE				; save new index of RAM block 2
	pop HL						; HL = amount of RAM to swap
	ld A, H					; A = H
	or L						; if not H | L = 0, HL = 0
	jr nz, RSwapRAMLp1				; then continue looping
	ei						; enable interrupts, flags preserved
	ret						; else, ret if no RAM left to swap
.endrelocate

; ---------------------------------------------------------------
; Textbox:
; Draws a textbox, displays text
; Input: HL = memory offset of text (NBTS, encoded)
; Flags: BlinkerOn, 1 = display blinker
; ---------------------------------------------------------------
; TODO: Magmar used Flamethrower!
; TODO: Text compression?
Textbox:				; ~246 bytes
.relocate SwapZone
	bit SilenceFlag, (IY + Asm_Flag2)
	ret nz				; ret if silence flag is set
	
	RCall( cTextboxInit )	; decodes text, clears space, init variables
	jr textbox1space		; skips decoding
	
textbox1spacedc:
	ld A, 30
	ld (TextSize), A
textbox1spacedc2:
	di
	RCall( cArcTextDc )
	ei
	
textbox1space:			; jr point
	RCall( cTextboxSpace )	; check for new line before displaying
textbox1get:				; jr point
	;call GetByte			; get char --> A
	ld A, (HL)
textbox1:
;	cp ' '				; if char == space:
;	jr z, textbox2		;    determine whether next word can fit in textbox
	cp TxtNum			; chks whether to disp number, don't save to (TxtTemp)
	jr z, TextBoxNum
	cp $F0				; chks if A < $F0
	jp c, TextboxPut
	
	ld (TxtTemp), HL		; save pos in text
	cp TxtAtk			; chks whether to disp attacker name
	jr z, TextBoxAtk
	cp TxtTrgt			; chks whether to disp target name
	jr z, TextBoxTrgt
	cp TxtMove			; chks whether to disp attacker move
	jr z, TextBoxMove
	cp TxtMove2			; chks whether to disp target move
	jr z, TextBoxMove2
	cp TxtMove3			; chks whether to disp arbitrary move
	jr z, TextBoxMove3
	cp TxtPKMN			; chks whether to disp pkmn name
	jr z, TextBoxPKMN
	cp TxtStat			; chks whether to disp stat
	jr z, TextBoxStat
	cp TxtItem			; chks whether to disp item
	jr z, TextBoxItem
	cp TxtTeam			; chks whether to disp 'Ally' or 'Foe'
	jr z, TextBoxTeam
	cp TxtAbil			; chks whether to disp target pkmn's ability
	jr z, TextBoxAbil
	jr textbox1get		; get another char if not recognized
	
textbox2:
;	RCall( cTextboxSpace )
;	jr textbox1
	
TextBoxAtk:
	ld HL, (AtkPKMN)		; get HL = attacking pkmn
TextBoxGetName:
	call GetPKMNSpecies		; get A = species ID
TextBoxGetName2:
	RCall( cGetName3 )		; get HL = species name
	jr textbox1spacedc
	
TextBoxGetName3:
	RCall( cGetName3 )		; get HL = species name
	ld A, 10
	ld (TextSize), A		; set textsize to 10
	jr textbox1spacedc2		; decode, skip setting textsize
	
TextBoxTrgt:
	ld HL, (TargetPKMN)
	jr TextBoxGetName		; jump ^
	
TextBoxStat:
	ld A, (TextStat)
	ld HL, StatDb
	RCall( cGetTable2 )
	jr textbox1spacedc
	
TextBoxMove:
	ld A, (AtkMove)
TextBoxMoveText:
	RCall( cGetMoveText )
TextBoxMoveTxtJr:
	jr textbox1spacedc
	
TextBoxMove2:
	ld A, (TargetMove)
	jr TextBoxMoveText		; jump ^
	
TextBoxMove3:
	ld A, (TextMove)
	jr TextBoxMoveText
	
TextBoxNum:
	ld A, (TextNum)
	add A, '0'			; get char code
	jr TextBox1			; A = first char
	
TextBoxPKMN:
	bit PKMNFlag, (IY + Asm_Flag1)
	jr z, TextBoxPKMNOpp
	ld HL, PlayerPKMN + PKMN_Species
	jr TextBoxPKMNEnd
	
TextBoxItem:
	ld A, (TextItem)
	RCall( cGetItemText )
	jr TextBoxMoveTxtJr
	
TextBoxTeam:
	bit PKMNFlag, (IY + Asm_Flag1)
	jr nz, TextBoxTeamAlly
TextBoxTeamFoe:
	ld HL, tFoe
	jr TextBoxMoveTxtJr
	
TextBoxTeamAlly:
	ld HL, tAlly
	jr TextBoxMoveTxtJr
	
TextBoxPKMNOpp:
	ld HL, OppPKMN + PKMN_Species
TextBoxPKMNEnd:
	ld A, (HL)
	jr TextBoxGetName3		; jump ^
	
TextBoxAbil:
	;ld HL, (TargetPKMN)		; HL = pt to target pkmn
	;call GetPKMNAbility2	; A = ability, disregard traced ability
	ld A, (TextAbil)
	RCall( cGetAbilityText )	; HL = pt to ability text
	jr TextBoxMoveTxtJr
	
TextboxPut:
	push AF			; save current char
	b_call(_VPutMap)		; display characters one at a time
	
	push HL
	ld HL, PlotSScreen + 48 * 12; copy buffer section
	ld DE, SaveSScreen + 48 * 12
	ld BC, 14 * (96 / 8)
	ldir
	pop HL
	;halt
	
	di
	RCall( cTextboxPutSub )	; determine whether to stop on next char
	ei
	pop DE				; get D = last char
	jr z, textbox0		; if z, then goto textbox0
	ld E, A			; save current char in E
	ld A, D			; get A = last char
	cp ' '				; check if last char is space
	ld A, E			; restore current char
	jp z, textbox1space		; then check for new line
	jp textbox1			; else, goto textbox1
	
textbox0:
	bit BlinkerOn, (IY + Asm_Flag1)	; if flag is set
	jr z, TextboxNoBlink
	RCall( cblinker )			; then blink
	
TextboxNoBlink:
	;ld B, 10				; else stall
textboxStallLp:
	;halt \ halt \ halt	; temp speed up
	
	;djnz textboxStallLp
	ret
.endrelocate
;textboxdone:
;.echo textboxdone - textbox

; ---------------------------------------------------------------
; TextboxPutSub
; Textbox subroutine, puts one character, determines if end
; Input: HL = pt to current text
; Output: z = signal for textbox0, nz = signal for textbox1
;         A = C = char, if nz
; ---------------------------------------------------------------

TextboxPutSub:
.relocate SwapZone
	inc HL				; next char
TextboxPutSubCall:
	;call textboxA		; A = C = char
	;call TextboxPutSubCall + (TextboxPutSubEnd - TextboxPutSubCall) + (textboxAR - TextboxSpace0)
	ld A, (HL)
	cp $20				; terminate if char < $20 or null, substitutes for 10-char name limit
	jr nc, TextboxPutSubRet
	ld DE, (TxtTemp)
	ld A, D
	or E				; only if temp is completely reset will the routine terminate
	ret z				; ret w/ z set
	ld HL, (TxtTemp)
	inc HL				; else display rest of string
	;call GetByteC
	;ld A, C
	ld A, (HL)
	ld DE, 0			; reset (TxtTemp) flag
	ld (TxtTemp), DE
TextboxPutSubRet:
	inc D				; D = 0 + 1, reset z flag for textbox1
	ret
TextboxPutSubEnd:
.endrelocate
	
; ---------------------------------------------------------------
; TextboxSpace
; Textbox subroutine, determines if next word can fit on line
; Input: (PenCol) = current column
;        HL = pt to current text
; ---------------------------------------------------------------

TextboxSpace:
.relocate SwapZone
TextboxSpace0:
	ld A, (PenCol)		; save initial x coordinate
	push AF
	push HL			; save HL
	ld D, H			; DE = HL
	ld E, L
textbox3:
	push DE
	ld A, (HL)
	;call textboxAR		; A = C = char
	inc HL				; next char
	pop DE
	cp $20				; if char = " "
	jr z, textbox4		; goto textbox4
	or A				; if char is null
	jr z, textbox4		; goto textbox4
	jr textbox3			; else loop
textbox4:
	or A
	sbc HL, DE
	ld A, L			; A = length of word
	cp 10
	jr c, textbox4lt10		; if A >= 10
	ld A, 10			; then A = 10, accounts for 10-char pkmn names, assumes words <= 10 chars
	
textbox4lt10:
	sla A
	sla A				; A = A * 4 (length in pixels)
	ld C, A			; C = length in pixels
	pop HL				; get HL, pt to current text
	pop AF				; get A, initial x coordinate
	inc A				; increase column by 1 to account for space
;	ld (PenCol), A
	add A, C			; A = C + PenCol (to find how far the next word extends)
	cp 88				; if A < 88
	jr c, textbox7		; goto textbox7	
	ld A, (PenRow)		; else, if pen is at first row
	cp 47
	jr z, textbox6		; skip ahead
	push HL
	RCall( cblinker )		; else scroll text
	ld A, (PenCol)		; clear blinker
	sub 5
	ld A, 56
	ld (PenRow), A		; set pen to second row
	set TextEraseBelow, (IY + TextFlags);
	ld A, $06			; blank character
	b_call(_VPutMap)
	ld A, $20			; another blank char
	b_call(_VPutMap)
	res TextEraseBelow, (IY + TextFlags);
	ld A, 9			; scroll 9 times
textbox5:
	ld HL, PlotSScreen + 48 * 12; (0, 48)
	ld DE, PlotSScreen + 47 * 12; (0, 47)
	ld BC, (62 - 47) * 12
	ldir
	halt
	dec A
	or A
	jr nz, textbox5
	pop HL				; get HL = pt to current text
textbox6:
	rTextCoord(2, 56)		; else next word put at (2, 56)
textbox7:
;	inc HL				; next char
;	call textboxAR		; A = C = char
	ret				; return as normal
	
textboxAR:				; subroutine that finds character in ROM or RAM and outputs to C and A
	bit TextROM, (IY + Asm_Flag1)
	jr nz, textboxA1R		; if (TextROM) == 1, ROM
	ld A, (HL)			; else, RAM
	ld C, A
	ret
textboxA1R:
	call GetByteC
	ld A, C
	ret
.endrelocate

; ---------------------------------------------------------------
; TextboxInit:
; Decodes text, clears space, zeros TxtTemp, set TextROM, coords
; ---------------------------------------------------------------

TextboxInit:
.relocate SwapZone
	di
	RCall( cArcTextDc2 )		; decode text, returns HL = pt to decoded text
	RCall( cClearText )		; clear textbox space, saves HL
	ei
	ld DE, 0			; zero out temp (when temp is /= 0, 'substring' is being displayed)
	ld (TxtTemp), DE
	set TextROM, (IY + Asm_Flag1); read from ROM by default
	rTextCoord(2, 47)		; start at (2, 47), HL not affected
	ret
.endrelocate

; ---------------------------------------------------------------
; blinker:
; Displays a blinker, and waits for a key press
; Input: (PenCol) and (PenRow) = position of blinker
; ---------------------------------------------------------------

blinker:
.relocate SwapZone
	res TempFlag, (IY + Asm_Flag2)	;reset counter (0 if down, 1 if up)
	set TextEraseBelow, (IY + TextFlags)
blinker1:
	ld B, 20 ;100; 255		;reset counter
	ld A, $FD			;monitor for [ENTER] press
	out (1), A
	ld A, $07			;diamond/arrow character
	b_call(_VPutMap)
	push BC
	ld HL, PlotSScreen + 48 * 12; copy buffer section
	ld DE, SaveSScreen + 48 * 12
	ld BC, 14 * (96 / 8)
	ldir
	pop BC
	
	ld A, (PenCol)		;return pencol to initial position
	sub 5
	ld (PenCol), A
blinker10:
	in A, (1)			;compare input with [ENTER] key
	cp $FE
	jr z, blinker6
	ld A, $BF			; monitor for [2ND] press
	out (1), A
	dec B				;count down
	xor A				;if B isn't at 0
	cp B
	jr nz, blinker5		;skip toggling
blinker2:
	ld A, (PenRow)		;toggle phase
	bit TempFlag, (IY + Asm_Flag2)
	jr z, blinker3		;if bit is 0, skip ahead
	inc A				;else,
	res TempFlag, (IY + Asm_Flag2)
	res TextEraseBelow, (IY + TextFlags)
	jr blinker4
blinker3:
	dec A
	set TempFlag, (IY + Asm_Flag2)
	set TextEraseBelow, (IY + TextFlags)
blinker4:
	ld (PenRow), A
blinker5:
	in A, (1)			; compare input with [2ND] key
	cp $DF
	jr nz, blinker1
blinker6:
	res TextEraseBelow, (IY + TextFlags)
	ret				;return
.endrelocate
	
	