; ===============================================================
; Attack
; ===============================================================

; ---------------------------------------------------------------
; Attack
; Attacks one target PKMN; damage; effects
; Input: (StatVars), etc.
; Output: c = fainted (user or target)
;         (Player/OppFainted)
; ---------------------------------------------------------------

Attack2:				; opponent entry point
	res TurnFlag, (IY + Asm_Flag1)
	set PKMNFlag, (IY + Asm_Flag1)
	ld HL, TurnNum
	inc (HL)
	ld HL, OppPKMN
	ld (AtkPKMN), HL
	ld DE, PlayerPKMN
	ld (TargetPKMN), DE
	ld A, (CurMoveOpp)
	ld (AtkMove), A
	ld A, (CurMovePlayer)
	ld (TargetMove), A
	jr Attack_Start

Attack:				; player entry point	
	set TurnFlag, (IY + Asm_Flag1)
	res PKMNFlag, (IY + Asm_Flag1)
	ld HL, TurnNum
	inc (HL)
	ld HL, PlayerPKMN
	ld (AtkPKMN), HL
	ld DE, OppPKMN
	ld (TargetPKMN), DE
	ld A, (CurMovePlayer)
	ld (AtkMove), A
	ld A, (CurMoveOpp)
	ld (TargetMove), A
	
Attack_Start:				; Input: HL = attacker, DE = target
	ld HL, 0
	ld (Damage), HL		; (Damage) is 0 by default
	
	ld HL, PKMNSwitch
	ld A, (HL)
	or A
	ld (HL), 0			; reset PKMNSwitch flag
	ret nz				; return if pkmn came in on switch, from roar
	
	ld A, (AtkMove)
	cp $FD				; if player is switching out
	jr c, Attack_NotSwitch
	set PKMNFlag, (IY + Asm_Flag1)
	rTextbox(tComeBack)		; //TODO: randomize text, disp minimize animation
	ld B, A
	xor A
	sub B				; A = 0 - move ($FD - $FF), A = 1 - 3
	call Turn_SwitchPlayer
	call Turn_SwitchPlayerAbil
	ret				; ret c if all user pkmn fainted from spikes
	; TODO: generalize to opponent switch
	
Attack_NotSwitch:
	
; ---------------------------------------------------------------
; Pre-attack priority:
; Recharge
; Sleep
; Frozen
; Flinch
; Disabled / Torment
; Confused
; Attract
; Paralyzed
; ---------------------------------------------------------------
	
Attack_Charge:			; assumes A = (AtkMove)
	cp $FC				; compare (AtkMove) to charge signal
	jr nz, Attack_NotCharge	; else if charge signal
	call FlipPKMNFlag
	rTextbox(tPKMNMustRecharge)	; then disp message
	call FlipPKMNFlag
	jp Attack_Return		; and return
	
Attack_NotCharge:
Attack_Sleep:
	ld HL, (AtkPKMN)
	ld BC, PKMN_Status
	add HL, BC
	ld A, (HL)			; A = status ailment
	cp stSLP
	jr nz, Attack_NotSLP		; if status = SLP
	push HL
	ld BC, PKMN_SLPCount - PKMN_Status
	add HL, BC
	ex DE, HL			; DE = pt to sleep counter
	pop HL
	ex DE, HL
	dec (HL)			; dec counter
	ex DE, HL
	jr z, Attack_WokeUp		; if counter == 0, then pkmn wakes up
	push HL
	ld HL, (AtkPKMN)
	call GetPKMNAbility		; A = atk pkmn ability, preserves HL, DE
	cp aEarlyBird
	pop HL
	jr nz, Attack_NotEarlyBird	; if ability is early bird
	ex DE, HL
	dec (HL)			; dec counter again
	ex DE, HL
	jr z, Attack_WokeUp		; if counter == 0, then pkmn wakes up
	
Attack_NotEarlyBird:
	rTextbox(tPKMNIsFastAsleep)
	ld A, (AtkMove)
	cp mSleepTalk			; if move = sleep talk
	jr z, Attack_NotSLP		; then continue, dont return
	jp Attack_ReturnSolar	; else, return (w/o resetting solarbeam / sky attack)
	
Attack_WokeUp:
	xor A				; A = stNone
	ld (HL), A
	ld BC, PKMN_Truant - PKMN_Status
	add HL, BC
	ld (HL), A			; reset truant status
	rTextbox(tPKMNWokeUp)
	xor A
	call FlipPKMNFlag
	RCall( cDispStatus )		; clear status icon
	call FlipPKMNFlag
	jr Attack_Flinch
	
Attack_NotSLP:
Attack_Frozen:
	ld HL, (AtkPKMN)
	ld BC, PKMN_Status
	add HL, BC
	ld A, (HL)			; A = status ailment
	cp stFRZ			; if pkmn frozen
	jr nz, Attack_NotFRZ
	ld B, 100
	push HL
	call iRandom
	pop HL
	cp 20				; 20% chance to defrost
	jr c, Attack_Defrost		; else pkmn is frozen solid
	rTextbox(tPKMNIsFrozenSolid)
	jp Attack_ReturnSolar	; else, return (w/o resetting solarbeam / sky attack)
	
Attack_Defrost:
	ld (HL), stNone
	rTextbox(tPKMNWasDefrosted)
	xor A
	call FlipPKMNFlag
	RCall( cDispStatus )
	call FlipPKMNFlag
	jr Attack_Disable		; jp to confuse?
	
Attack_NotFRZ:
Attack_Truant:
	ld HL, (AtkPKMN)
	call GetPKMNAbility		; A = pkmn ability, HL preserved
	cp aTruant			; if ability is not truant
	jr nz, Attack_NotTruant	; then skip
	
	ld BC, PKMN_Truant
	add HL, BC
	ld A, (HL)			; A = pkmn truant status
	or A				; if A = 0
	ld (HL), 0			; reset truant status
	jr z, Attack_Truant0		; then set truant to 1 for next turn
	
	rTextbox(tLoafingAround)	; disp loafing message
	jp Attack_Return		; return
	
	; TODO: set truant status to first turn when slaking brought in after pkmn FNT to EOT damage
	; hyper beam charges on loafing turn
	
Attack_Truant0:
	ld (HL), 1
	
Attack_NotTruant:
Attack_Flinch:
	ld HL, (AtkPKMN)
	call GetPKMNFlinch		; A = flinch status, HL preserved
	jr z, Attack_NotFlinch	; if A = 0, then skip
	
	call GetPKMNAbility		; A = ability, HL preserved
	cp aInnerFocus		; if ability is not inner focus
	jr nz, Attack_NotFocus	; then skip
	
	ld (TextAbil), A		; set (TextAbil) to Inner Focus ID
	ld A, stFlinching
	ld (TextStat), A		; set (TextStat) to flinching ID
	rTextbox(tPKMNAbilPrevStat)	; disp 'PKMN's Inner Focus prevents flinching!'
	jr Attack_NotFlinch		; else disp message, continue
	
Attack_NotFocus:
	rTextbox(tPKMNFlinched)	; if flinched, disp message
	call PKMNInterrupt2		; interrupt locked moves of (AtkPKMN)
	jp Attack_ReturnSolar	; skip turn (w/o resetting solarbeam / sky attack)
	
Attack_NotFlinch:
Attack_Disable:
	ld BC, PKMN_DisabledMove
	call Attack_DisableSub	; if atk move is disabled move, then set atk move to struggle
	
Attack_NotDisabled:
Attack_Tormented:
	ld A, (AtkMove)
	cp mRollout
	jr z, Attack_NotTormented	; rollout is carried out
	
	ld BC, PKMN_TormentMove
	call Attack_DisableSub	; if atk move is tormented move, then set atk move to struggle
	
Attack_NotTormented:
Attack_Confusion:
	ld HL, (AtkPKMN)
	ld BC, PKMN_Confusion
	add HL, BC
	ld A, (HL)
	or A				; confused if A > 0
	jr z, Attack_NotConfused
	cp 1
	jr z, Attack_SnapConfusion
	
	dec A				; dec counter
	ld (HL), A
	rTextbox(tPKMNIsConfused)
	ld B, 255
	call iRandom			; 50/50 chance of hitting self
	bit 0, A
	jr z, Attack_ConfusionCont
	
	rTextbox(tItHurtItselfInItsConfusion)
	ld HL, (AtkPKMN)
	ld (TargetPKMN), HL		; temp set target as self
	ld A, mHitInConfusion
	ld (AtkMove), A
	di
	RCall( cCalcDamage )
	ei
	RCall( cDoDamageFlip )
	call c, Turn_Fainted
	ret c
	call PKMNInterrupt		; interrupt locked moves
	jp Attack_Return
	
Attack_SnapConfusion:
	ld (HL), 0			; reset counter
	rTextbox(tPKMNSnappedOutOfConfusion)
	;jr Attack_ConfusionCont
	
Attack_ConfusionCont:
	
Attack_NotConfused:
Attack_Infatuation:
	ld HL, (AtkPKMN)
	push HL
	ld BC, PKMN_Infatuation
	add HL, BC
	ld A, (HL)
	pop HL
	or A				; infatuation if 1
	jr z, Attack_NotInfatuated
	
	rTextbox(tPKMNIsInLoveWith)
	ld B, 255
	call iRandom
	bit 0, A			; 50/50 chance of immobilization
	jr z, Attack_InfatuationCont
	rTextbox(tPKMNIsImmobilizedByLove)
	call PKMNInterrupt2		; interrupt locked moves
	jp Attack_Return
	
Attack_InfatuationCont:
Attack_NotInfatuated:
Attack_Paralysis:
	ld HL, (AtkPKMN)
	call GetPKMNStatus		; A = status
	cp stPAR			; if pkmn paralyzed
	jr nz, Attack_NotPAR
	ld B, 100
	call iRandom
	cp 25				; 25% chance of paralysis
	jr nc, Attack_PARCont	; 75% chance of attacking
Attack_Paralyzed:
	rTextbox(tPKMNIsParalyzedCantMove)
	call PKMNInterrupt		; interrupt locked moves
	jp Attack_Return
	
Attack_NotPAR:
Attack_PARCont:
Attack_Continue:
Attack_Stall:
	call IsStallMove		; if using stall move
	jr c, Attack_NotStall
	ld HL, (AtkPKMN)
	call GetPKMNStall
	jr nz, Attack_NotStall	; if 2nd turn, then use move
	jp Attack_Effect		; else skip to attack effect
	
Attack_NotStall:
Attack_FocusPunch:
	ld A, (AtkMove)
	cp mFocusPunch		; if using focus punch
	jr nz, Attack_NotFocusPunch
	ld HL, (AtkPKMN)
	ld BC, PKMN_Focus
	add HL, BC
	ld A, (HL)
	or A				; and lost focus
	jr nz, Attack_NotLoseFocus
	rTextbox(tPKMNLostFocus)	; then disp msg, and end attack
	jp Attack_End
	
Attack_NotFocusPunch:
Attack_NotLoseFocus:
Attack_SolarBeam:
	ld A, (AtkMove)
	cp mSolarbeam
	jr nz, Attack_NotSolarbeam	; if move is solar beam
	
	ld HL, (AtkPKMN)
	ld BC, PKMN_Solar
	add HL, BC
	ld A, (HL)			; A = solar beam status
	or A
	jr nz, Attack_UseMove	; if charged, then use move
	
	call ChkCloudNine		; nc = pkmn has cloud nine, saves HL
	jr nc, Attack_SolarBeamNoWeather
	
	ld A, (Weather)
	cp wSun
	jr z, Attack_UseMove		; if sunny, then use move
	
Attack_SolarBeamNoWeather:
	ld (HL), 1			; else, charge solarbeam
	rTextbox(tPKMNTookInSunlight); disp message
	jp Attack_ReturnSolar	; and return
	
Attack_NotSolarbeam:			; assumes A = (AtkMove)
	cp mSkyAttack
	jr nz, Attack_NotSkyAttack	; if move is sky attack
	
	ld HL, (AtkPKMN)
	ld BC, PKMN_SkyAttack
	add HL, BC
	ld A, (HL)			; A = sky attack status
	or A
	jr nz, Attack_UseMove	; if charged, use move
	
	inc A				; A = 0 + 1
	ld (HL), A			; else, charge sky atack
	rTextbox(tPKMNIsGlowing)	; disp message
	jp Attack_ReturnSolar
	
Attack_NotSkyAttack:
	
; ---------------------------------------------------------------
; Attack_UseMove
; ---------------------------------------------------------------

Attack_UseMove:
	call Attack_PP		; deduct 1 PP from attack, select struggle if PP = 0
	
	rTextbox(tPKMNUsedMove)	; display 'PKMN used MOVE' text
	
	ld HL, (TargetPKMN)
	call GetPKMNAbility		; A = target pkmn ability
	cp aDamp
	jr nz, Attack_NotDamp	; if target pkmn ability is damp
	
	ld A, (AtkMove)
	cp mExplosion
	jr nz, Attack_NotDamp	; and atk move is explosion
	
	rTextbox(tDampPreventsMove)	; then disp damp message
	jp Attack_Return		; return w/o dealing damage / effects
	
Attack_NotDamp:
	call Attack_Explosion	; destroys user if attack is explosion
	call Attack_BrickBreak	; brick break has effect regardless of ghost / miss
	call Attack_Torment		; if torment in effect, then disable current move
	
	ld A, (AtkMove)		; if attack hits twice
	RCall( cGetMoveEffect )
	cp qHitTwice
	jr nz, Attack_NotHitTwice
	ld A, 2
	ld (HitRepeat), A		; then set HitRepeat to 2
	
Attack_NotHitTwice:
	ld A, (AtkMove)
	cp mDreamEater
	jr nz, Attack_NotDreamEater	; if attack is dream eater
	ld HL, (TargetPKMN)
	call GetPKMNSub
	jr nz, Attack_ButItFailed	; fail if target has sub
	call GetPKMNStatus
	cp stSLP
	jr z, Attack_Asleep		; and target is not asleep
Attack_ButItFailed:
	call AttackEffect_ButItFailed
	jp Attack_Return		; then fail and return
	
Attack_NotDreamEater:
Attack_Asleep:
	ld A, (AtkMove)
	cp mFakeOut
	jr nz, Attack_NotFakeOut	; if attack is fake out
	call GetLastMoveHL		; and last move of atk pkmn == none
	jr z, Attack_FakeOutFirst	; then attack hits
	jp Attack_Effect		; else end, and disp fail message
	
Attack_NotFakeOut:
Attack_FakeOutFirst:
	ld A, (AtkMove)
	cp mSleepTalk			; if attack is NOT sleep talk
	jr nz, Attack_NotSleepTalk	; then skip
	ld HL, (AtkPKMN)
	call GetPKMNStatus
	cp stSLP			; if status is NOT sleep
	jr nz, Attack_ButItFailed	; then disp fail msg, and return
	
Attack_SleepTalkRand:
	ld B, 4
	call iRandom			; 0 <= A < 4
	ld D, 0
	ld E, A			; DE = zero extended A
	ld HL, (AtkPKMN)
	ld BC, PKMN_Moves
	add HL, BC			; HL = pt to move array (4 elements, 1 bytes / element)
	add HL, DE			; HL = pt to random move (1 byte)
	ld A, (HL)			; A = random move
	cp mSleepTalk			; if move = sleep talk
	jr z, Attack_SleepTalkRand	; then go back and pick another move
	ld (AtkMove), A		; else, set (AtkMove) to random move
	jp Attack_UseMove		; go back to Attack_UseMove
	
Attack_NotSleepTalk:			; assumes A = (AtkMove)
	cp mMetronome			; if attack is NOT metronome
	jr nz, Attack_NotMetronome	; then skip
	
Attack_Metronome:
	ld B, 180			; else
	call iRandom			; A = rand move btwn acid armor and zap cannon, 0 <= A < 180
	
	ld HL, AE_SpecMetronomeBan	; HL = pt to start of forbidden move list
	ld B, 10			; B = 10, number of moves on list
	
Attack_MetronomeLp:
	cp (HL)			; compare A and (HL)
	jr z, Attack_Metronome	; re-choose move if A is on list
	inc HL				; move HL to next move
	djnz Attack_MetronomeLp	; loop until B = 0
	ld (AtkMove), A		; set (AtkMove) to random move A
	jp Attack_Continue		; attack using random move
	
Attack_NotMetronome:
	ld HL, (TargetPKMN)
	call GetPKMNAbility		; A = target pkmn ability
	cp aSoundproof
	jr nz, Attack_NotSound	; if ability is soundproof
	ld (TextAbil), A		; set (TextAbil) to aSoundproof
	
	ld A, (AtkMove)
	ld (TextMove), A		; set (TextMove) to atk move
	RCall( cGetMoveSound )	; nz = sound move, preserves A
	jr z, Attack_NotSound	; and move is sound based
	
	rTextbox(tPKMNAbilBlocksMove); disp 'block' message, preserves A
	
	cp mPerishSong		; if move is perish song, then perish song affects attacking pkmn
	call z, AttackEffect_VolStatusPerishAtk
	; TODO: if soundproof pkmn uses baton pass, then perish song affects recipient
	
	jp Attack_Return		; return, skip dodamage / attack effects
	
Attack_NotSound:
	ld HL, (TargetPKMN)
	call GetPKMNAbility		; A = target pkmn ability
	cp aSturdy
	jr nz, Attack_NotSturdy	; if ability is sturdy
	ld (TextAbil), A		; set (TextAbil) to aSturdy
	
	ld A, (AtkMove)
	RCall( cGetMoveEffect )	; A = atk move effect
	cp qOHKO
	jr nz, Attack_NotSturdy	; and move is OHKO move
	
	ld A, stOHKOs
	ld (TextStat), A		; set (TextStat) to stOHKOs
	rTextbox(tTrgtAbilPrevStat)	; disp 'prev' message
	
	jp Attack_Return		; return, skip dodamage / attack effects
	
Attack_NotSturdy:
	RCall( cCalcMiss )		; calculates whether attack is hit / miss
	jr c, Attack_Hit
Attack_Miss:
	ld HL, (AtkPKMN)
	ld BC, PKMN_Stall
	add HL, BC
	ld (HL), 0			; set stall to 0
	call PKMNInterrupt2		; interrupt locked moves
	
	ld HL, LevitateAct
	ld A, (HL)
	ld (HL), 0			; unconditionally reset levitateact
	or A				; if levitate activated
	jr nz, Attack_MissLevitate	; then display levitate message, return
	
	rTextbox(tPKMNAttackMissed)	; display 'PKMN MOVE missed' text
	; TODO: if using future sight, then attack fails, not miss
	
Attack_MissReturn:
	ld A, (AtkMove)
	cp mHiJumpKick
	jp z, AttackEffect		; jump to effect if high jump kick
	
	RCall( cGetMoveEffect )	; A = move effect ID
	cp qDuration
	jp z, AttackEffect		; jump to effect if duration move (petal dance, thrash)
	
	jp Attack_ReturnFNT		; else return
	
Attack_MissLevitate:
	rTextbox(tPKMNMakesGroundMovesMissWithLevitate)
	jr Attack_MissReturn
	
Attack_Hit:
	ld A, (AtkMove)
	cp mFutureSight
	jp z, Attack_Effect		; jump to effect if future sight
	
	; no damage if using water | elec move and target has water | volt absorb
	call Attack_Absorb		; return z if water | volt absorb activated
	jp z, Attack_Return		; return, skip dodamage / attack effects
	
Attack_NotAbsorb:
	; no damage if using fire move and target has flash fire
	call Attack_FlashFire	; return z if flash fire activated
	jp z, Attack_Return
	
Attack_NotFlashFire:
	di
	RCall( cCalcDamage )
	ei
	or A				; if move does damage (A = 0)
	jr z, Attack_Dam		; then do damage
	dec A				; if move has no pow (A = 1)
	jp z, Attack_Effect		; deal effects
	
Attack_NoAffect:			; else, move has no effect (A = 2)
	ld A, (AtkMove)
	RCall( cGetMoveProtect )	; if move protect flag is not set
	jp z, Attack_Effect		; then go to effect
	call Attack_TypeEff
	jr Attack_MissReturn		; else return
	
Attack_Dam:
	; (Damage) = min(Damage, current HP)
	call Attack_DamClamp
	
	ld A, (AtkMove)
	call Attack_IsTrgtProtect	; if damaging move && protect, then protect against hit
	jr c, Attack_MissReturn
	
	call IsTargetSub		; nz = whether target has sub
	jr z, Attack_DoDamage	; then do damage, no sub
	
	ex DE, HL			; DE = damage
	ld H, 0
	ld L, A			; HL = zero extended A, HP of substitute
	sbc HL, DE			; c flag cleared by subroutine
	ex DE, HL			; DE = sub HP - damage
	push BC \ pop HL		; HL = pt to sub status
	jr z, Attack_SubBroke
	jr c, Attack_SubBroke
	ld (HL), E			; set new sub HP
	
	rTextbox(tTheSubstituteTookDamageForPKMN)
	
	jr Attack_DamFocusCont	; skip do damage, endure / focus band, losing focus
	
Attack_SubBroke:
	ld (HL), 0			; set sub status to 0
	call DispPKMNSprite		; disp front / back sprite of target pkmn
	
	rTextbox(tSubstituteFaded)
	
	jr Attack_DamFocusCont	; skip do damage, endure / focus band, losing focus
	
Attack_DoDamage:
	RCall( cDoDamage )
	
	call Attack_EndureText
	call Attack_FocusBandText
	
	bit PKMNFlag, (IY + Asm_Flag1)
	jr z, Attack_DamOppLoseFocus
Attack_DamPlayerLoseFocus:
	xor A				; player loses focus if hit
	ld (PlayerPKMN + PKMN_Focus), A
	jr Attack_DamFocusCont
	
Attack_DamOppLoseFocus:
	xor A				; opp loses focus if hit
	ld (OppPKMN + PKMN_Focus), A
Attack_DamFocusCont:
	
	call Attack_CritHit
	
	; King's Rock - attacks have 10% chance to flinch target
	call Attack_KingsRock
	
	; Contact abilities - 30% of inflicting status on attacker
	call Attack_ContactAbil
	
	; Hit repeat
	ld HL, HitRepeat
	xor A
	or (HL)
	jr z, Attack_NoRep		; skip if HitRepeat is 0
	dec (HL)			; otherwise, dec HitRepeat counter
	
	ld DE, (PlayerFainted)	; player, opp fainted are continguous in memory
	ld A, E
	or D
	jr nz, Attack_NoRep		; skip if either player or opp has fainted
	
	ld A, (HL)
	or A
	jp nz, Attack_Hit		; hit again if HitRepeat is nonzero
	
Attack_NoRep:
	call Attack_TypeEff
	
	call Attack_DestinyBond	; return c if destiny bond activated
	jr c, Attack_Fainted2	; if both pkmn FNTed, then skip move effects, white herb, shell bell
	call Attack_Grudge
Attack_NoDam:
	
Attack_End:
	ld HL, (PlayerFainted)	; player, opp fainted bools are continuous in memory
	ld A, L
	or H				; if H, L are both 0
	jr z, Attack_NotFainted	; then no pkmn fainted
	
Attack_Fainted:			; else, fall through to fainted
	ld A, (AtkMove)
	RCall( cGetMoveFNT )		; if move FNT flag bit is not set
	call z, AttackEffect		; then process attack effects
	call Attack_WhiteHerbSub	; check for white herb activation
	call Attack_ShellBell	; Shell Bell - attack restores 1/8 of damage dealt
	
Attack_Fainted2:
	call ResetSolarbeam
	call SetLastMove
	scf
	ret
	
Attack_NotFainted:

Attack_Effect:
	call AttackEffects		; determine attack effects, white herb, shell bell
	
Attack_ReturnFNT:
	ld HL, PKMNFNT		; if PKMNFNT
	ld A, (HL)
	ld (HL), 0			; unconditionally reset PKMNFNT
	or A
	jr nz, Attack_Fainted2	; then go to attack_fainted, return w/ c set
	
Attack_Return:
	call ResetSolarbeam
Attack_ReturnSolar:			; return without resetting solarbeam / sky attack
	call SetLastMove
	scf				; return w/ flag c unset if not fainted
	ccf
	ret
	
; ---------------------------------------------------------------
; subroutines
; ---------------------------------------------------------------

Attack_CritHit:			; subroutine - displays whether attack is crit hit
	ld HL, CritBool
	ld A, (HL)
	or A
	ret z				; ret if no critical hit
	xor A
	ld (HL), A			; else, reset critical hit status
	
	call Attack_IsLevelDmg
	ret z				; ret if attack is level damage move
	
	rTextboxRet(tACriticalHit)	; display 'A critical hit' text

Attack_TypeEff:			; subroutine - displays type effectiveness
	call Attack_IsLevelDmg
	ret z				; ret if attack is level damage move
	
	ld A, (TypeEff)
	or A				; 0 -> not affected
	jr z, Attack_NotAffect
	cp 4				; 1 | 2 -> not very eff
	jr c, Attack_NotVeryEff
	ret z				; 4 -> effective
	jr Attack_SuperEff		; else (8 | 16) -> super eff
	
Attack_NotAffect:
	rTextboxRet(tItDoesntAffectPKMN)
	
Attack_NotVeryEff:
	rTextboxRet(tItsNotVeryEffective)
	
Attack_SuperEff:
	rTextboxRet(tItsSuperEffective)
	
Attack_IsLevelDmg:			; subroutine - checks if atk move is level damage move, input: (AtkMove), output: z = is level damage
	ld A, (AtkMove)
	RCall( cGetMoveEffect )
	cp qLevelDmg
	ret
	
; ---------------------------------------------------------------
; AttackEffects
; Does attack effect, white herb, and shell bell
; ---------------------------------------------------------------

AttackEffects:
	call AttackEffect		; determine attack effects
	call Attack_WhiteHerbSub	; check for white herb activation
	jr Attack_ShellBell		; Shell Bell - attack restores 1/8 of damage dealt
	
; ---------------------------------------------------------------
; White Herb 
; If any stat stage < 0, all stats < 0 become 0, item depleted
; ---------------------------------------------------------------

Attack_WhiteHerbSub:			; subroutine for white herb
	ld BC, Attack_WhiteHerb
	jp Turn_Order			; Turn_Priority is random
	
Attack_WhiteHerb:			; HL = pt to detailed data
	push HL
	push DE
	call Attack_WhiteHerb2	; call main part of routine
	pop DE
	pop HL
	or A				; reset c flag
	ret 				; ret w/ nc
	
Attack_WhiteHerb2:
	call GetPKMNItem		; A = pkmn item, HL, DE preserved
	cp iWhiteHerb
	ret nz				; ret if not holding white herb
	
	ld BC, PKMN_AtkBoost
	add HL, BC			; HL = pt to atk boost
	ld D, H			; DE = pt to atk boost
	ld E, L
	ld B, 7			; 7 diff stat boosts
Attack_WhiteHerbLp:			; loop - iterates 7 times
	ld A, (HL)			; A = stat boost
	cp 128				; if A > 128 (negative unsigned int, < 0)
	jr nc, Attack_WhiteHerbNeg	; then jump to white herb activation
	inc HL
	djnz Attack_WhiteHerbLp
	ret				; ret w/ nc if no stat boosts < 0
	
Attack_WhiteHerbNeg:
	ld H, D			; HL = pt to atk boost
	ld L, E
	ld B, 7			; 7 diff stat boosts
Attack_WhiteHerbLp2:
	ld A, (HL)			; A = stat boost
	cp 128				; if A < 128 (positive unsigned int, >= 0)
	jr c, Attack_WhiteHerbPos	; then skip
	xor A
	ld (HL), A			; else stat boost = 0
	
Attack_WhiteHerbPos:
	inc HL				; ascending from atk boost
	djnz Attack_WhiteHerbLp2
	
	ld BC, PKMN_EvaBoost + 1 - PKMN_Item
	or A				; reset c flag
	sbc HL, BC			; HL = pt to item
	ld (HL), iNoItem		; deplete white herb
	
	rTextboxRet(tPKMNWhiteHerb)	; disp msg
	
; ---------------------------------------------------------------
; Kings Rock
; Attacks have 10% chance to flinch target
; ---------------------------------------------------------------

Attack_KingsRock:
	ld HL, (AtkPKMN)
	call GetPKMNItem		; A = pkmn item
	cp iKingsRock
	ret nz				; ret if item is not kings rock
	ld A, (AtkMove)
	RCall( cGetMoveFlinch )	; get move flinch flag
	ret z				; ret if flinch bit is zero
	ld B, 100
	call iRandom			; 0 <= A < 100
	cp 10				; 10% chance to flinch
	ret nc
	ld HL, (TargetPKMN)
	ld BC, PKMN_Flinch
	add HL, BC
	ld (HL), 1			; set flinch
	ret
	
; ---------------------------------------------------------------
; Shell Bell
; Restores 1/8 of damage as HP
; ---------------------------------------------------------------

Attack_ShellBell:
	ld HL, (AtkPKMN)
	call GetPKMNItem		; A = pkmn item
	cp iShellBell
	ret nz				; if holding shell bell
	
	ld (TextItem), A		; set (TextItem) to shell bell ID
	call ChkFullHP
	ret nc				; and not at full HP or 0 HP
	ld HL, (Damage)
	ld A, H
	or L
	ret z				; and not (Damage = 0)
	call SetAtkFlag		; heal atk pkmn
	rTextbox(tPKMNShellBell)	; then disp msg
	call Calc8			; HL = HL / 8
	RCall( cDoHeal )		; restore 1/8 of damage as health
	ret
	
; ---------------------------------------------------------------
; Attack_ContactAbil
; If contact move, and target has contact abil, then 30% status
; ---------------------------------------------------------------

Attack_ContactAbil:
	ld A, (AtkMove)
	RCall( cGetMoveContact )
	ret z				; ret if not contact move
	
	ld B, 100
	call iRandom			; 0 <= A < 100
	cp 30				; if A >= 30
	ret nc				; then ret, 30% prb not activated
	
	call FlipPKMNFlag		; target is user
	call Attack_ContactAbilCalc	; calculate abil effects
	jp FlipPKMNFlag		; restore pkmn flag
	
Attack_ContactAbilCalc:
	ld HL, (TargetPKMN)
	call GetPKMNAbility		; A = target pkmn ability
	ld D, A			; D = target pkmn ability
	ld HL, (AtkPKMN)		; HL = pt to atk pkmn
	call GetPKMNStatus		; z = pkmn status is stNone, HL, DE preserved
	ret nz				; if status not stNone, then ret
	ld A, D			; restore A = target pkmn ability
	
	cp aCuteCharm
	jr z, Attack_ContactAbilCuteCharm
	cp aFlameBody
	jr z, Attack_ContactAbilFlameBody
	cp aPoisonPoint
	jr z, Attack_ContactAbilPoisonPoint
	cp aStatic
	jr z, Attack_ContactAbilStatic
	cp aEffectSpore
	ret nz
	
Attack_ContactAbilEffectSpore:
	ld B, 100
	call iRandom			; 0 <= A < 100
	
	cp 30				; 30% prb poison (9 / 30)
	jr c, Attack_ContactAbilSporePoison
	cp 63				; 33% prb paralysis (10 / 30)
	jr c, Attack_ContactAbilSporeParalyze
	ld A, qSleep			; else, 36% prb sleep (11 / 30)
	
Attack_ContactEffectCall:
	jp AttackEffect_StatusCall	; HL = pt to pkmn data
	
Attack_ContactAbilStatic:
Attack_ContactAbilSporeParalyze:
	ld A, qParalyze
	jr Attack_ContactEffectCall	; jp AE_StatusParalyze
	
Attack_ContactAbilCuteCharm:
	ld A, qInfatuate
	jr Attack_ContactEffectCall	; jp AE_VolInfatuation
	
Attack_ContactAbilFlameBody:
	ld A, qBurn
	jr Attack_ContactEffectCall	; jp AE_StatusBurn
	
Attack_ContactAbilPoisonPoint:
Attack_ContactAbilSporePoison:
	ld A, qPoison
	jr Attack_ContactEffectCall	; jp AE_StatusPoison
	
	; TODO: text is 'pkmn's ability status'ed trgtpkmn'
	; TODO: status abil prevented by safeguard (?), substitute (?)
	
; ---------------------------------------------------------------
; Attack_PP
; Deducts 1 PP from current attack
; ---------------------------------------------------------------

Attack_PP:
	call Attack_PPGet		; get pt to PP of move
	ret nz				; ret if no move found
	
	cp (HL)			; if no PP (A = 0 = (HL))
	jr z, Attack_PPStruggle	; then struggle
	dec (HL)			; else, dec PP by 1
	
	push HL			; save pt to PP
	ld HL, (TargetPKMN)		; HL = pt to target pkmn
	call GetPKMNAbility		; A = pkmn ability, HL, DE preserved
	pop HL				; get pt to PP
	cp aPressure			; if ability is not pressure
	ret nz				; then return
	
	xor A				; A = 0
	cp (HL)			; if no PP
	jr z, Attack_PPStruggle	; then struggle
	dec (HL)			; else, dec PP by 1 again
	ret
	
Attack_PPStruggle:
	ld A, mStruggle
	ld (DE), A			; set attacking move to struggle
	ret
	
Attack_PPGet:				; subroutine - input: (AtkMove), output: z = move found, A = 0, DE = pt to AtkMove, HL = pt to PP
	ld DE, AtkMove
	ld A, (DE)			; A = current move
	
Attack_PPGet2:			; input: A = move
	ld HL, (AtkPKMN)
	ld BC, PKMN_Moves
	add HL, BC			; HL = pt to pkmn moves array
	ld B, 4			; 4 moves in array
Attack_PPLp:
	cp (HL)			; if move matches
	jr z, Attack_PPDeduct	; then break out of loop
	inc HL				; else, inc HL to next move
	djnz Attack_PPLp		; loop until B = 0
	ret				; if no match, then ret w/ nz
	
Attack_PPDeduct:			; deducts 1 PP from move
	ld BC, PKMN_PP - PKMN_Moves
	add HL, BC			; HL = pt to corresponding PP value
	xor A				; A = 0
	ret				; ret w/ z
	
; ---------------------------------------------------------------
; Attack_DamClamp
; Clamp damage value to be less than target pkmn's current HP
; ---------------------------------------------------------------

Attack_DamClamp:			; subroutine - clamp damage value, input: (TargetPKMN), (Damage), output: HL = (Damage) = clamped damgage
	ld HL, (TargetPKMN)
	call GetPKMNHP
	ex DE, HL			; DE = target pkmn current HP
	ld HL, (Damage)		; HL = damage
	call CpHLDE
	ret c				; if HL < DE (damage < HP), then skip
	ex DE, HL			; otherwise
	ld (Damage), HL		; set (Damage) = HL = HP
	ret
	
; ---------------------------------------------------------------
; Attack_BrickBreak
; If attack is brick break, and target has screen, then shatter
; ---------------------------------------------------------------

Attack_BrickBreak:
	ld A, (AtkMove)
	cp mBrickBreak		; if attack is brick break
	ret nz
	
	call GetLight			; HL = pt to light screen, reflect status of target
	ld A, (HL)			; A = light screen status
	or A
	inc HL				; doesn't affect flags
	jr nz, Attack_BrickLight
	ld A, (HL)			; A = reflect status
	or A
	jr nz, Attack_BrickReflect
	ret				; skip if neither light screen nor reflect
	
Attack_BrickLight:
Attack_BrickReflect:			; HL = pt to reflect status
	ld (HL), 0
	dec HL
	ld (HL), 0			; reset light screen / reflect status
	rTextboxRet(tWallShattered)	; disp wall shatter message, continue on w/ attack
	
; ---------------------------------------------------------------
; Attack_Explosion
; If attack is explosion, then clear HP bar, set user HP to 0
; ---------------------------------------------------------------

Attack_Explosion:
	ld A, (AtkMove)		; if move is explosion
	cp mExplosion
	ret nz
	
	ld HL, (AtkPKMN)
	
Attack_ExplosionSub:			; subroutine - HL = pt to pkmn, 
	push HL
	ld BC, PKMN_HP
	add HL, BC			; HL = pt to current HP (2 bytes)
	xor A
	ld (HL), A
	inc HL
	ld (HL), A			; set both bytes at (HL) to $00
	pop DE				; DE = pt to atk pkmn data
	bit PKMNFlag, (IY + Asm_Flag1)
	jr nz, Attack_ExplosionOpp
Attack_ExplosionPlayer:		; if z, then player is user
	ld A, 1			; A = 1 for player
	ld (PlayerFainted), A	; set (PlayerFainted)
	jr Attack_ExplosionCont
	
Attack_ExplosionOpp:			; else if nz, then opp is user
	ld A, 1
	ld (OppFainted), A		; set (OppFainted)
	inc A				; A = 2 for opp

Attack_ExplosionCont:		; A = 1 | 2
	ld (PKMNFNT), A
	RCall( cDispHP )		; disp empty HP bar
	ret
	
; ---------------------------------------------------------------
; Attack_EndureText
; If (EndureAct) set, then display endure message
; ---------------------------------------------------------------

Attack_EndureText:
	ld A, (EndureAct)
	or A
	ret z				; if endure flag set
	rTextboxRet(tPKMNEnduredTheHit); disp msg
	
; ---------------------------------------------------------------
; Attack_FocusBandText
; If (FocusBandAct) set, then display focus band message, reset
; ---------------------------------------------------------------

Attack_FocusBandText:
	ld HL, FocusBandAct
	ld A, (HL)
	or A
	ret z				; if focus band flag set
	xor A
	ld (HL), A			; reset focus band activation flag
	rTextboxRet(tPKMNHeldOnFocusBand); disp msg
	
; ---------------------------------------------------------------
; Attack_DestinyBond
; If target fainted, and has destiny set to 1, then user FNTs
; Output: c = destiny bond activated, nc = not
; ---------------------------------------------------------------

Attack_DestinyBond:
	call Attack_DestinyBondGet
	or A				; if A = 0, c flag cleared
	ret z				; then ret w/ nc, no faint
	
	ld A, (DE)			; A = target destiny bond status
	cp 1				; if A != 1
	scf 
	ccf				; reset c flag
	ret nz				; then return w/ nc, no destiny bond on current turn
	
	rTextbox(tPKMNTookPKMNWithIt)
	
	call GetPKMNMaxHP
	RCall( cDoDamageFlip )
	scf				; set c flag
	ret				; ret w/ c
	
Attack_DestinyBondGet:		; subroutine - get A = whether target fainted, DE = pt to target destiny, HL = atk pkmn
	bit PKMNFlag, (IY + Asm_Flag1)
	jr nz, Attack_DestinyBondPlayer
Attack_DestinyBondOpp:		; if target is opp
	ld A, (OppFainted)		; then check opp faint status
	ld DE, OppPKMN + PKMN_Destiny
	ld HL, PlayerPKMN
	ret
	
Attack_DestinyBondPlayer:		; else if target is player
	ld A, (PlayerFainted)	; then check player faint status
	ld DE, PlayerPKMN + PKMN_Destiny
	ld HL, OppPKMN
	ret
	
; ---------------------------------------------------------------
; Attack_Grudge
; If target fainted, and has grudge set to 1, then user loses PP
; ---------------------------------------------------------------

Attack_Grudge:
	call Attack_DestinyBondGet
	or A				; if A = 0
	ret z				; then ret, no faint
	
	inc DE				; Grudge - Destiny = 1
	ld A, (DE)			; A = target grudge status
	dec A				; if A != 1
	ret nz				; then ret
	
	call Attack_GrudgeGetMove	; A = move of atk pkmn (metronome / sleep talk included)
	call Attack_PPGet2		; HL = pt to PP of move, A = 0
	ret nz				; ret if move not found
	
	ld (HL), A			; set PP of move to 0
	rTextboxRet(tPKMNLostPP)	; disp msg and ret
	
Attack_GrudgeGetMove:		; subroutine - get atk pkmn move, input: TurnFlag, output: A = move
	bit TurnFlag, (IY + Asm_Flag1)
	jr nz, Attack_GrudgeGetMovePlayer
	
Attack_GrudgeGetMoveOpp:
	ld A, (CurMoveOpp)
	ret
	
Attack_GrudgeGetMovePlayer:
	ld A, (CurMovePlayer)
	ret
	
; ---------------------------------------------------------------
; Attack_Torment
; If torment set to 1 or torment move is not None, then block move
; ---------------------------------------------------------------
	
Attack_Torment:
	ld HL, (AtkPKMN)
	ld BC, PKMN_Torment
	add HL, BC
	ld A, (HL)
	inc HL				; PKMN_TormentMove - PKMN_Torment = 1
	dec A				; if torment status is 1, then effect applies
	jr z, Attack_TormentEffect
	ld A, (HL)
	inc A				; z flag set if A = $FF
	ret z				; if torment move is not None, then effect applies
	
Attack_TormentEffect:
	ld A, (AtkMove)
	cp mStruggle			; don't get original move if struggle used
	call nz, Attack_GrudgeGetMove ; A = attacking move (metronome / sleep talk included), HL preserved
	ld (HL), A			; set torment move to current attacking move
	ret
	
; ---------------------------------------------------------------
; Attack_Absorb
; Heals target if water | volt absorb and move is water | elec
; Output: z = move absorbed, nz = not absorbed
; ---------------------------------------------------------------

Attack_Absorb:
	ld A, (AtkMove)
	cp mThunderWave
	jr nz, Attack_AbsorbNotTW	; if AtkMove is thunder wave
	inc A				; A = mThunderWave + 1, reset z flag
	ret				; ret w/ nz, TW not absorbed
	
Attack_AbsorbNotTW:
	RCall( cGetMoveType )	; A = atk move type
	ld D, aVoltAbsorb
	cp eElectric			; if move is electric type
	jr z, Attack_VoltAbsorb	; check for volt absorb
	ld D, aWaterAbsorb
	cp eWater			; if move is water type
	jr z, Attack_WaterAbsorb	; check for water absorb
	ret				; else skip, ret w/ nz
	
Attack_VoltAbsorb:
Attack_WaterAbsorb:
	ld HL, (TargetPKMN)
	call GetPKMNAbility		; A = target pkmn ability, HL, DE preserved
	cp D
	ret nz				; and target ability is volt | water absorb
	ld (TextAbil), A		; set (TextAbil) to aVoltAbsorb | aWaterAbsorb
	
	call ChkFullHP		; return nc if HP >= max HP, preserves HL
	jr nc, Attack_AbsorbRet	; don't heal if at max HP
	
	call GetFourthMaxHP		; HL = 1/4 of target max HP
	RCall( cDoHeal )		; heal target pkmn
	
Attack_AbsorbRet:
	rTextbox(tAbsorbMadeMoveUseless)
	xor A				; A = 0, set z flag
	ret				; ret w/ z, move successfully absorbed
	
; ---------------------------------------------------------------
; Attack_FlashFire
; Activates flash fire if attacking move is fire type
; Output: z = flash fire, nz = not flash fire
; ---------------------------------------------------------------

Attack_FlashFire:
	ld A, (AtkMove)
	cp mSunnyDay			; if move is sunny day
	jr nz, Attack_FlashFireNotSun
	inc A
	ret nz				; then ret w/ nz, flash fire not activated
	
Attack_FlashFireNotSun:
	RCall( cGetMoveType )	; A = atk move type
	cp eFire			; if move is not fire type
	ret nz				; then skip, ret w/ nz
	
	ld HL, (TargetPKMN)
	call GetPKMNAbility		; A = target pkmn ability, HL preserved
	cp aFlashFire			; if ability is not flash fire
	ret nz				; then skip, ret w/ nz
	
	ld (TextAbil), A		; set (TextAbil) to aFlashFire
	rTextbox(tFlashFireRaisedPow)	; saves all regs
	ld BC, PKMN_FlashFire
	add HL, BC
	ld (HL), 1			; set flash fire status to 1, activated
	xor A				; A = 0, set z flag
	ret				; ret w/ z
	
	; TODO: if pkmn if frozen and hit w/ fire move, then defrosted, takes damage, no flash fire activation
	
; ---------------------------------------------------------------
; SetLastMove
; Sets last move, and last damage, misc end of attack changes
; ---------------------------------------------------------------

SetLastMove:
	ld HL, (AtkPKMN)		; record move as last move
	ld A, (AtkMove)
	cp $FC
	jr z, SetLastMoveSkipCharge	; skip if last move is charge, last move should be hyper beam
	cp mStruggle
	jr z, SetLastMoveSkipStruggle
	ld BC, PKMN_LastMove
	add HL, BC
	ld (HL), A
	
SetLastMoveSkipCharge:
SetLastMoveSkipStruggle:
	ld HL, (Damage)		; copy (Damage) to (LastDmg)
	ld (LastDmg), HL
	
	ld HL, (AtkPKMN)
	call GetPKMNItem		; A = atk pkmn item
	ld BC, PKMN_Choice		; BC = Choice offset
	add HL, BC			; HL = pt to choice band status
	cp iChoiceBand		; if item not choice band
	ld (HL), 0			; choice band not activated by default
	jr nz, Attack_NotChoiceBand	; then skip
	ld (HL), 1			; activate choice band
	
Attack_NotChoiceBand:
	ld HL, TypeEff		; reset type effectiveness to 4
	ld (HL), 4
	
	xor A
	ld HL, FocusBandAct		; reset focus band activation status
	ld (HL), A
	ld HL, EndureAct		; reset endure activation status
	ld (HL), A
	ld HL, HitRepeat
	ld (HL), A			; reset hit repeat
	ld HL, PKMNFNT
	ld (HL), A			; reset PKMNFNT (fixes skip turn after FNT from double edge bug)
	ret
	
; ---------------------------------------------------------------
; ResetSolarbeam
; Resets solarbeam status
; ---------------------------------------------------------------

ResetSolarbeam:
	xor A				; A = 0
	ld HL, (AtkPKMN)
	ld BC, PKMN_Solar
	add HL, BC
	ld (HL), A			; unconditionally reset solarbeam status
	ld BC, PKMN_SkyAttack - PKMN_Solar
	add HL, BC
	ld (HL), A			; unconditionally reset sky attack status
	ret
	
; ---------------------------------------------------------------
; Attack_DisableSub
; If (AtkMove) matches move at BC, then set (AtkMove) to struggle
; ---------------------------------------------------------------

Attack_DisableSub:			; subroutine - if (AtkMove) matches move at BC offset, then set (AtkMove) to struggle, input: BC = offset
	ld HL, (AtkPKMN)
	add HL, BC
	ld A, (HL)
	cp $FF
	ret z
	
	ld HL, AtkMove
	ld B, (HL)
	cp B
	ret nz				; disable specific move
	
	ld A, mStruggle
	ld (HL), A			; set AtkMove to mStruggle
	ret
	
; ---------------------------------------------------------------
; Attack_IsTrgtProtect
; Determine if target used protect, set TrgtProtect accordingly
; Same I/O as IsTargetProtect, destroys A
; ---------------------------------------------------------------

Attack_IsTrgtProtect:
	call IsTargetProtect
	jr c, Attack_IsTrgtProtectYes
	xor A
	ld (TrgtProtect), A		; set (TrgtProtect) to 0, if not protect
	ret
	
Attack_IsTrgtProtectYes:
	ld A, 1
	ld (TrgtProtect), A		; set (TrgtProtect) to 1, if protect
	ret
	