; ===============================================================
; Routines
; ===============================================================

; ---------------------------------------------------------------
; TextboxBlink
; Sets BlinkerOn flag, calls textbox, then resets
; Same I/O as Textbox
; ---------------------------------------------------------------

TextboxBlink:
	set BlinkerOn, (IY + Asm_Flag1)
	RCall( cTextbox )				; disp msg, w/ blinker
	res BlinkerOn, (IY + Asm_Flag1)
	ret
	
; ---------------------------------------------------------------
; GetPKMNStatus
; Gets pkmn status from detailed data
; Input: HL = Pt to detailed data
; Output: A = Status
;         z = Status is zero, nz = otherwise
; Preserves HL, routine must be in executable block
; ---------------------------------------------------------------

GetPKMNStatus:
	ld C, PKMN_Status
GetPKMN:			; subroutine - input: C = pkmn offset, HL = pt to pkmn data, output: A = attribute, HL preserved
	ld B, 0		; BC = zero extended C
	push HL
	add HL, BC
	ld A, (HL)		; A = attribute
	or A			; z if attribute is zero, c flag cleared
	pop HL
	ret

; ---------------------------------------------------------------
; GetPKMNSpecies
; Gets pkmn species from detailed data
; Input: HL = Pt to detailed data
; Output: A = Species ID
; Preserves HL
; ---------------------------------------------------------------

GetPKMNSpecies:
	ld C, PKMN_Species
	jr GetPKMN
	
; ---------------------------------------------------------------
; GetPKMNItem
; Gets pkmn item from detailed data
; Input: HL = Pt to detailed data
; Output: A = Item
; Preserves HL
; ---------------------------------------------------------------

GetPKMNItem:
	ld C, PKMN_Item
	jr GetPKMN

; ---------------------------------------------------------------
; GetPKMNAbility
; Gets pkmn ability from detailed data, or swapped / traced abil
; Input: HL = Pt to detailed data
; Output: A = Ability
; Preserves HL
; ---------------------------------------------------------------

GetPKMNAbility:
	call GetPKMNSkillSwap	; A = swapped ability (0 if none), HL preserved
	or A
	res 7, A			; A = A & ~0x80, clear bit 7, flags preserved
	ret nz				; ret w/ swapped ability if A != 0
	call GetPKMNAbility2		; A = ability, HL preserved
	cp aTrace			; if ability != trace
	ret nz				; then ret
	jr GetPKMNTrace		; else, get A = traced ability
	
; ---------------------------------------------------------------
; GetPKMNAbility2
; Gets pkmn ability from detailed data, disregards traced ability
; Input: HL = Pt to detailed data
; Output: A = Ability
; Preserves HL
; ---------------------------------------------------------------

GetPKMNAbility2:
	ld C, PKMN_Abil
	jr GetPKMN

; ---------------------------------------------------------------
; GetPKMNMaxHP
; Gets pkmn max HP from detailed data
; Input: HL = Pt to detailed data
; Output: HL = max HP
; ---------------------------------------------------------------

GetPKMNMaxHP:
	ld C, PKMN_MaxHP
GetPKMN2:				; subroutine - input: C = pkmn offset, HL = pt to data, output: HL = attribute
	ld B, 0
	add HL, BC
	call LdHLIndRAM
	ret
	
; ---------------------------------------------------------------
; GetPKMNHP
; Gets pkmn current HP from detailed data
; Input: HL = Pt to detailed data
; Output: HL = current HP
; ---------------------------------------------------------------

GetPKMNHP:
	ld C, PKMN_HP
	jr GetPKMN2

; ---------------------------------------------------------------
; GetPKMNSpd
; Gets pkmn speed from detailed data
; Input: HL = Pt to detailed data
; Output: HL = speed stat
; ---------------------------------------------------------------

GetPKMNSpd:
	ld C, PKMN_Spd
	jr GetPKMN2
	
; ---------------------------------------------------------------
; GetPKMNLeech
; Gets pkmn leech status from detailed data
; Input: HL = Pt to detailed data
; Output: A = Leech status
; Preserves HL
; ---------------------------------------------------------------

GetPKMNLeech:
	ld C, PKMN_Leech
	jr GetPKMN
	
; ---------------------------------------------------------------
; GetPKMNEndure
; Gets pkmn endure status from detailed data
; Input: HL = Pt to detailed data
; Output: A = Endure status
; Preserves HL
; ---------------------------------------------------------------

GetPKMNEndure:
	ld C, PKMN_Endure
	jr GetPKMN
	
; ---------------------------------------------------------------
; GetPKMNStall
; Gets pkmn stall status from detailed data
; Input: HL = Pt to detailed data
; Output: A = Status
; Preserves HL
; ---------------------------------------------------------------

GetPKMNStall:
	ld C, PKMN_Stall
	jr GetPKMN
	
; ---------------------------------------------------------------
; GetPKMNTrap
; Gets pkmn trap status from detailed data
; Input: HL = Pt to detailed data
; Output: A = Trap status
;         z = no trap
; Alters HL
; ---------------------------------------------------------------

GetPKMNTrap:
	ld BC, PKMN_Trap
	add HL, BC
	ld A, (HL)
	or A
	ret
	
; ---------------------------------------------------------------
; GetPKMNIngrain
; Gets pkmn ingrain status from detailed data
; Input: HL = Pt to detailed data
; Output: A = Ingrain status
; Preserves HL
; ---------------------------------------------------------------

GetPKMNIngrain:
	ld C, PKMN_Ingrain
	jr GetPKMN
	
; ---------------------------------------------------------------
; GetPKMNFlinch
; Gets pkmn flinch status from detailed data
; Input: HL = Pt to detailed data
; Output: A = Flinch status
; Preserves HL
; ---------------------------------------------------------------

GetPKMNFlinch:
	ld C, PKMN_Flinch
	jr GetPKMN
	
; ---------------------------------------------------------------
; GetPKMNTrace
; Gets pkmn traced ability from detailed data
; Input: HL = Pt to detailed data
; Output: A = Ability ID of traced ability
; Preserves HL
; ---------------------------------------------------------------

GetPKMNTrace:
	ld C, PKMN_Trace
	jr GetPKMN
	
; ---------------------------------------------------------------
; GetPKMNSkillSwap
; Gets pkmn swapped ability from detailed data
; Input: HL = Pt to detailed data
; Output: A = Ability ID of swapped ability (0 if none)
; Preserves HL
; ---------------------------------------------------------------

GetPKMNSkillSwap:
	ld C, PKMN_SkillSwap
	jr GetPKMN
	
; ---------------------------------------------------------------
; GetPKMNSub
; Gets pkmn substitute status from detailed data
; Input: HL = Pt to detailed data
; Output: A = HP of sub (0 if none)
; Preserves HL
; ---------------------------------------------------------------

GetPKMNSub:
	ld C, PKMN_Substitute
	jr GetPKMN
	
; ---------------------------------------------------------------
; GetHalfMaxHP
; Gets 1/2 of pkmn's max HP
; Input: HL = Pt to detailed data
; Output: HL = 1/2 HP
; ---------------------------------------------------------------

GetHalfMaxHP:
	call GetPKMNMaxHP		; HL = max HP
	srl H				; HL = HL / 2
	rr L
	ret

; ---------------------------------------------------------------
; GetFourthMaxHP
; Gets 1/4 of pkmn's max HP
; Input: HL = Pt to detailed data
; Output: HL = 1/4 HP
; ---------------------------------------------------------------

GetFourthMaxHP:
	call GetHalfMaxHP		; HL = max HP / 2
	srl H				; HL = HL / 2
	rr L
	ret

; ---------------------------------------------------------------
; GetEighthMaxHP
; Gets 1/8 of pkmn's max HP
; Input: HL = Pt to detailed data
; Output: HL = 1/8 HP
; ---------------------------------------------------------------

GetEighthMaxHP:
	call GetFourthMaxHP		; HL = max HP / 2 / 2
	srl H				; HL = HL / 2
	rr L
	ret
	
; ---------------------------------------------------------------
; GetSixteenthMaxHP
; Gets 1/16 of pkmn's max HP
; Input: HL = Pt to detailed data
; Output: HL = 1/16 HP
; ---------------------------------------------------------------

GetSixteenthMaxHP:
	call GetEighthMaxHP		; HL = HL / 2 / 2 / 2
	srl H				; HL = HL / 2
	rr L
	ret
	
; ---------------------------------------------------------------
; GetThirdMaxHP
; Gets 1/3 of pkmn's max HP
; Input: HL = Pt to detailed data
; Output: HL = 1/3 max HP
; ---------------------------------------------------------------

GetThirdMaxHP:
	call GetPKMNMaxHP		; HL = max HP
	ld D, 3
	RCall( cDivHL )		; L = HL / 3
	ld H, 0
	ret
	
; ---------------------------------------------------------------
; GetTwoThirdsMaxHP
; Gets 2/3 of pkmn's max HP
; Input: HL = Pt to detailed data
; Output: HL = 2/3 max HP
; ---------------------------------------------------------------
	
GetTwoThirdsMaxHP:
	call GetThirdMaxHP		; HL = 1/3 max HP
	sla L				; HL = HL * 2
	rl H
	ret
	
; ---------------------------------------------------------------
; GetSixthMaxHP
; Gets 1/6 of pkmn's max HP
; Input: HL = Pt to detailed data
; Output: HL = 1/6 max HP
; ---------------------------------------------------------------
	
GetSixthMaxHP:
	call GetThirdMaxHP		; HL = 1/3 max HP
	srl L				; HL = HL / 2
	rr H
	ret
	
; ---------------------------------------------------------------
; PokeHasStatus
; Checks whether pkmn has status
; Input: HL = pkmn to check
; Output: c = pkmn has status
;         C = status
; Preserves HL, A
; ---------------------------------------------------------------

PokeHasStatusTrgt:			; TODO: unused?
	push HL
	ld HL, (TargetPKMN)
	jr PokeHasStatusCont		; $ + 2 + 1
	
PokeHasStatus:
	push HL
PokeHasStatusCont:
	ld B, A
	push BC
	call GetPKMNStatus		; A = status
	pop BC
	cp 1				; c if A < 1, nc if A >= 1
	ccf				; assumes stNone = 0
	ld C, A
	pop HL
	ld A, B
	ret
	
; ---------------------------------------------------------------
; IsTargetProtect
; Returns whether target pkmn is protecting or not, disp msg
; Input: A = attacking move
; Output: c = protect, nc = no protect
;         (TrgtProtect)
; Preserves HL, A
; ---------------------------------------------------------------

IsTargetProtect:
	push HL
	RCall( cGetMoveProtect )	; preserves A
	pop HL
	ret z				; ret w/ nc if protect flag not set
	
	push HL			; preserve HL
	ld HL, (TargetPKMN)
	ld BC, PKMN_Protect
	add HL, BC
	ld B, A			; preserve A
	ld A, (HL)
	pop HL
	or A				; resets carry flag
	ld A, B			; restore A
	ret z				; if target pkmn did not use protect, ret w/ z, nc
	
IsTargetProtectYes:
	rTextbox(tPKMNProtectedItself)
	scf
	ret
	
IsTargetProtectNo:
	;or A				; reset carry flag
	;ret
	
; ---------------------------------------------------------------
; PKMNProtect
; Determines if protect/detect/endure succeeds
; Input: (AtkPKMN) = pt to data of atk pkmn
; Output: c = fail, nc = success
;         HL = pt to counter, if success
; ---------------------------------------------------------------

PKMNProtect:
	ld HL, (AtkPKMN)
	ld BC, PKMN_Counter			; HL = pt to counter
	add HL, BC
	ld A, (TurnNum)			; fail if second turn
	cp 2
	jr z, PKMNProtectFailed
	push HL
	ld BC, PKMN_Counter - PKMN_LastMove
	or A
	sbc HL, BC
	ld A, (HL)				; A = last move ID
	pop HL
	cp mProtect				; if last move not protect/detect/endure, then success
	jr z, PKMNProtectCont
	cp mDetect
	jr z, PKMNProtectCont
	cp mEndure
	jr z, PKMNProtectCont
	jr PKMNProtectSuccess
	
PKMNProtectCont:
	ld B, (HL)				; B = counter
	ld A, B
	or A					; if counter == 0
	ld A, 128				; A = %10000000
	jr z, PKMNProtectSuccess		; then success, set counter to 1
	
PKMNProtectLp:
	srl A					; logical right shift, bit 7 reset
	djnz PKMNProtectLp		; for each counter, A /= 2 (A = 128, 64, 32, 16 ...)
PKMNProtect0:
	push AF
	push HL
	ld B, 128
	call iRandom				; 0 <= A < 128
	pop HL
	pop DE					; D = prob
	cp D					; if A >= D, then fail
	jr nc, PKMNProtectFailed
PKMNProtectSuccess:
	inc (HL)				; inc counter if success
	or A					; reset c flag, success
	ret
	
PKMNProtectFailed:				; HL = pt to counter, resets counter, calls textbox, ret w/ c
	ld (HL), 0				; reset counter
	call AttackEffect_ButItFailed
	scf					; set c flag, fail
	ret

; ---------------------------------------------------------------
; PKMNBerryCure
; Tests if PKMN carries berry, cures, disp msg, clear status, 
; deplete berry
; Input: HL = pt to pkmn status
;        B  = berry item ID
;        DE = pt to message to disp
; ---------------------------------------------------------------

PKMNBerryCure:
	push DE
	push BC
	ld BC, PKMN_Status - PKMN_Item
	or A					; set c flag to 0
	sbc HL, BC				; HL = pt to pkmn item
	ld A, (HL)				; A = item ID
	ex DE, HL				; DE = pt to pkmn item
	pop BC
	pop HL					; get message as HL
	
	cp iLumBerry				; if holding lum berry
	jr nz, PKMNBerryCureNotLum
	ld HL, tPKMNLumBerry
	jr PKMNBerryCureLum
	
PKMNBerryCureNotLum:
	cp B
	ret nz					; if holding berry
	
PKMNBerryCureLum:
	push DE
	ld (TextItem), A			; (TextItem) = berry item ID
	RCall( cTextbox )			; then disp msg
	pop HL
	ld (HL), iNoItem			; item is depleted
	ld BC, PKMN_Status - PKMN_Item
	add HL, BC
	ld A, stNone
	ld (HL), A				; cure status
	RCall( cDispStatus )			; clear status field
	ret

; ---------------------------------------------------------------
; DoReflectDmg
; Perform counter / mirror coat effect
; Input: C = 1 if phys (counter), 2 if spec (mirror coat)
; Output: c = all pkmn fainted
; ---------------------------------------------------------------

DoReflectDmg:
	ld A, (TypeEff)
	or A					; clears c flag
	ret z					; do nothing if target not affected, ret nc
	ld A, (TurnNum)
	cp 2
	jp nz, AttackEffect_ButItFailed	; fail if not turn 2
	
	push BC				; save C
	ld HL, (TargetPKMN)
	ld BC, PKMN_LastMove
	add HL, BC
	ld A, (HL)				; A = last move of target pkmn (should be first move in turn)
	RCall( cGetMoveType )
	pop BC					; get C
	cp eThreshold
	ld A, C				; A = 1 (if counter), 2 (if mirror coat)
	jr c, DoReflectDmgPhys
DoReflectDmgSpec:				; attack is spec
	cp 2
	jp nz, AttackEffect_ButItFailed	; fail if spec, and not mirror coat
	jr DoReflectDmgCont
	
DoReflectDmgPhys:				; attack is phys
	cp 1
	jp nz, AttackEffect_ButItFailed	; fail if phys, and not counter
	
DoReflectDmgCont:
	ld HL, (LastDmg)			; HL = total value of last damage done
	ld A, H
	or L
	jp z, AttackEffect_ButItFailed	; fail if no damage done
	sla L
	rl H					; HL *= 2
	RCall( cDoDamageFlip )		; damage opp
	;call c, Turn_Fainted			; returns c if all pkmn fainted
	ret
	
; ---------------------------------------------------------------
; ChkParty
; Checks pkmn in player / opp party
; Input: (PKMNFlag) = 1 for player, 0 for opp
; Output: A = team number | 0 (misc)
;         B = 1 (player) | 4 (opp)
;         c = fail, nc = no fail
; ---------------------------------------------------------------

ChkParty:
	bit PKMNFlag, (IY + Asm_Flag1)
	jr nz, ChkPartyPlayer
ChkPartyOpp:						; check opponent party
	ld HL, OppPKMN2 + PKMN_Status
	ld B, 4					; team num ranges from (4 - 6)
	jr ChkPartyCont
	
ChkPartyPlayer:					; check player party
	ld HL, TeamPKMN2 + PKMN_Status
	ld B, 1					; team num ranges from (1 - 3)
	
ChkPartyCont:
	ld A, (HL)					; A = status of 2nd pkmn in party
	ld DE, BYTES_PER_TPKMN
	add HL, DE					; HL = pt to status of 3rd pkmn in party
	cp stFNT
	ld A, (HL)					; A = status of 3rd pkmn in party
	jr nz, ChkParty2NotFNT
ChkParty2IsFNT:
	cp stFNT
	jr nz, ChkParty2IsFNT3NotFNT
ChkParty2IsFNT3IsFNT:				; if 2nd pkmn is FNT and 3rd pkmn is FNT
	scf						; set carry, to signal fail
	ret						; then fail
	
ChkParty2IsFNT3NotFNT:				; if 2nd pkmn is FNT and 3rd pkmn is not FNT
	ld A, B
	inc A
	inc A						; A = 3 | 6
	or A						; reset carry, no fail
	ret
	
ChkParty2NotFNT:					; if 2nd pkmn is not FNT
	cp stFNT
	jr nz, ChkParty2NotFNT3NotFNT
ChkParty2NotFNT3IsFNT:				; if 2nd pkmn is not FNT and 3rd pkmn is FNT
	ld A, B
	inc A						; A = 2 | 5
	or A						; reset carry, no fail
	ret
	
ChkParty2NotFNT3NotFNT:				; if 2nd pkmn is not FNT and 3rd pkmn is not FNT
	xor A						; A = 0, choose random pkmn, resets carry
	ret
	
; ---------------------------------------------------------------
; PKMNIsType
; Returns whether pkmn is given type
; Input: HL = Pt to detailed data
;        E = type to check
; Output: A = 0 if pkmn is type, not 0 otherwise
;         z/nc if pkmn is type, nz/nc otherwise
; Preserves HL
; ---------------------------------------------------------------

PKMNIsType:
	push HL
	push DE
	RCall( cGetType )
	call GetByte
	pop DE
	sub E				; if type = E, B = A = 0
	inc HL
	push DE
	call GetByteC
	pop DE
	pop HL
	ld B, A
	ld A, C
	sub E				; if type = E, A = 0
	or A				; if A == 0 || B == 0, then z
	ret z
	ld A, B
	or A				; or resets c flag
	ret z
	xor A				; else, return nz
	inc A
	ret
	
; ---------------------------------------------------------------
; ChkCloudNine
; Returns whether a pkmn on field has cloud nine ability
; Output: nc = has cloud nine, c = no cloud nine
; Saves A, HL, DE
; ---------------------------------------------------------------

ChkCloudNine:
	push AF
	push HL
	call ChkCloudNine2
	pop HL
	pop BC
	ld A, B
	ret
	
ChkCloudNine2:			; subroutine - check if pkmn has cloud nine, regs not saved
	ld HL, (AtkPKMN)
	call GetPKMNAbility
	cp aCloudNine			; if atk pkmn has cloud nine
	ret z				; ret w/ z, nc
	
	ld HL, (TargetPKMN)
	call GetPKMNAbility
	cp aCloudNine			; if target pkmn has cloud nine
	ret z				; ret w/ z, nc
	scf
	ret				; else, ret w/ c
	
; ---------------------------------------------------------------
; IsSafeguard
; Returns whether target pkmn has safeguard
; Input: (PKMNFlag) = 1 if player is target, 0 if opp
; Output: c = safeguard, nc = none
; Preserves HL, A
; ---------------------------------------------------------------

IsSafeguard:
	ld D, A			; preserve A as D
	call GetSafeguard		; A = safeguard status of target, DE preserved
	or A
	ld A, D
	scf				; set c, z unaffected
	ret nz				; ret w/ c if safeguard != 0
	ccf
	ret				; else, ret w/ nc
	
; ---------------------------------------------------------------
; GetSafeguard
; Returns safeguard status
; Input: (PKMNFlag) = 1 if player is target, 0 if opp
; Output: A = safeguard status of target
; ---------------------------------------------------------------

GetSafeguard:
	bit PKMNFlag, (IY + Asm_Flag1)
	jr nz, GetSafeguardPlayer
GetSafeguardOpp:
	ld A, (OppSafeguard)
	ret
	
GetSafeguardPlayer:
	ld A, (UserSafeguard)
	ret
	
; ---------------------------------------------------------------
; GetLight
; Returns pt to light screen, reflect status
; Input: (PKMNFlag) = 1 if player is target, 0 if opp
; Output: HL = pt to light screen status
; ---------------------------------------------------------------

GetLight:
	bit PKMNFlag, (IY + Asm_Flag1)
	jr nz, GetLightPlayer
GetLightOpp:
	ld HL, OppLight
	ret
	
GetLightPlayer:
	ld HL, UserLight
	ret
	
; ---------------------------------------------------------------
; GetLastMove
; Returns A = pkmn's last move, z if no last move
; Input: HL = pt to pkmn data
; Output: A = last move
;         z = no last move, nz = last move
; ---------------------------------------------------------------

GetLastMoveHL:				; get last move of atk pkmn
	ld HL, (AtkPKMN)

GetLastMove:
	ld BC, PKMN_LastMove
	add HL, BC				; HL = pt to last move
	ld A, (HL)
	cp $FF
	ret
	
; ---------------------------------------------------------------
; FlipPKMNFlag
; Inverts PKMNFlag
; Input: (PKMNFlag)
; Output: (PKMNFlag)
; ---------------------------------------------------------------

FlipPKMNFlag:
	bit PKMNFlag, (IY + Asm_Flag1)
	jr z, FlipPKMNFlag0
FlipPKMNFlag1:
	res PKMNFlag, (IY + Asm_Flag1)
	ret
	
FlipPKMNFlag0:
	set PKMNFlag, (IY + Asm_Flag1)
	ret
	
; ---------------------------------------------------------------
; SetAtkFlag
; Sets PKMNFlag to AtkPKMN
; Input: (PKMNFlag), (TurnFlag)
; Output: (PKMNFlag)
; ---------------------------------------------------------------
	
SetAtkFlag:
	bit TurnFlag, (IY + Asm_Flag1)
	jr nz, SetAtkFlag1
	
	res PKMNFlag, (IY + Asm_Flag1)
	ret
	
SetAtkFlag1:
	set PKMNFlag, (IY + Asm_Flag1)
	ret
	
; ---------------------------------------------------------------
; IsStallMove
; Returns if stall move is used (bounce, dig, dive, fly)
; Input: (AtkMove)
; Output: c = not stall, nc = stall move
; ---------------------------------------------------------------

IsStallMove:
	ld HL, AtkMove
	ld A, (HL)
	cp mBounce
	ret z
	cp mDig
	ret z
	cp mDive
	ret z
	cp mFly
	ret z
	scf
	ret	
	
; ---------------------------------------------------------------
; IsTargetSub
; Returns if target pkmn has a substitute up
; Input: (TargetPKMN)
; Output: z = not sub, nz = sub (c flag cleared)
;         A = sub status of target
;         BC = pt to sub status
; Preserves HL, DE
; ---------------------------------------------------------------
	
IsTargetSub:
	push HL			; save HL
	ld HL, (TargetPKMN)
	ld BC, PKMN_Substitute
	add HL, BC
	ld A, (HL)			; A = target sub status
	push HL \ pop BC		; BC = 
	pop HL				; get HL
	or A				; if A = 0 (c flag cleared)
	ret
	
; ---------------------------------------------------------------
; SetFlinch
; Sets flinch status to 1, deals w/ substitute
; Input: HL = pt to pkmn detailed status
; ---------------------------------------------------------------

SetFlinch:
	ld BC, PKMN_Flinch
	add HL, BC
	ld (HL), 1				; set flinch status to 1
	ret
	
; ---------------------------------------------------------------
; SetCharge
; Set charge status to 1
; Input: HL = pt to pkmn detailed status
; ---------------------------------------------------------------

SetCharge:
	ld BC, PKMN_Charge
	add HL, BC
	ld (HL), 1
	ret
	
; ---------------------------------------------------------------
; Calc10
; Calculates 10% of HL, stores in DE
; Input: HL = number
; Output: DE = HL / 10
; Preserves HL
; ---------------------------------------------------------------
	
Calc10:
	push HL
	ld D, 10
	RCall( cDivHL )		; L = HL / 10
	ld H, 0
	ex DE, HL			; DE = HL / 10
	pop HL
	ret
	
; ---------------------------------------------------------------
; Calc20
; Calculates 20% of HL, stores in DE
; Input: HL = number
; Output: DE = HL / 10 * 2
; Preserves HL
; ---------------------------------------------------------------

Calc20:
	call Calc10			; DE = HL / 10
	sla E
	rl D				; DE = DE * 2
	ret
	
; ---------------------------------------------------------------
; Calc50
; Calculates 50% of HL, stores in DE
; Input: HL = number
; Output: DE = HL / 50
; Preserves HL
; ---------------------------------------------------------------

Calc50:
	push HL		; save HL
	srl H
	rr L
	ex DE, HL		; DE = HL / 2
	pop HL			; restore HL
	ret
	
; ---------------------------------------------------------------
; Calc1p5
; Calculates 150% of DE, stores in DE
; Input: DE = number
; Output: DE = DE * 1.5
; Preserves HL
; ---------------------------------------------------------------

Calc1p5:
	push HL
	ld H, D		; HL = DE
	ld L, E
	srl D			; DE /= 2
	rr E
	add HL, DE		; HL = DE * 1.5
	ex DE, HL		; DE = HL
	pop HL
	ret
	
; ---------------------------------------------------------------
; Calc8
; Calculates 1 / 8 of HL, stores in HL
; Input: HL = number
; Output: HL = HL / 8
; ---------------------------------------------------------------

Calc8:
	srl H			; HL /= 2
	rr L
	srl H			; HL /= 2
	rr L
	srl H			; HL /= 2
	rr L
	ret
	
; ---------------------------------------------------------------
; ChkFullHP
; Checks if PKMN has full HP, returns flag. Also checks if HP = 0
; Input: HL = pt to pkmn data
; Output: z if equal, nc if HP >= max HP, c if HP < max HP
; Preserves HL
; ---------------------------------------------------------------

ChkFullHP:
	push HL
	call GetPKMNMaxHP			; HL = max HP
	ex DE, HL				; DE = max HP
	pop HL
	
	push HL
	call GetPKMNHP			; HL = current HP, DE not affected
	ld A, L
	or H					; if HL = 0 (c flag cleared)
	jr z, ChkFullHPSkip			; then don't compare, ret w/ nc and z
	call CpHLDE				; compare HL and DE
	
ChkFullHPSkip:
	pop HL
	ret
	
; ---------------------------------------------------------------
; ChkHalfHP
; Checks if PKMN has less than 1/2 max HP, returns flag
; Input: HL
; Output: c if HP < 1/2 max HP, nc otherwise, z if equals
; Preserves HL
; ---------------------------------------------------------------

ChkHalfHP:
	push HL
	push HL
	call GetHalfMaxHP		; HL = 1/2 max HP
	ex DE, HL			; DE = 1/2 max HP
	pop HL
	call GetPKMNHP		; HL = current HP, DE not affected
	call CpHLDE			; c if HL < DE, nc if HL >= DE
	pop HL
	ret
	
; ---------------------------------------------------------------
; ChkFourthHP
; Checks if PKMN has less than 1/4 max HP, returns flag
; Input: HL
; Output: c if HP < 1/4 max HP, nc otherwise, z if equal
; Preserves HL
; ---------------------------------------------------------------

ChkFourthHP:
	push HL
	push HL
	call GetFourthMaxHP		; HL = 1/4 max HP
	ex DE, HL			; DE = 1/4 max HP
	pop HL
	call GetPKMNHP		; HL = current HP, DE not affected
	call CpHLDE			; c if HL < DE, nc if HL >= DE
	pop HL
	ret
	
; ---------------------------------------------------------------
; ChkThirdHP
; Checks if PKMN has less than 1/3 max HP, returns flag
; Input: (AtkPKMN)
; Output: c if HP < 1/3 max HP, nc otherwise
; Preserves HL
; ---------------------------------------------------------------

ChkThirdHP:
	push HL			; save HL
	ld HL, (AtkPKMN)		; HL = pt to atk pkmn
	push HL			; save HL, pt to data
	call GetThirdMaxHP		; HL = 1/3 max HP
	ex DE, HL			; DE = 1/3 max HP
	pop HL				; get HL, pt to data
	call GetPKMNHP		; HL = current HP, saves DE
	call CpHLDE			; compare HL and DE
	pop HL				; get HL, flags unchanged
	ret
	
; ---------------------------------------------------------------
; HealHalfHP
; Heals half of max HP
; Input: HL = pt to pkmn data
;        (PKMNFlag) = 1 if heal player, 0 if heal opp
; ---------------------------------------------------------------

HealHalfHP:
	call GetHalfMaxHP			; HL = max HP / 2
	RCall( cDoHeal )			; perform heal
	ret
	
; ---------------------------------------------------------------
; HealHalfDamage
; Heals by half of damage done
; Input: (PKMNFlag) = 1 if heal player, 0 if heal opp
; ---------------------------------------------------------------

HealHalfDamage:
	call GetHalfDamage			; HL = damage / 2
	RCall( cDoHeal )			; heal by HL
	ret
	
; ---------------------------------------------------------------
; GetHalfDamage
; Gets half of damage as HL
; Output: HL = (Damage) / 2
; ---------------------------------------------------------------

GetHalfDamage:
	ld HL, (Damage)			; HL = Damage as calculated by CalcDamage
	srl H					; HL /= 2
	rr L
	ret
			
; ---------------------------------------------------------------
; PKMNInterrupt
; Resets rollout, when interrupted (miss, confusion, paralysis)
; Input: (AtkPKMN)
; ---------------------------------------------------------------

PKMNInterrupt:
	ld HL, (AtkPKMN)
	ld BC, PKMN_Duration
	add HL, BC
	ld (HL), 0
	
PKMNInterrupt2:					; subroutine - interrupt rollout, not Petal Dance / Thrash
	call PKMNGetRollout
	ld (HL), 0
	ret
	
PKMNGetRollout:					; subroutine - gets rollout status of (AtkPKMN)
	ld HL, (AtkPKMN)
	ld BC, PKMN_Rollout
	add HL, BC
	ret
	
; ---------------------------------------------------------------
; DispPKMNSprite
; Display front / back sprite, depending on PKMNFlag
; ---------------------------------------------------------------

DispPKMNSprite:
	bit PKMNFlag, (IY + Asm_Flag1)
	jp nz, BattleGraphicsPlayerSprite

DispPKMNSpriteOpp:
	jp BattleGraphicsOppSprite
	
; ---------------------------------------------------------------
; CopyOpp / CopyPlayer
; Copies OppPKMN / PlayerPKMN to data of first team member
; ---------------------------------------------------------------
	
CopyOpp:			; subroutine - copies opppkmn to data of team first member
	ld HL, OppPKMN
	ld DE, OppPKMN1
	ld BC, BYTES_PER_TPKMN
	ldir				; copy to team data
	ret
	
CopyPlayer:
	ld HL, PlayerPKMN
	ld DE, TeamPKMN1
	ld BC, BYTES_PER_TPKMN
	ldir					; copy to team data
	ret
	
; ---------------------------------------------------------------
; SwapRAMB
; Swap B bytes of RAM
; Input: HL = Pt to RAM block 1
;        DE = Pt to RAM block 2
;        B = Bytes to swap, > 0
; ---------------------------------------------------------------

SwapRAMB:
SwamRAMBLp:
	ld C, (HL)
	ld A, (DE)
	ld (HL), A
	ld A, C
	ld (DE), A
	inc HL
	inc DE
	djnz SwamRAMBLp
	ret
	
; ---------------------------------------------------------------
	
ChkOPErr:				; check if op1 has error, exp > 3
	ld A, (OP1 + 1)
	cp $84
	ret c
	;jp Quit			; quit if exp >= 4
	b_call(_OP1Set0)		; set OP1 to 0, if exp >= 4
	ret
	
;SaveOP:
;	ld D, 0
;	ld E, A
;	ld HL, Buffer
;	add HL, DE
;	ex DE, HL
;;	ld HL, OP1
;	ld BC, 11
;	ldir
	
;	or A
;	ld B, TMINt
;	jr z, SaveOPVar
;;	cp 12
;	ld B, TMAXt
;	jr z, SaveOPVar
;	cp 24
;	ld B, THETMINt
;	jr z, SaveOPVar
;;	cp 36
;	ld B, THETMAXt
;	jr z, SaveOPVar
;	ld B, TSTEPt
	
;SaveOPVar:
;	ld A, B
;	b_call(_StoSysTok) ;;; debug
;	ret
	
.echo "Swap Zone: "
.echo ZoneSize
.echo " bytes.\n"

Battle_End:				; Must be at end of Battle code
.echo "Battle: "			; prints out battle code size in bytes
.echo Battle_End - Battle
.echo " bytes. "
.echo "("
.echo 100 * (Battle_End - Battle) / (CodeSize) ; (8811)
.echo "%)\n"

.echo "Buffer: "
.echo CodeSize - (Battle_End - Battle)
;.echo " | "
;.echo (Battle_End - Battle) - (Overworld_End - Init)
.echo " bytes.\n"
.fill CodeSize - (Battle_End - Battle), $00			; buffer to prevent Relocate from corrupting data

.endrelocate
RBattle_End:
