; ===============================================================
; Turn
; ===============================================================

; ---------------------------------------------------------------
; Turn:
; Determines priorities, damage, effects, end of turn conditions,
; handles in-turn switches
; Input: A = Player move ($FF | $FE | $FD if switching out)
; 	  B = Opp move
; 	  (StatVars), etc.
; Output: c = battle is over
; ---------------------------------------------------------------

Turn:
	ld (CurMovePlayer), A	; save moves
	push AF
	ld A, B
	ld (CurMoveOpp), A
	pop AF
	ld HL, TurnNum		; reset turn number to first turn
	ld (HL), 0
	
	ld A, (CurMovePlayer)
	cp mFocusPunch		; if player move is focus punch
	jr nz, TurnNotPlayerFocus
	set PKMNFlag, (IY + Asm_Flag1)
	rTextbox(tPKMNIsTighteningItsFocus); then display msg
	ld A, 1
	ld (PlayerPKMN + PKMN_Focus), A
	
TurnNotPlayerFocus:
	ld A, (CurMoveOpp)
	cp mFocusPunch		; if opp move is focus punch
	jr nz, TurnNotOppFocus
	res PKMNFlag, (IY + Asm_Flag1)
	rTextbox(tPKMNIsTighteningItsFocus)
	ld A, 1
	ld (OppPKMN + PKMN_Focus), A
	
TurnNotOppFocus:
	ld A, (CurMoveOpp)
	ld B, A			; get opp move
	ld A, (CurMovePlayer)	; get player move
	RCall( cCalcPriority )
	jr nc, Turn_Opp
	
Turn_Player:				; Player atk first
	call Attack
	jr c, Turn_EndFainted
	call Attack2
	jr c, Turn_EndFainted
	jr Turn_End
	
Turn_Opp:				; Opp atk first
	call Attack2
	jr c, Turn_EndFainted
	call Attack
	jr c, Turn_EndFainted
	jr Turn_End
	
Turn_EndFainted:
	call Turn_Fainted
	ret c
	
Turn_End:
	call Turn_EOT			; handle end-of-turn effects
	ret c
	
	scf
	ccf
	ret
	
; ---------------------------------------------------------------
; subroutines
; ---------------------------------------------------------------

Turn_EOT:				; end-of-turn effects, return c if all pkmn fainted
	
; ---------------------------------------------------------------
; End of turn effects
; ---------------------------------------------------------------
; Tier 1
; A - Reflect wears off
; B - Light Screen wears off
; C - Safegaurd fades
; ---------------------------------------------------------------
; Tier 2
; A - Wish activates
; ---------------------------------------------------------------
; Tier 3
; A - Weather - Hail, Sandstorm, Rain, and Sunlight
; ---------------------------------------------------------------
; Tier 4
; A - Shed Skin heals the user
; B - Ingrain recovers HP
; C - Rain Dish recovers HP
; D - Leftovers recovery/Berry activation (if HP <= 25% | 50% before 4A)
; E - Leech Seed damage
; F - Burn, Poison, Nightmare
; G - Curse
; H - Wrap and other 'vortex' moves
; ---------------------------------------------------------------
; Tier 5
; A - Yawn
; ---------------------------------------------------------------
; Tier 6
; A - Doom Desire, Future Sight
; ---------------------------------------------------------------

Turn_EOT1:	
	call Turn_SetPriority	; sets priority for Wish
	
Turn_EOT1A:				; reflect
	call Turn_Reflect
	
Turn_EOT1B:				; light screen
	call Turn_LightScreen
	
Turn_EOT1C:				; safeguard
	call Turn_Safeguard
	
Turn_EOT2:
	
Turn_EOT2A:
	call Turn_Wish		; wish activation
	
Turn_EOT3:
	
Turn_EOT3A:				; weather effects
	call Turn_Weather
	ret c
	
Turn_EOT4:
	call Turn_SetPriority	; takes weather into account (swift swim)
	
Turn_EOT4A:
	call Turn_HP25		; determines if HP <= 25% (occurs before 4A)
	call Turn_HP50		; determines if HP <= 50% (occurs before 4A)
	
Turn_EOT4B:
	call Turn_Ingrain		; ingrain heal
	
Turn_EOT4C:
	call Turn_RainDish		; rain dish heal
	
Turn_EOT4D:
	call Turn_Leftovers
	call Turn_Pinch		; pinch berry effects (liechi, petaya, salac, sitrus, leppa)
	
Turn_EOT4E:				; leech seed damage
	call Turn_Leech
	ret c
	
Turn_EOT4F:				; burn, poison, nightmare damage
	call Turn_Burn
	ret c
	call Turn_Poison
	ret c
	call Turn_BadPoison
	ret c
	call Turn_Nightmare
	ret c
	
Turn_EOT4G:
	
Turn_EOT4H:
	call Turn_Trap		; trap damage (sand tomb, wrap)
	ret c
	
Turn_EOT5:
	
Turn_EOT5A:
	call Turn_Yawn
	
Turn_EOT6:
	
Turn_EOT6A:				; future sight damage
	call Turn_Future
	ret c
	
Turn_EOTMisc:
	call Turn_Disable
	call Turn_Encore
	call Turn_Perish		; priority?? after leftover/poison (Gen 5)
	ret c
	
	call Turn_Misc
	
	scf
	ccf
	ret

; ---------------------------------------------------------------
; End of turn subroutines
; ---------------------------------------------------------------

; ---------------------------------------------------------------
; Turn_Reflect
; ---------------------------------------------------------------

Turn_Reflect:
	ld HL, UserReflect		; user first, arbitrary priority
	set PKMNFlag, (IY + Asm_Flag1)
	call Turn_ReflectCalc
	ld HL, OppReflect
	res PKMNFlag, (IY + Asm_Flag1)
	call Turn_ReflectCalc
	ret
	
Turn_ReflectCalc:			; subroutine, checks if reflect counter is zero
	ld A, (HL)
	or A
	ret z				; ret w/ nc
	dec A
	ld (HL), A
	or A
	ret nz				; fade if A decremented = 0
	
Turn_ReflectFade:
	ld A, mReflect
	ld (TextMove), A
	rTextboxRet(tYourTeamsShieldStoppedWorking)
	
; ---------------------------------------------------------------
; Turn_LightScreen
; ---------------------------------------------------------------

Turn_LightScreen:
	ld HL, UserLight		; user first, arbitrary priority
	set PKMNFlag, (IY + Asm_Flag1)
	call Turn_LightScreenCalc
	ld HL, OppLight
	res PKMNFlag, (IY + Asm_Flag1)
	call Turn_LightScreenCalc
	ret
	
Turn_LightScreenCalc:			; subroutine, checks if light screen counter is zero
	ld A, (HL)
	or A
	ret z					; ret if no light screen
	dec A
	ld (HL), A
	or A
	ret nz					; fade if A decremented = 0
	
Turn_LightScreenFade:
	ld A, mLightScreen
	ld (TextMove), A
	rTextboxRet(tYourTeamsShieldStoppedWorking)
	
; ---------------------------------------------------------------
; Turn_Safeguard
; ---------------------------------------------------------------

Turn_Safeguard:
	ld HL, UserSafeguard		; user first, arbitrary priority
	set PKMNFlag, (IY + Asm_Flag1)
	call Turn_SafeguardCalc
	ld HL, OppSafeguard
	res PKMNFlag, (IY + Asm_Flag1)
	call Turn_SafeguardCalc
	ret
	
Turn_SafeguardCalc:			; subroutine, checks if light screen counter is zero
	ld A, (HL)
	or A
	ret z				; ret if no light screen
	dec A
	ld (HL), A
	or A
	ret nz				; fade if A decremented = 0
	
Turn_SafeguardFade:
	rTextboxRet(tPKMNsSafeguardFaded)

; ---------------------------------------------------------------
; Turn_Wish
; ---------------------------------------------------------------

Turn_Wish:				; activates wish
	ld BC, Turn_WishCalc
	jp Turn_Order

Turn_WishCalc:			; HL = pt to data
	ex DE, HL			; save HL as DE
	bit PKMNFlag, (IY + Asm_Flag1)
	jr nz, Turn_WishCalcPlayer
Turn_WishCalcOpp:
	ld HL, OppWish
	jr Turn_WishCalcCont
	
Turn_WishCalcPlayer:
	ld HL, UserWish
	
Turn_WishCalcCont:
	ld A, (HL)
	or A
	ret z				; ret if no wish used
	dec A
	ld (HL), A
	or A
	ret nz				; wish comes true when A decremented = 0
	
Turn_WishCalcTrue:
	ex DE, HL			; get pt to data as HL
	call ChkFullHP
	ret nc				; wish does nothing if already at full HP
	
	rTextbox(tPKMNsWishCameTrue)
	call HealHalfHP
	or A
	ret				; ret w/ nc
	
; ---------------------------------------------------------------
; Turn_Weather
; ---------------------------------------------------------------

Turn_Weather:				; sun, rain, sandstorm, hail
	ld A, (Weather)
	or A				; clear c flag
	ret z				; ret w/ nc if wNone
	ld HL, WeatherCount
	cp wSun
	jr z, Turn_WeatherSun
	cp wRain
	jr z, Turn_WeatherRain
	cp wSandstorm
	jr z, Turn_WeatherSandstorm
	jp Turn_WeatherHail		; wHail
	
Turn_WeatherSun:
	ld A, (HL)
	or A				; clear c flag
	jr z, Turn_WeatherSunStop	; stop sun if counter at zero
	dec A				; else dec counter
	ld (HL), A
	rTextboxRet(tSunlightIsStrong); ret w/ nc
	
Turn_WeatherSunStop:
	xor A				; ld A, wNone, clear c flag
	ld (Weather), A
	rTextboxRet(tSunlightFaded)	; ret w/ nc
	
Turn_WeatherRain:
	ld A, (HL)
	or A				; clear c flag
	jr z, Turn_WeatherRainStop	; stop rain if counter at zero
	dec A				; else dec counter
	ld (HL), A
	rTextboxRet(tRainContinuesToFall); ret w/ nc
	
Turn_WeatherRainStop:
	xor A				; ld A, wNone, clear c flag
	ld (Weather), A
	rTextboxRet(tTheRainStopped); ret w/ nc
	
Turn_WeatherSandstorm:		; sandstorm - non-rock/ground/steel pkmn lose 1/16 HP
	ld A, (HL)
	or A				; stop sandstorm if counter at zero
	jr z, Turn_WeatherSandstormStop
	
	cp $FF				; don't dec counter if indefinite
	jr z, Turn_WeatherSandstormRage
	dec A				; else dec counter
	ld (HL), A
	
Turn_WeatherSandstormRage:
	rTextbox(tSandstormRages)
	
	call ChkCloudNine		; nc = pkmn has cloud nine
	ret nc				; no sandstorm damage if cloud nine
	
	ld BC, Turn_WeatherSandstormCalc	; deal damage, order by priority
	jp Turn_Order
	
Turn_WeatherSandstormCalc:		; subroutine - deals 1/16 dmg if not rock/ground/steel type, HL = pt to detailed data
	ld E, eRock
	call PKMNIsType		; preserves HL
	ret z				; ret w/ nc if equal
	ld E, eGround
	call PKMNIsType
	ret z				; ret w/ nc if equal
	ld E, eSteel
	call PKMNIsType
	ret z				; ret w/ nc if equal
	call GetPKMNStall		; preserves HL
	cp evDig			; no dmg if dig / dive
	ret z				; ret w/ nc if equal
	cp evDive
	ret z				; ret w/ nc if equal
	rTextbox(tPKMNIsBuffetedByTheSandstorm)
	jr Turn_WeatherDamage16	; do 1/16 max HP as damage, return c if all pkmn FNT
	
Turn_WeatherSandstormStop:
	xor A				; ld A, wNone, clear c flag
	ld (Weather), A
	rTextboxRet(tSandstormSubsided)
	
Turn_WeatherHail:			; hail - non-ice pkmn lose 1/16 HP
	ld A, (HL)
	or A
	jr z, Turn_WeatherHailStop	; stop hail if counter at zero
	dec A				; else dec counter
	ld (HL), A
	rTextbox(tHailContinuesToFall)
	
	call ChkCloudNine		; nc = pkmn has cloud nine
	ret nc				; no hail damage if cloud nine
	
	ld BC, Turn_WeatherHailCalc
	jp Turn_Order
	
Turn_WeatherHailCalc:		; subroutine - deals 1/16 dmg if not ice type, HL = pt to detailed data
	ld E, eIce
	call PKMNIsType		; preserves HL
	ret z				; ret w/ nc if ice-type
	call GetPKMNStall		; preserves HL
	cp evDig			; no dmg if dig / dive
	ret z
	cp evDive
	ret z
	rTextbox(tPKMNIsStrickenByHail)
	
Turn_WeatherDamage16:
	call GetSixteenthMaxHP	; HL = 1/16 of max HP
	RCall( cDoDamage )
	call c, Turn_Fainted		; return c if all pkmn fainted
	ret
	
Turn_WeatherHailStop:
	xor A				; ld A, wNone, clear c flag
	ld (Weather), A
	rTextboxRet(tHailStopped)
	
; ---------------------------------------------------------------
; Turn_HP25
; ---------------------------------------------------------------
	
Turn_HP25:				; determines if HP <= 25% max
	ld BC, Turn_HP25Calc
	jp Turn_Order
	
Turn_HP25Calc:			; HL = pt to detailed data
	push HL
	push HL
	call GetFourthMaxHP		; HL = 1/4 max HP
	ex DE, HL			; DE = 1/4 max HP
	pop HL
	call GetPKMNHP		; HL = current HP, DE not affected
	call CpHLDE			; if HL >= DE, but not HL == DE
	pop HL
	jr z, Turn_HP25CalcJr
	ret nc				; then return w/ nc
	
Turn_HP25CalcJr:
	ld BC, PKMN_HP25		; else set HP25 flag
	add HL, BC			; c should not be set
	ld (HL), 1
	ret
	
; ---------------------------------------------------------------
; Turn_HP50
; ---------------------------------------------------------------

Turn_HP50:				; determines if HP <= 50% max
	ld BC, Turn_HP50Calc
	jp Turn_Order
	
Turn_HP50Calc:			; HL = pt to detailed data
	call ChkHalfHP
	jr z, Turn_HP50CalcJr	; if HL >= DE, but not HL == DE
	ret nc				; then return

Turn_HP50CalcJr:
	ld BC, PKMN_HP50		; else set HP50 flag
	add HL, BC			; c should not be set
	ld (HL), 1
	ret
	
; ---------------------------------------------------------------
; Turn_Ingrain
; ---------------------------------------------------------------

Turn_Ingrain:				; if planted roots, heal 1/16 HP
	ld BC, Turn_IngrainCalc
	jp Turn_Order
	
Turn_IngrainCalc:			; HL = pt to pkmn data
	call GetPKMNIngrain		; A = ingrain status, preserves HL
	ret z				; ret if no ingrain
	call ChkFullHP		; preserves HL
	ret nc				; ret if HP >= max HP
	
	rTextbox(tPKMNAbsorbtedNutrients); disp ingrain message
	call GetSixteenthMaxHP	; HL = 1/16 of max HP
	RCall( cDoHeal )		; heal by 1/16 max HP
	or A
	ret
	
; ---------------------------------------------------------------
; Turn_RainDish
; ---------------------------------------------------------------

Turn_RainDish:			; if rain dish and raining, heal 1/16 HP
	ld BC, Turn_RainDishCalc
	jp Turn_Order
	
Turn_RainDishCalc:
	call Turn_RainDishCalc2
	or A
	ret				; ret w/ nc
	
Turn_RainDishCalc2:
	call GetPKMNAbility		; A = pkmn ability, preserves HL
	cp aRainDish
	ret nz				; ret if not rain dish
	
	ld (TextAbil), A		; set (TextAbil) to rain dish ID
	
	ld A, (Weather)		; A = weather status
	cp wRain
	ret nz				; ret if not raining
	
	call ChkFullHP
	ret nc				; ret if HP >= max HP
	rTextbox(tRainDishRestoredHP)
	call GetSixteenthMaxHP	; HL = 1/16 of max HP
	RCall( cDoHeal )		; heal by 1/16 max HP
	ret
	
; ---------------------------------------------------------------
; Turn_Leftovers
; ---------------------------------------------------------------

Turn_Leftovers:			; if holding leftovers, heal 1/16 HP
	ld BC, Turn_LeftoversCalc
	jp Turn_Order
	
Turn_LeftoversCalc:			; HL = pt to detailed data
	call Turn_LeftoversCalc2
	or A
	ret				; ret w/ nc
	
Turn_LeftoversCalc2:
	call GetPKMNItem		; A = pkmn item, HL preserved
	cp iLeftovers
	ret nz				; ret if not holding leftovers
	
	ld (TextItem), A		; set (TextItem) to leftovers ID
	call ChkFullHP
	ret nc				; ret if HP >= max HP
	rTextbox(tPKMNLeftovers)	; disp leftovers message
	call GetSixteenthMaxHP	; HL = 1/16 of max HP
	RCall( cDoHeal )		; heal by 1/16 max HP
	ret
	
; ---------------------------------------------------------------
; Turn_Pinch
; ---------------------------------------------------------------

Turn_Pinch:				; if HP <= 25%, then activate pinch berries
	ld BC, Turn_PinchCalc
	jp Turn_Order
	
Turn_PinchCalc:			; HL = pt to detailed data
	call Turn_PinchCalc2
	or A
	ret 				; ret w/ nc
	
Turn_PinchCalc2:
	push HL			; save HL, pt to data
	call Turn_PinchCalcLeppa	; use leppa berry, if item
	pop HL				; get HL, pt to data
	
	ld BC, PKMN_HP25
	add HL, BC
	ld A, (HL)
	or A
	jr nz, Turn_PinchCalc25	; jump if HP25 flag is set
	inc HL				; PKMN_HP50 - PKMN_HP25 = 1
	ld A, (HL)
	or A
	ret z				; ret if HP50 flag not set
	
Turn_PinchCalc50:
	call Turn_PinchCalcSub	; HL = pt to pkmn item + 1
	dec HL
	ld A, (HL)			; A = pkmn item
Turn_PinchCalc50Jr:			; if HP <= 25%, then HP must be <= 50%
	cp iSitrusBerry
	ret nz
	
Turn_PinchCalcSitrus:
	; Sitrus Berry - restore health by 30 HP when HP <= 50%, deplete item
	ld (TextItem), A		; set (TextItem) to A
	call Turn_PinchCalcSub2	; depletes item
	ld HL, 30
	RCall( cDoHeal )
	rTextboxRet(tPKMNSitrusBerry)
	
Turn_PinchCalc25:
	call Turn_PinchCalcSub
	ld A, (HL)			; A = pkmn item
	ld (TextItem), A		; set (TextItem) to A
	
	cp iLiechiBerry
	jr z, Turn_PinchCalcLiechi
	cp iPetayaBerry
	jr z, Turn_PinchCalcPetaya
	cp iSalacBerry
	jr z, Turn_PinchCalcSalac
	jr Turn_PinchCalc50Jr	; if not holding liechi, petaya, or salac, try sitrus
	
Turn_PinchCalcLiechi:
	; Liechi Berry - raises Attack by 1 stage when HP <= 25%, one time use
	ld B, stAtk
	call AttackEffect_StatRaiseSilence
	rTextboxRet(tPKMNLiechiBerry)
	
Turn_PinchCalcPetaya:
	; Petaya Berry - raises Sp Attack by 1 stage when HP <= 25%, one time use
	ld B, stSAtk
	call AttackEffect_StatRaiseSilence
	rTextboxRet(tPKMNPetayaBerry)
	
Turn_PinchCalcSalac:
	; Salac Berry - raises Speed by 1 stage when HP <= 25%, one time use
	ld B, stSpd
	call AttackEffect_StatRaiseSilence
	rTextboxRet(tPKMNSalacBerry)
	
AttackEffect_StatRaiseSilence:	; subroutine - call stat raise w/ silence flag, input: B = stat, HL = pt
	call Turn_PinchCalcSub2	; saves B
	set SilenceFlag, (IY + Asm_Flag2)
	;;;call AttackEffect_StatRaiseC
	res SilenceFlag, (IY + Asm_Flag2)
	ret
	
Turn_PinchCalcLeppa:
	; Leppa Berry - if PP of any move is 0, then set to 10
	call GetPKMNItem		; A = pkmn item, HL preserved
	cp iLeppaBerry		; if not leppa berry
	ret nz				; then ret
	
	ld (TextItem), A		; set (TextItem) to A
	ld D, H
	ld E, L			; DE = HL, pt to data
	ld BC, PKMN_PP
	add HL, BC			; HL = pt to PP array
	xor A				; A = 0
	ld B, 4			; check 4 PP values
	
Turn_PincCalcLeppaLp:
	cp (HL)			; if PP value is not zero
	jr nz, Turn_PincCalcLeppaN0	; then skip to loop
	
	ld (HL), 10			; else, set PP to 10
	ex DE, HL			; HL = pt to data
	ld BC, PKMN_Item
	add HL, BC
	ld (HL), iNoItem		; leppa berry is depleted
	rTextboxRet(tPKMNLeppaBerry)
	
Turn_PincCalcLeppaN0:
	djnz Turn_PincCalcLeppaLp	; loop until B = 0
	ret
	
Turn_PinchCalcSub:			; subroutine - resets flag, points HL to item data
	xor A				; else reset HP25 | HP50 flag
	ld (HL), A
	ld BC, PKMN_HP25 - PKMN_Item
	or A				; reset c flag
	sbc HL, BC			; HL = pt to item
	ret
	
Turn_PinchCalcSub2:			; subroutine - depletes item, points HL to atk boost, C = 1, saves B
	ld (HL), iNoItem		; set item to iNoItem
	ld DE, PKMN_AtkBoost - PKMN_Item
	add HL, DE			; HL = pt to atk boost
	ld C, 1
	ret
	
; ---------------------------------------------------------------
; Turn_Leech
; ---------------------------------------------------------------

Turn_Leech:				; sap 1/8 HP, heal user
	ld BC, Turn_LeechCalc
	jp Turn_Order
	
Turn_LeechCalc:			; HL = pt to detailed data
	call GetPKMNLeech		; A = pkmn leech status, HL preserved
	ret z				; ret if no leech
	
	call GetPKMNAbility		; A = pkmn ability, HL preserved
	cp aLiquidOoze
	jr z, Turn_LeechCalcOoze
	
	rTextbox(tPKMNsHealthIsSappedByLeechSeed)
	
	call Turn_CalcDamage8	; damage target by 1/8 max HP
	push AF			; save c flag
	RCall( cDoHealFlip )		; heal user
	pop AF
	call c, Turn_Fainted		; return c if all pkmn fainted
	ret
	
Turn_LeechCalcOoze:
	rTextbox(tSuckedUpLiquidOoze)
	
	call Turn_CalcDamage8	; damage target by 1/8 max HP
	push AF			; save c flag
	RCall( cDoDamageFlip )	; damage user
	call c, Turn_Fainted		; return c if all pkmn fainted
	push AF
	pop BC				; save c flag in C
	pop AF				; get c flag
	call c, Turn_Fainted		; return c if all pkmn fainted
	ret c				; ret w/ c if all pmn fainted
	push BC
	pop AF				; get c flag from C
	ret				; ret w/ c or nc
	
Turn_CalcDamage8:			; subroutine - damages w/ 1/8 max HP, input: HL = pt to data
	call GetEighthMaxHP		; HL = 1/8 max HP
	push HL
	RCall( cDoDamage )		; damage target
	pop HL
	ret
	
; ---------------------------------------------------------------
; Turn_Burn
; ---------------------------------------------------------------

Turn_Burn:				; if pkmn burned, lose 1/8 HP
	ld BC, Turn_BurnCalc
	jp Turn_Order
	
Turn_BurnCalc:			; HL = pt to detailed data
	call GetPKMNStatus		; A = status
	cp stBRN
	scf
	ccf
	ret nz				; ret if PKMN not burned, nc
	
	rTextbox(tPKMNIsHurtByItsBurn)
	
	call Turn_CalcDamage8	; damage target by 1/8 max HP
	call c, Turn_Fainted		; return c if all pkmn fainted
	ret
	
; ---------------------------------------------------------------
; Turn_Poison
; ---------------------------------------------------------------

Turn_Poison:				; if pkmn poisoned, lose 1/8 HP
	ld BC, Turn_PoisonCalc
	jp Turn_Order
	
Turn_PoisonCalc:			; HL = pt to detailed data
	call GetPKMNStatus		; A = status
	cp stPSN
	scf
	ccf
	ret nz				; ret if PKMN not poisoned, nc
	
	rTextbox(tPKMNIsHurtByPoison)
	
	call Turn_CalcDamage8	; damage target by 1/8 max HP
	call c, Turn_Fainted		; return c if all pkmn fainted
	ret
	
; ---------------------------------------------------------------
; Turn_BadPoison
; ---------------------------------------------------------------

Turn_BadPoison:			; if pkmn badly poisoned, lose x/16 HP
	ld BC, Turn_BadPoisonCalc
	jp Turn_Order
	
Turn_BadPoisonCalc:			; HL = pt to detailed data
	call GetPKMNStatus		; A = status, HL preserved
	cp stBadPSN
	scf
	ccf
	ret nz				; ret if PKMN not badly PSNed, nc
	
	rTextbox(tPKMNIsHurtByPoison)
	
	push HL
	call GetSixteenthMaxHP
	ld D, L			; D = max HP / 16
	pop HL
	
	ld BC, PKMN_BadPSN
	add HL, BC			; HL = pt to bad psn counter
	ld A, (HL)
	inc (HL)			; stop increment after A = 16?
	ld H, A
	ld L, D
	call MultHL			; HL = counter * (max HP / 16)
	RCall( cDoDamage )		; do damage, return c if fainted
	call c, Turn_Fainted		; return c if all pkmn fainted
	ret
	
; ---------------------------------------------------------------
; Turn_Nightmare
; ---------------------------------------------------------------

Turn_Nightmare:			; if pkmn under nightmare, lose 1/4 HP
	ld BC, Turn_NightmareCalc
	jp Turn_Order
	
Turn_NightmareCalc:			; HL = pt to detailed data
	call GetPKMNStatus		; A = pkmn status
	ld BC, PKMN_Nightmare
	add HL, BC			; HL = pt to nightmare status
	cp stSLP
	jr nz, Turn_NightmareCalcNotSLP
	ld A, (HL)			; A = nightmare status
	or A				; if A = 0, (clear c flag)
	ret z				; then ret w/ nc
	
	rTextbox(tPKMNIsLockedInANightmare)
	
	sbc HL, BC			; HL = pt to detailed data
	call GetFourthMaxHP		; HL = 1/4 pkmn max HP
	RCall( cDoDamage )		; do damage, return c if fainted
	call c, Turn_Fainted		; return c if all pkmn fainted
	ret
	
Turn_NightmareCalcNotSLP:
	xor A				; A = 0, clear c flag
	ld (HL), A			; reset nightmare status
	ret				; ret w/ nc
	
; ---------------------------------------------------------------
; Turn_Trap
; ---------------------------------------------------------------

Turn_Trap:				; if trapped, then do 1/16 damage
	ld BC, Turn_TrapCalc
	jp Turn_Order
	
Turn_TrapCalc:			; HL = pt to detailed data
	ld D, H
	ld E, L			; DE = HL
	call GetPKMNTrap		; HL = pt to trap status, A = trap status, DE preserved
	ret z				; ret w/ nc if no trap
	dec A
	ld (HL), A			; decrement turn counter
	ld H, D
	ld L, E			; get HL = pt to detailed data
	or A
	jr z, Turn_TrapCalcFree
	
	rTextbox(tPKMNIsHurt)
	
	call GetSixteenthMaxHP	; HL = 1/16 of max HP
	RCall( cDoDamage )		; do damage, return c if fainted
	call c, Turn_Fainted		; return c if all pkmn fainted
	ret
	
Turn_TrapCalcFree:
	rTextbox(tPKMNWasFreed)	; disp free message
	or A				; reset carry flag
	ret
	
; ---------------------------------------------------------------
; Turn_Yawn
; ---------------------------------------------------------------

Turn_Yawn:				; if yawn counter == 1, then SLP
	ld BC, Turn_YawnCalc
	jp Turn_Order
	
Turn_YawnCalc:			; HL = pt to detailed data
	push HL
	ld BC, PKMN_Yawn
	add HL, BC
	ld A, (HL)
	ex DE, HL			; DE = pt to yawn counter
	pop HL
	or A				; if yawn == 0, then ret
	ret z
	cp 1				; if counter at 1, then SLP
	jr z, Turn_YawnCalcSLP
	dec A				; else, dec counter
	ex DE, HL
	ld (HL), A
	ret
	
Turn_YawnCalcSLP:
	ld BC, PKMN_Status
	add HL, BC
	ld A, (HL)			; A = status
	or A
	ret nz				; if already status, then ret
	ex DE, HL
	ld (HL), 0			; reset yawn counter
	ex DE, HL
	ld A, qSleep
	call AttackEffect_Call	; call AE_StatusSleep
	or A
	ret				; ret w/ nc
	
; ---------------------------------------------------------------
; Turn_Future
; ---------------------------------------------------------------

Turn_Future:				; decs future counter, deals damage
	ld BC, Turn_FutureCalc
	jp Turn_Order
	
Turn_FutureCalc:
	bit PKMNFlag, (IY + Asm_Flag1)
	jr z, Turn_FutureCalcUser

Turn_FutureCalcOpp:
	ld HL, OppFutureTurn
	jr Turn_FutureCalcCont
	
Turn_FutureCalcUser:
	ld HL, UserFutureTurn
	
Turn_FutureCalcCont:
	ld A, (HL)
	or A
	ret z				; ret w/ nc if A is zero
	dec A
	ld (HL), A			; else, dec A and write to (HL)
	or A
	ret nz				; ret w/ nc if counter not at 0
	
	call FlipPKMNFlag
	rTextbox(tPKMNTookTheAttack)
	
	ld A, mFutureHit
	ld (AtkMove), A		; set attacking move as future sight hit
	di
	RCall( cCalcDamage )		; TODO: factor in FutureSpAtk / SpDef / Hit
	ei
	RCall( cDoDamage )
	call c, Turn_Fainted		; ret c if all pkmn fainted
	jp FlipPKMNFlag		; preserves c flag
	
; ---------------------------------------------------------------
; Turn_Disable
; ---------------------------------------------------------------
	
Turn_Disable:				; decs disable counter
	ld BC, Turn_DisableCalc
	jp Turn_Order
	
Turn_DisableCalc:
	ld BC, PKMN_DisabledMove
	add HL, BC
	ld A, (HL)			; A = disabled move
	inc A				; cp $FF
	ret z				; ret if no disabled move
	inc HL				; assumes Count - Move = 1
	ld A, (HL)			; A = disable turn count
	dec A				; dec A
	ld (HL), A
	or A
	ret nz				; ret if A not at 0
	
Turn_DisableNoMore:
	dec HL				; assumes Count - Move = 1
	ld (HL), $FF			; set to no disabled move
	rTextboxRet(tPKMNIsDisabledNoMore)
	
; ---------------------------------------------------------------
; Turn_Encore
; ---------------------------------------------------------------

Turn_Encore:				; decs encore counter
	ld BC, Turn_EncoreCalc
	jp Turn_Order
	
Turn_EncoreCalc:
	ld BC, PKMN_EncoreMove
	add HL, BC
	ld A, (HL)
	inc A				; cp $FF
	ret z				; ret if no encore move
	inc HL				; assumes Count - Move = 1
	ld A, (HL)
	dec A
	ld (HL), A
	or A
	ret nz
	
Turn_EncoreNoMore:
	dec HL
	ld (HL), $FF
	rTextboxRet(tPKMNsEncoreEnded)
	
; ---------------------------------------------------------------
; Turn_Perish
; ---------------------------------------------------------------

Turn_Perish:				; dec counter, if 1 then faint
	ld BC, Turn_PerishCalc
	jp Turn_Order
	
Turn_PerishCalc:
	ld D, H			; DE = HL
	ld E, L
	ld BC, PKMN_PerishCount
	add HL, BC
	ld A, (HL)			; 0 <= A <= 4
	or A
	ret z				; ret w/ c unset if no perish count
	dec A
	ld (TextNum), A		; perish count = A - 1
	ld (HL), A
	push DE
	rTextbox(tPKMNsPerishCount)
	pop HL				; HL = pt to detailed data
	or A
	ret nz				; ret if not fainted
Turn_PerishFaint:
	call GetPKMNMaxHP		; HL = max HP
	RCall( cDoDamage )
	call c, Turn_Fainted		; return c if all pkmn fainted
	ret
	
; ---------------------------------------------------------------
; Turn_Misc
; ---------------------------------------------------------------

Turn_Misc:				; misc end of turn effects
	xor A				; reset PKMNSwitch flag
	ld (PKMNSwitch), A
	xor A				; set init opt = Fight
	ld (BattleMenuX), A
	ld (BattleMenuY), A
	
	ld BC, Turn_MiscCalc
	jp Turn_Order
	
Turn_MiscCalc:
	ld BC, PKMN_Protect
	add HL, BC
	ld (HL), 0			; reset protect status
	inc HL
	inc HL				; Endure - Protect = 2
	ld (HL), 0			; reset endure status
	ld BC, PKMN_Flinch - PKMN_Endure
	add HL, BC
	ld (HL), 0			; reset flinch status
	ld BC, PKMN_Torment - PKMN_Flinch
	add HL, BC
	call Turn_MiscCalcDec
	
Turn_MiscCalcSkip:
	ld BC, PKMN_Destiny - PKMN_Torment
	add HL, BC
	call Turn_MiscCalcDec
	
	inc HL				; Grudge - Destiny = 1, fall through
Turn_MiscCalcDec:			; subroutine - adds BC to HL, dec (HL) if value at (HL) is not zero
	ld A, (HL)
	or A
	ret z				; skip if value at (HL) is zero
	dec (HL)			; else, dec counter (if (HL) was 2, then effect works next turn, if (HL) was 1, then effect stops)
	ret
	
; ---------------------------------------------------------------
; Turn_Order
; ---------------------------------------------------------------

Turn_Order:				; handles turn priority, BC = routine pt
	call Turn_Priority		; BC not affected
	push BC
	push DE
	push HL
	ld IX, Turn_OrderRet
	push IX			; push ret address
	push BC			; BC on top of stack
	ret				; PC = BC, BC popped off stack
Turn_OrderRet:
	pop DE
	pop HL
	pop BC
	ret c				; ret if subroutine returns c, all pkmn fainted
	call FlipPKMNFlag		; HL, BC not affected
	
	push BC			; BC on top of stack
	ret				; PC = BC, BC popped off stack
	
; ---------------------------------------------------------------
; Turn_SetPriority
; ---------------------------------------------------------------
	
Turn_SetPriority: 			; sets player/opp priority
	ld A, (CurMoveOpp)
	ld B, A
	ld A, (CurMovePlayer)
	
	RCall( cCalcPriority )
	ld A, 1			; A = 1 by default, c flag not changed
	jr c, Turn_SetPriorityPlayer
Turn_SetPriorityOpp:
	xor A				; if opp, then A = 0
	;ld (EOTPriority), A
	;ret
	
Turn_SetPriorityPlayer:
	;ld A, 1
	ld (EOTPriority), A		; set priority, return
	ret
	
; ---------------------------------------------------------------
; Turn_Priority
; ---------------------------------------------------------------

Turn_Priority:			; sets HL and DE based on priority
	ld A, (EOTPriority)
	or A
	jr nz, Turn_PriorityPlayer
Turn_PriorityOpp:			; Opp - HL = OppPKMN, DE = PlayerPKMN, PKMNFlag reset
	ld HL, OppPKMN
	ld DE, PlayerPKMN
	res PKMNFlag, (IY + Asm_Flag1)
	ret
	
Turn_PriorityPlayer:			; Player - HL = PlayerPKMN, DE = OppPKMN, PKMNFlag set
	ld HL, PlayerPKMN
	ld DE, OppPKMN
	set PKMNFlag, (IY + Asm_Flag1)
	ret

; ---------------------------------------------------------------
; PKMN Fainting
; ---------------------------------------------------------------

Turn_Fainted:				; handles pkmn fainting (player + opp), return c if all pkmn fainted
	ld HL, PlayerFainted		; in case of draw (player + opp faint), opp wins
	ld A, (HL)
	or A
	call nz, Turn_FaintedPlayer
	push AF			; save c flag, don't ret until opp fainted
	
	ld HL, OppFainted
	ld A, (HL)
	or A
	call nz, Turn_FaintedOpp
	pop BC				; get C = c flag, F register not affected
	ret c				; return c if all pkmn fainted
	
	push BC
	pop AF				; get F = c flag
	ret c				; ret if all pkmn fainted, no switch
	
	call Turn_Switch		; select pkmn, switch data, display switch, ret c if all pkmn fainted b/c of switch
	ret				; end turn if fainted, no attacks, trigger end-of-turn effects
	
Turn_FaintedPlayer:			; subroutine - animation, player pkmn fainting, next pkmn selection
	ld HL, PlotSScreen + 43 * 12 + (8 / 8)
	call Turn_FaintedAnim	; animation at (8, 43), bottom left
	ld HL, PlotSScreen + (40 / 8) + (26 * 12)
	call Turn_FaintedFill	; clear player HP box
	
	ld HL, PlayerPKMN + PKMN_Status
	ld (HL), stFNT		; set player pkmn status to FNT
	set PKMNFlag, (IY + Asm_Flag1)
	ld HL, tPKMNFainted
	call TextboxBlink		; disp 'pkmn fainted' msg, wait for keypress
	call CopyPlayer		; copy PlayerPKMN to TeamPKMN1
	call AllPKMNFainted
	ret				; return c if all pkmn fainted
	
Turn_FaintedOpp:			; subroutine - animation, opp pkmn fainting, next pkmn selection
	ld HL, PlotSScreen + 24 * 12 + (56 / 8)
	call Turn_FaintedAnim	; animation at (56, 24)
	ld HL, PlotSScreen + (0 / 8) + (1 * 12)
	call Turn_FaintedFill	; clear opp HP box
	
	ld HL, OppPKMN + PKMN_Status
	ld (HL), stFNT		; set opp pkmn status to FNT
	res PKMNFlag, (IY + Asm_Flag1)
	ld HL, tPKMNFainted
	call TextboxBlink		; disp 'pkmn fainted' msg, wait for keypress
	call CopyOpp			; copy OppPKMN to OppPKMN1
	call AllPKMNFaintedOpp
	ret				; return c if all pkmn fainted
	
Turn_FaintedFill:			; subroutine - fills buffer section at (X, Y), input: HL = PlotSScreen + (x / 8) + (y * 12)
	ld BC, 256 * (56 / 8) + 18
	xor A				; A = 0
	RCall( cFillBuffSect )	; rFillBuffSect(0, X, Y, 56, 18)
	ret
	
Turn_FaintedAnim:			; subroutine - faining animation, scrolls down 26 times, input: HL = bottom left of region in PlotSScreen
	ld B, 26			; scroll 26 times
Turn_FaintedAnimLp:			; loop - repeats scrolling animation 26 times
	push HL
	push BC
	
	di
	ld B, 25			; move 25 rows of pixels down
Turn_FaintedAnimLp2:			; loop - moves 32 x 25 region of pixels down by one pixel, once
	push HL
	push BC
	
	ld B, 4			; 4 * 8 = 32 pixels across
Turn_FaintedAnimLp3:			; loop - copies one row of 32 pixels onto row immediately below
	ld A, (HL)			; A = 8 pixel long region in PlotSScreen
	ld DE, PlotSScreen - SaveSScreen
	or A
	sbc HL, DE			; move HL to corresponding byte in SaveSScreen
	ld C, (HL)			; C = 8 pixel long region in SaveSScreen
	ld DE, 12			; DE = number of bytes per row
	add HL, DE			; move HL down one row
	ld (HL), C			; copy pixel byte to SaveSScreen
	ld DE, PlotSScreen - SaveSScreen
	add HL, DE			; move HL to corresponding byte in PlotSScreen
	ld (HL), A			; copy pixel byte to PlotSScreen
	ld DE, 12			; DE = number of bytes per row
	or A				; reset c flag
	sbc HL, DE			; move HL back up one row
	inc HL				; move 8 pixels right to next region
	djnz Turn_FaintedAnimLp3	; loop until B = 0
	
	pop BC
	pop HL				; restore HL - points to leftmost pixel byte in row
	or A				; reset c flag
	sbc HL, DE			; move HL up one row, DE = 12
	djnz Turn_FaintedAnimLp2	; loop until B = 0
	ei
	;halt
	
	pop BC
	pop HL				; restore HL - points to bottom left of region
	djnz Turn_FaintedAnimLp	; loop until B = 0
	ret
	
; ---------------------------------------------------------------
; PKMN Switching
; ---------------------------------------------------------------

Turn_Switch:				; subroutine - select pkmn, swap data, display switch, ret c if all pkmn fainted b/c of spikes
	ld A, (PlayerFainted)
	or A
	jr z, Turn_SwitchSkip
	
	call Turn_SelectPlayer
	call Turn_SwitchPlayer
	
Turn_SwitchSkip:
	ld A, (OppFainted)
	or A
	jr z, Turn_SwitchSkip2
	
	call BattleBarOpp		; display opp bar
	ld B, 60
	call delayb			; pause
	call Turn_SelectOpp
	call Turn_SwitchOpp
	
Turn_SwitchSkip2:
	ld A, (PlayerFainted)
	or A
	jr z, Turn_SwitchSkip3
	
	call Turn_SwitchPlayerAbil	; return c if all pkmn fainted, from spikes
	jr c, Turn_SwitchSkip4	; skip if all pkmn fainted
	
Turn_SwitchSkip3:
	ld A, (OppFainted)
	or A
	jr z, Turn_SwitchSkip4
	
	call Turn_SwitchOppAbil	; return c if all pkmn fainted, from spikes
	
Turn_SwitchSkip4:			; flags not changed
	ld HL, PlayerFainted
	ld (HL), 0
	ld HL, OppFainted
	ld (HL), 0
	ret
	
Turn_SwitchPlayerAbil:		; subroutine - runs switch-in ability routines for player pkmn, and spikes damage
	call Turn_PriorityPlayer	; load HL = PlayerPKMN, set PKMNFlag
	
	jr Turn_SwitchCalc
	
Turn_SwitchOppAbil:			; subroutine - runs switch-in ability routines for opp pkmn, and spikes damage
	call Turn_PriorityOpp	; load HL = OppPKMN, res PKMNFlag
	
Turn_SwitchCalc:
	call Battle_SandStreamCalc2	; check sand stream for opp pkmn
	call Battle_IntimidateCalc2	; check intimidate for opp pkmn
	call Battle_TraceCalc2	; check trace for opp pkmn
	
	jp Battle_Spikes		; check for spikes damage
	
Turn_SelectPlayer:			; subroutine - select next player pkmn -> A, party UI
	set SwitchFlag, (IY + Asm_Flag2)	; switch after faint
	call RParty
	res SwitchFlag, (IY + Asm_Flag2)
	
	ld A, (PartySel)		; A = selected pkmn (0 - 2), when UI is closed
	cp 3
	jr nc, Turn_SelectPlayer	; A must be < 3, go back if A >= 3
	
	call Battle_Redraw		; saves AF, draws UI + graphics
	
	inc A
	;ld (NextPlayerPKMN), A	; A = team number (1 - 3)
	ret
	
Turn_SelectOpp:			; subroutine - select next opp pkmn -> A
	di
	ld HL, OppPKMN2 + PKMN_Moves
	call Turn_SelectOppScore	; B = C = pkmn 2 score
	push BC			; save pkmn 2 score
	
	ld HL, OppPKMN3 + PKMN_Moves
	call Turn_SelectOppScore	; B = C = pkmn 3 score
	pop AF				; A = pkmn 2 score
	ei
	cp B				; if A < B
	jr c, Turn_SelectOpp3	; then choose pkmn 3
Turn_SelectOpp2:			; else choose pkmn 2
	ld A, 5			; 2nd pkmn in opp team is number 5
	jr Turn_SelectOppCont
	
Turn_SelectOpp3:
	ld A, 6			; 3rd pkmn in opp team is number 6
	
Turn_SelectOppCont:
	;ld (NextOppPKMN), A
	ret
	
Turn_SelectOppScore:			; subroutine - calculates score for pkmn, input: HL = pt to pkmn moves, output: B = C = score
	push HL
	ld BC, PKMN_Status - PKMN_Moves
	add HL, BC
	ld A, (HL)			; A = pkmn status
	pop HL
	cp stFNT			; if pkmn is FNT
	ld C, 0			; score is 0 by default
	ret z				; then return 0 if pkmn is FNT
	
	ld B, 4			; 4 moves in array
Turn_SelectOppScoreLp:
	push HL
	push BC
	ld A, (HL)			; A = pkmn move
	ld HL, PlayerPKMN
	RCall( cCalcEff )		; calculates effectiveness of move, A = eff
	pop BC
	pop HL
	add A, C
	ld C, A			; C = C + eff
	inc HL				; get next move
	djnz Turn_SelectOppScoreLp	; loop until B = 0
	ld B, C			; B = C
	ret
	
Turn_SwitchPlayer:			; subroutine - switches player pkmn, A = team number (1 - 3)
Turn_SwitchPlayerNoFNT:
	ld HL, PlayerPKMN
	push AF
	call Turn_SwitchNaturalCure	; heal status, if natural cure
	pop AF
	
	call CopyPlayer
	
	push AF			; A = team number
	call BattleInitPlayerPKMN	; switch selected pkmn data into PlayerPKMN
	pop AF
	
Turn_SwitchPlayerNoInit:
	call BattleInitGet
	;call Turn_SwitchGetPKMN	; HL = pt to pkmn in player's team
	ld DE, TeamPKMN1
	;ld BC, BYTES_PER_TPKMN
	;call SwapRAM			; swap HL w/ first pkmn in TeamPKMN
	ld B, BYTES_PER_TPKMN
	call SwapRAMB
	
Turn_SwitchPlayerNoSwap:
	;rDrawRectW3(46, 27, 48, 14) ;(46, 27, 48, 14)
	call BattleGraphicsPlayer2
	
	set PKMNFlag, (IY + Asm_Flag1); disp player's pkmn name
	rTextbox(tGoForIt)		; disp msg
	
	;ld HL, OppPKMN + PKMN_Leech
	;ld (HL), 0			; resets leech seed
	ld HL, OppPKMN + PKMN_Infatuation
	ld (HL), 0			; resets opponent's infatuation
	ld HL, OppPKMN + PKMN_PrevEscape
	ld (HL), 0			; resets prev escape
	
	xor A				; init menu pos = (0, 0)
	ld (BatAtkX), A
	ld (BatAtkY), A
	
	ret
		
Turn_SwitchGetPKMN:			; subroutine - gets player pkmn, A = team num (1 - 3 | 4 - 6)
	ld BC, TeamPKMN1
	jr Turn_SwitchGetPKMNCont
	
Turn_SwitchGetPKMNOpp:
	ld BC, OppPKMN1
	sub 3				; A = A - 3, (4 - 6) -> (1 - 3)
	
Turn_SwitchGetPKMNCont:		; A = team num (1 - 3)
	dec A
	ld H, A
	ld L, BYTES_PER_TPKMN
	push BC
	call MultHL
	pop BC				; get base address
	add HL, BC 			; HL = TeamPKMN1 + (28 * (A - 1))
	ret
	
Turn_SwitchOpp:			; subroutine - switches opp pkmn, A = team number (4 - 6)
Turn_SwitchOppNoFNT:
	ld HL, OppPKMN
	push AF
	call Turn_SwitchNaturalCure	; heal status, if natural cure
	pop AF
	
	call CopyOpp
	
	push AF
	call BattleInitOpp
	pop AF
	
Turn_SwitchOppNoInit:
	call BattleInitGet
	;call Turn_SwitchGetPKMNOpp	; HL = pt to pkmn in opp's team
	ld DE, OppPKMN1
	;ld BC, BYTES_PER_TPKMN
	;call SwapRAM			; swap HL w/ first pkmn in OppPKMN
	ld B, BYTES_PER_TPKMN
	call SwapRAMB
	
	;rDrawRectW3(2, 2, 48, 14)
	call BattleGraphicsOpp2
	
	res PKMNFlag, (IY + Asm_Flag1); disp opp pkmn name
	rTextbox(tSentOut)		; disp msg
	
	xor A
	;ld HL, PlayerPKMN + PKMN_Leech
	;ld (HL), 0			; resets leech seed
	ld HL, PlayerPKMN + PKMN_Infatuation
	ld (HL), A			; resets infatuation
	ld HL, PlayerPKMN + PKMN_PrevEscape
	ld (HL), A			; resets prev escape
	
	ret
	
Turn_SwitchOpp1:
	ld HL, OppPKMN + PKMN_TeamNum	; switch OppPKMN data into team data
	ld A, (HL) 			; HL = OppPKMN1 + (28 * (A - 1))
	sub 4
	ld H, A
	ld L, BYTES_PER_TPKMN
	call MultHL
	ld BC, OppPKMN1
	add HL, BC
	ex DE, HL
	ld HL, OppPKMN
	ld BC, BYTES_PER_TPKMN
	ldir
	
	ld HL, PlayerPKMN + PKMN_Infatuation
	ld (HL), 0			; resets infatuation
	
	ret
	
Turn_SwitchNaturalCure:		; subroutine - cures status, if ability is natural cure, input: HL = pt to pkmn data
	call GetPKMNAbility		; A = pkmn ability, HL preserved
	cp aNaturalCure
	ret nz				; ret if ability is not natural cure
	
	ld BC, PKMN_Status
	add HL, BC
	ld A, (HL)			; A = pkmn status
	or A
	ret z				; ret if no status
	cp stFNT
	ret nc				; ret if status = FNT
	
	xor A				; A = 0
	ld (HL), A			; set status to stNone, if not stNone or stFNT before
	ret
	
; ---------------------------------------------------------------
; AllPKMNFainted
; Determines whether all pkmn in party have fainted
; Output: c = all pkmn fainted
;         (BattleWin) = 0 if nc, 1 if player win, $FF if opp win
; ---------------------------------------------------------------

AllPKMNFaintedOpp:			; entry point for opponent
	ld HL, OppPKMN1 + PKMN_Status
	ld B, 1
	jr AllPKMNFaintedTest

AllPKMNFainted:			; entry point for player
	ld HL, TeamPKMN1 + PKMN_Status
	ld B, $FF
	
AllPKMNFaintedTest:
	ld DE, BYTES_PER_TPKMN	; unrolled loop
	ld A, (HL)
	cp stFNT
	ccf
	ret nc				; return nc if status < stFNT (not fainted)
	add HL, DE
	ld A, (HL)
	cp stFNT
	ccf
	ret nc				; return nc if status < stFNT (not fainted)
	add HL, DE
	ld A, (HL)
	cp stFNT
	ccf
	ret nc				; return nc if status < stFNT (not fainted)
	ld A, B
	ld (BattleWin), A
	scf
	ret				; else, return c and (BattleWin) (all pkmn fainted)