; ===============================================================
; Research
; Generates opposing team, and conducts research
; ===============================================================

Research:
	call ResearchUI
	call ResearchGenOpp
	
	call ResearchType
	call ResearchDispType
	
	call ResearchStyle
	call ResearchDispStyle
	
	ld A, (MatchNum)
	cp 1
	jr z, Research1st
	
	ld A, 2			; if not first match
	ld (ResearchOpt), A		; initial option = swap
	jr ResearchCursor
	
Research1st:				; if first match
	xor A
	ld (ResearchOpt), A		; initial option = rent
	
ResearchCursor:
	call ResearchDispCursor
	
ResearchLp:
	RCall( cGetInput )
	or A
	jr z, ResearchLp
	cp gkEnter
	jr z, ResearchEnd
	cp gk2nd
	jr z, ResearchEnd
	cp gkClear
	jp z, Quit
	cp gkMode
	jp z, Quit
	cp gkRight
	jr z, ResearchRight
	cp gkLeft
	jr z, ResearchLeft
	
	jr ResearchLp
	
ResearchRight:
	ld A, (ResearchOpt)
	cp 1
	jr z, ResearchLp
	cp 4
	jr z, ResearchLp
	call ResearchDispCursor
	ld A, (ResearchOpt)
	inc A
	ld (ResearchOpt), A
	jr ResearchCursor
	
ResearchLeft:
	ld A, (ResearchOpt)
	or A
	jr z, ResearchLp
	cp 2
	jr z, ResearchLp
	call ResearchDispCursor
	ld A, (ResearchOpt)
	dec A
	ld (ResearchOpt), A
	jr ResearchCursor
	
ResearchDispCursor:			; disp XOR cursor at (A, 57)
	ld HL, ResearchCursorTab	; get x pos from table
	ld B, 0
	ld C, A
	add HL, BC
	ld A, (HL)
	ld L, 57			
	ld IX, sPokeball
	RCall( cgsPutSprite )
	ret
	
ResearchEnd:
	ld A, (ResearchOpt)
	ret
	
ResearchCursorTab:
.db 0, 22, 0, 25, 61

; ---------------------------------------------------------------
; ResearchUI
; Draws general UI
; ---------------------------------------------------------------

ResearchUI:
	di
	RCall( cClearBuff )
	
	set TextInverse, (IY + TextFlags)
	rFillBuffSect(BFill, 0, 0, 96, 7)
	rArcText(33, 0, tMatchNum)	; Disp Match #
	ld A, (MatchNum)		; Match# --> HL
	ld H, 0
	ld L, A
	RCall( cIntStr )
	b_call(_VPutS)
	res TextInverse, (IY + TextFlags)
	
	rArcText(2, 8, tBattle)	; Disp Battle #
	ld HL, BattleNum
	call LdHLInd
	RCall( cIntStr )
	RCall( cRightText )
	b_call(_VPutS)
	rArcText(2, 15, txtRound)	; Disp Round #
	ld A, (RoundNum)
	ld H, 0
	ld L, A
	RCall( cIntStr )
	RCall( cRightText )
	b_call(_VPutS)
	rCopyBuffSect(0, 0, 96, 21)	; Black and white
	
	rDrawBoxLbl(0, 23, 96)	; Type box
	rDrawBoxLbl(0, 40, 96)	; Style box
	
	set TextInverse, (IY + TextFlags)
	rArcText(2, 23, tPreferredType)	; Type text
	rCopyBuffSect(2, 23, 89, 6)
	rFillBuffSectL(WFill, 0, 23, 96, 6)
	rArcText(2, 40, tPreferredStyle)	; Style text
	rCopyBuffSect(2, 40, 91, 6)
	rFillBuffSectL(WFill, 0, 40, 96, 6)
	res TextInverse, (IY + TextFlags)
	
	ld A, (MatchNum)
	cp 1
	jr z, ResearchUI1st
	
	rArcText(7, 57, tSwap)		; 'Swap'
	rArcText(32, 57, tContinue)		; 'Continue'
	rVPutS(68, 57, tQuit)		; 'Quit'
	jr ResearchUIEnd
	
ResearchUI1st:				; if first match
	rArcText(7, 57, tRent)		; 'Rent'
	rVPutS(29, 57, tQuit)		; 'Quit'

ResearchUIEnd:
	rCopyBuffSect(0, 58, 80, 5)
	ei
	ret
	
; ---------------------------------------------------------------
; ResearchGenOpp
; Generates opponent's team
; ---------------------------------------------------------------

ResearchGenOpp:
	ld HL, SaveVar
	ld A, (HL)
	or A
	ld (HL), 0			; unconditionally reset savevar
	ret nz				; ret if savevar set
	
	di
ResearchGenOppNew:			; randomly generate 3 pokemon for opp team
	ld B, 3
	ld HL, OppTeamData
ResearchGenOppLp:
	push BC
	call GenPoke
	pop BC
	djnz ResearchGenOppLp
	
	#ifdef TEST
	jr ResearchGenOppSkip
	#endif
	
	call SelectTeamCpGetPoke	; copy GetPoke into SwapZone
	
	ld B, 3			; check 3 pkmn in opp team
	ld HL, OppTeamData		; HL = pt to start of opp team data
ResearchGenOppCheck:			; check if any of opp pkmn have same species as player pkmn
	push HL
	pop DE				; DE = pt to opp team data
	push BC
	call SelectTeamPokeID	; A = pkmn ID of opp pkmn, DE saved
	pop BC
	
	ld C, 3			; check 3 pkmn in player team
	ld HL, TeamData		; HL = pt to start of player team data
ResearchGenOppCheckLp:
	call ComparePokeID		; compares poke ID at A to poke ID given by HL
	jr z, ResearchGenOppNew	; generate new opp team if two identical species
	dec C
	jr nz, ResearchGenOppCheckLp; loop until C = 0
	
	ld C, 3			; check 3 pkmn in prev opp team
	ld HL, PrevOppTeam		; HL = pt to start of prev opp data
ResearchGenOppCheckLp2:
	call ComparePokeID		; compares poke ID at A to poke ID given by HL
	jr z, ResearchGenOppNew	; generate new opp team if two identical species
	dec C
	jr nz, ResearchGenOppCheckLp2; loop until C = 0
	
	ld C, B			; C = 3 - 1
	dec C				; C = 2 - 0
	jr z, ResearchGenOppCheckSkip ; skip if last pkmn
	ld HL, 3
	add HL, DE			; HL = next pkmn, check remaining pkmn in party for matches
ResearchGenOppCheckLp3:
	call ComparePokeID		; compares poke ID at A to poke ID given by HL
	jr z, ResearchGenOppNew	; generate new opp team if two identical species
	dec C
	jr nz, ResearchGenOppCheckLp3; loop until C = 0
	
ResearchGenOppCheckSkip:
	ld HL, 3			; HL = 3, bytes per pkmn
	add HL, DE			; HL = DE + 3, move HL to next pkmn
	djnz ResearchGenOppCheck	; loop until B = 0
	
ResearchGenOppSkip:
	RCall( cCopySave )
	ei
	ret
	
; ---------------------------------------------------------------
; ResearchType
; Finds type preference
; Input: (OppTeamData)
; Output: A = Type ID, $FF if no type
; ---------------------------------------------------------------

ResearchType:
	ld HL, ResearchTypeDat	; clear out buffer
	ld DE, ResearchTypeDat + 1
	ld BC, 17
	ld (HL), 0
	ldir
	
	ld HL, OppTeamData
	ld B, 3
ResearchTypeLp:
	push BC
	push HL
	RCall( cGetType )
	call GetByte
	inc HL
	call GetByteC			; Type2 --> C
	ld B, A			; Type1 --> A, B
	push BC
	call ResearchTypeInp
	pop BC
	ld A, C
	cp B				; If Type1 != Type2:
	jr z, ResearchTypeSkip	; inc type counter
	call ResearchTypeInp
ResearchTypeSkip:
	pop HL
	inc HL				; HL += 3
	inc HL
	inc HL
	pop BC
	djnz ResearchTypeLp
	
	ld HL, ResearchTypeDat
	ld B, 17			; iter counter
	ld D, 0			; Type
ResearchTypeLp2:
	ld A, (HL)
	cp 2				; if A >= 2
	jr c, ResearchTypeSkip2	; return D
	ld A, D
	ret
ResearchTypeSkip2:			; Else
	inc D				; next type
	inc HL				; HL ++
	djnz ResearchTypeLp2
	
	ld A, $FF			; If no type has count >= 2
	ret				; return $FF
	
ResearchTypeInp:			; subroutine - inc type counter
	ld HL, ResearchTypeDat	; input: A = Type ID (0 <= A < 17)
	ld D, 0
	ld E, A
	add HL, DE
	inc (HL)
	ret
	
; ---------------------------------------------------------------
; ResearchDispType:
; Disps type on research menu
; Input: A = Type ID, or $FF if None
; ---------------------------------------------------------------

ResearchDispType:
	di
	cp $FF
	jr nz, ResearchHasFavType	; If A == $FF:
	ld HL, tNone			;    Type = None
	jr ResearchHasNoFavType	; Else:
ResearchHasFavType:			;    Type = A
	set ROMFlag, (IY + Asm_Flag1)
	RCall( cGetTypeText )
	res ROMFlag, (IY + Asm_Flag1)
ResearchHasNoFavType:
	rTextCoord(2, 30)
	RCall( cArcText )
	ei
	ret
	
; ---------------------------------------------------------------
; ResearchStyle
; Finds battle style preference
; Input: (OppTeamData)
; Output: A = (Style) = Style
; ---------------------------------------------------------------

; ---------------------------------------------------------------
; Styles:
; 0 = Free-spirited and unrestrained
; 1 = Total preparation
; 2 = Slow and steady
; 3 = One of endurance
; 4 = High risk, high return
; 5 = Weakening the foe
; 6 = Impossible to predict
; 7 = Depends on the battle's flow
; 8 = Flexibly adaptable to the situation
; ---------------------------------------------------------------

ResearchStyle:
	di
	call ResearchStyleSub
	ld (Style), A
	ei
	ret
	
ResearchStyleSub:
	ld HL, ResearchStyleDat
	ld DE, ResearchStyleDat + 1
	ld BC, 7
	ld (HL), 0
	ldir
	
	ld HL, OppTeamData
	ld B, 3
ResearchStyleLp:			; For each Poke
	push BC
	push HL
	RCall( cGetMove )
	ld B, 4
ResearchStyleLpLp:			; For each move
	push BC
	push HL
	call GetByte
	call ResearchStyleMove	; Inc style counter
	pop HL
	inc HL
	pop BC
	djnz ResearchStyleLpLp
	pop HL
	inc HL				; HL += 3
	inc HL
	inc HL
	pop BC
	djnz ResearchStyleLp
	
	ld HL, ResearchStyleDat
	ld B, 7
	ld C, 0			; Criteria counter
ResearchStyleLp2:			; For each category
	ld D, 2			; If B => 5 (category <= 3)
	ld A, B			; D = 2
	cp 8 - 3			; Else (category > 3)
	jr c, ResearchStyleSkip	; D = 3
	inc D
ResearchStyleSkip:
	ld A, (HL)
	ld (HL), 0			; set to 0 by default
	cp D				; If A >= D
	jr c, ResearchStyleSkip2
	ld (HL), 1			; set to 1
	inc C				; increase counter
ResearchStyleSkip2:
	inc HL
	djnz ResearchStyleLp2
	
	ld A, C
	or A
	jr nz, ResearchStyleSkip3	; If no categories meet criteria
	xor A				; return 0
	ret
ResearchStyleSkip3:
	cp 3
	jr c, ResearchStyleSkip4	; If 3+ categories meet criteria
	ld A, 8			; return 8
	ret
ResearchStyleSkip4:
	
	ld HL, ResearchStyleDat + 6
	ld B, 7
ResearchStyleLp3:
	ld A, (HL)
	or A
	jr z, ResearchStyleSkip5	; if A != 0 (or == 1)
	ld A, B			; return B
	ret
ResearchStyleSkip5:
	dec HL
	djnz ResearchStyleLp3
	
	xor A				; This shouldn't really happen, but...
	ret				; ...if it does, return 0
	
ResearchStyleMove:			; subroutine - inc style counter of move
	ld HL, ResearchStyleDb	; Input: A = Move ID
	ld DE, ResearchStyleDat
	ld C, 7
ResearchStyleMoveLp:			; For each category
	ld B, (HL)			; B = size of category
	inc HL
ResearchStyleMoveLpLp:		; For each Move in category
	cp (HL)			; If Moves are equal
	jr nz, ResearchStyleMoveSkip
	ex DE, HL
	inc (HL)			; Then increment counter, (DE) ++
	ex DE, HL
ResearchStyleMoveSkip:
	inc HL
	djnz ResearchStyleMoveLpLp
	
	inc DE
	dec C
	jr nz, ResearchStyleMoveLp
	ret

; ---------------------------------------------------------------
; ResearchDispStyle
; Disp style on research menu
; Input: A = Style
; ---------------------------------------------------------------

ResearchDispStyle:
	di
	ld HL, tarStyle
	RCall( cGetTable2ROM)
	rTextCoord(2, 47)
	RCall( cArcText )
	ei
	ret
	