; ===============================================================
; Select Team Menu
; ===============================================================

; ---------------------------------------------------------------
; SwapTeam
; Draws UI, allows user to choose pkmn to swap
; Inputs: (TeamData)
; Outputs: (TeamData)
; ---------------------------------------------------------------

SwapTeam:					; choose player pkmn to swap
	set SwapFlag, (IY + Asm_Flag1)	; SwapFlag = 1 (swap)
	set SwapPlayer, (IY + Asm_Flag2)	; SwapPlayer = 1 (player swap)
	ld A, 2
	ld (SelectMax), A
	
	ld HL, TeamData			; copy team data
	call SwapTeamSetTeamArr		; into team arr
	
	jp SelectTeamSkipFlag		; skip over flag reset

SwapTeamOpp:					; choose opp pkmn to swap with
	set SwapFlag, (IY + Asm_Flag1)	; SwapFLag = 1 (swap)
	res SwapPlayer, (IY + Asm_Flag2)	; SwapPlayer = 0 (opp swap)
	ld A, 2
	ld (SelectMax), A
	
	ld HL, PrevOppTeam			; use team of previous opponent
	call SwapTeamSetTeamArr
	
	jr SelectTeamSkipFlag

SwapTeamSetTeamArr:				; copies 9 bytes at HL into team arr, fill rest w/ 0
	ld DE, SelectTeamArr			; copy team data into SelectTeamArr
	ld BC, 9
	ldir					; DE = SelectTeamArr + 9
	push DE
	pop HL					; HL = SelectTeamArr + 9
	inc DE					; DE = SelectTeamArr + 10
	ld (HL), 0				; fill remaining 6 bytes w/ 0
	ld BC, 6
	ldir
	ret

; ---------------------------------------------------------------
; SelectTeam:
; Draws UI, allows user to browse pokes and make selections
; Inputs: None
; Outputs: (TeamData)
; ---------------------------------------------------------------

SelectTeam:
	res SwapFlag, (IY + Asm_Flag1)	; SwapFlag = 0 (rent)
	set SwapPlayer, (IY + Asm_Flag2)	; SwapPlayer = 1 (no opp swap)
	ld A, 5
	ld (SelectMax), A
	
	xor A
	ld (SelectTeamQuan), A		; init #pokes = 0
	
	inc A					; A = 1
	ld (PKMNSaveCurrent), A
	
	di	
	ld B, 5				; fill selections w/ null
	ld HL, SelectTeam1
SelectTeam0Lp:
	ld (HL), $FF
	inc HL
	djnz SelectTeam0Lp
	
SelectTeamNew:				; Generate new array of pkmn
	ld B, 6				; Randomly chooses 6 pkmn
	ld HL, SelectTeamArr
SelectTeamRand:
	push BC
	call GenPoke
	pop BC
	djnz SelectTeamRand
	
	#ifdef TEST
	jr SelectTeamCheckSkip
	#endif
	
	call SelectTeamCpGetPoke		; copies GetPoke into SwapZone
	
	ld B, 5				; check all but last pkmn
	ld HL, SelectTeamArr			; HL = pt to start of team array
SelectTeamCheck:				; check for multiple pkmn w/ same species
	push BC
	call SelectTeamPokeID		; A = species ID, saves DE
	pop BC
	
	push HL
	pop DE					; DE = HL, save pt to current pkmn
	
	ld C, B				; C = number of pkmn to check after first (5 .. 1)
	ld HL, 3				; HL = 3, size of individual pkmn in array
	add HL, DE				; HL = DE + 3, move HL to next pkmn
SelectTeamCheckLp:
	call ComparePokeID			; compare poke ID of A to pkmn at HL
	jr z, SelectTeamNew			; if identical species, then generate new array
	dec C					; dec C
	jr nz, SelectTeamCheckLp		; loop until C = 0
	
	ld C, 3				; C = number of pkmn in opp team to check
	ld HL, OppTeamData			; HL = pt to start of opp team data
SelectTeamCheckLp2:
	call ComparePokeID			; compare poke ID of A to pkmn at HL
	jr z, SelectTeamNew			; if identical species, then generate new array
	dec C					; dec C
	jr nz, SelectTeamCheckLp2		; loop until C = 0
	
	ld HL, 3				; HL = 3, size of individual pkmn in array
	add HL, DE				; HL = DE + 3, move HL to next pkmn
	djnz SelectTeamCheck			; loop until B = 0
SelectTeamCheckSkip:
	ei
	
SelectTeamSkipFlag:				; converging point for rent / swap
	call SelectTeamDrawUI
	call SelectTeamText
	
	xor A					; init selection is first (0)
	ld (SelectTeamNum), A
SelectTeamDraw:
	call SelectTeamPokeName
	call SelectTeamCursor
	
SelectTeamKeyLp:
	RCall( cGetInput ) ;b_call(_GetCSC)
	or A
	jr z, SelectTeamKeyLp
	
	cp gkEnter
	jp z, SelectTeamA
	cp gk2nd
	jp z, SelectTeamA
	cp gkAlpha
	jr z, SelectTeamClear
	cp gkMode
	jp z, PartyQuit
	cp gkLeft
	jr z, SelectTeamLeft
	cp gkRight
	jr z, SelectTeamRight
	cp gkClear
	jr z, SelectTeamClear
	
	jr SelectTeamKeyLp
	
SelectTeamLeft:
	ld A, (SelectTeamNum)
	or A					; if A = 0
	jr nz, SelectTeamLeftN0		; then put cursor at max right position
	call SelectTeamCursor2
	ld A, (SelectMax)
	ld (SelectTeamNum), A
	jr SelectTeamDraw
SelectTeamLeftN0:
	call SelectTeamCursor2
	ld A, (SelectTeamNum)		; else dec A
	dec A
	ld (SelectTeamNum), A
	jp SelectTeamDraw
	
SelectTeamRight:
	ld A, (SelectMax)
	ld B, A				; B = (SelectMax)
	ld A, (SelectTeamNum)
	cp B					; if A = (SelectMax)
	jr nz, SelectTeamRightN5		; then set A = 0
	call SelectTeamCursor2
	xor A
	ld (SelectTeamNum), A
	jp SelectTeamDraw
SelectTeamRightN5:
	call SelectTeamCursor2		; else inc A
	ld A, (SelectTeamNum)
	inc A
	ld (SelectTeamNum), A
	jp SelectTeamDraw
	
SelectTeamClear:
	bit SwapFlag, (IY + Asm_Flag1)
	jr z, SelectTeamKeyLp		; return to lp if renting
	bit SwapPlayer, (IY + Asm_Flag2)
	jr nz, SelectTeamClearPlayer
SelectTeamClearOpp:				; if swapping opp pkmn, return to player pkmn selection
	call SelectTeamScroll
	jp SwapTeam
	
SelectTeamClearPlayer:			; if swapping player pkmn, return to main prog
	xor A
	ld (SelectTeamOptNum), A		; set opt num to 0
	
SelectTeamClearDraw:
	call SelectTeamDrawMenu		; draw background for menu
	
	di
	set TextInverse, (IY + TextFlags)	; Disp prompt text
	rArcText(1, 49, tQuitSwapping)	; 'Quit swapping?' text
	rCopyBuffSect(0, 50, 52, 5)
	rArcText(64, 43, tYes)		; Disp Yes/No options
	rArcText(64, 50, tNo)
	rCopyBuffSect(64, 44, 10, 12)
	res TextInverse, (IY + TextFlags)
	ei
	
	call SelectTeamOptCursor
	
	RCall( cWaitYes )		; waits for yes / no response
	jr nc, SelectTeamClearDraw	; redraw if return nc, option not selected
	
	or A				; If 'Yes'
	ret z				; Then return
	call SelectTeamRedraw	; Else 'No', redraw UI	
	jp SelectTeamKeyLp		; return to key loop
	
SelectTeamA:					; jump to selectteamopt, or selectteamok, depending on PlayerSwap flag
	bit SwapPlayer, (IY + Asm_Flag2)	; if SwapPlayer = 1 
	jr nz, SelectTeamOpt			; then go to opt (TODO: optimize?)
	jp SelectTeamOK			; else go to ok (opp swap)
	
SelectTeamCpGetPoke:				; subroutine - copies GetPoke into SwapZone
	ld BC, GetPoke			; BC = index of GetPoke routine in data
	call RCallCopy			; copy routine at BC into SwapZone
	ret
	
SelectTeamPokeID:				; subroutine - inline GetPokeID, assumes GetPoke in SwapZone, saves HL
	push HL
	call SwapZone
	call GetByte
	pop HL
	ret
	
ComparePokeID:				; subroutine - compares poke ID, input: HL = pt to pkmn data, A = poke ID, output: z, preserves BC, A
	push BC				; save BC
	push AF				; save A
	call SelectTeamPokeID		; A = pkmn ID
	ld B, A				; B = pkmn ID
	pop AF					; get A = pkmn ID to compare to
	cp B					; compare A and B
	pop BC					; get BC
	inc HL
	inc HL
	inc HL					; HL += 3, move HL to next pkmn, flags unchanged
	ret					; ret w/ z if equal, nz otherwise
	
; ---------------------------------------------------------------
; Select Team pkmn Option menu
; ---------------------------------------------------------------

SelectTeamOpt:
	xor A				; init opt = Summary
	ld (SelectTeamOptNum), A
	
SelectTeamOptDraw2:
	call SelectTeamCur		; HL = pt to pkmn data
	call SelectTeamOKDrawPKMN1
	
SelectTeamOptDraw:
	call SelectTeamDrawMenu
	
	di
	set TextInverse, (IY + TextFlags)	; draw text
	rArcText(63, 43, tSummary)
	ld HL, 50 * 256 + 63		; (63, 50)
	ld (PenCol), HL
	
	bit SwapFlag, (IY + Asm_Flag1)
	jr z, SelectTeamOptRent	; If swapping...
	ld HL, tSwap			; ...Then disp swap text
	jr SelectTeamOptCont
	
SelectTeamOptRent:			; If renting and...
	call SelectTeamOptTest	; If current poke is not yet rented...
	jr z, SelectTeamOptRented	; ...Then disp rent text
	ld HL, tRent
	jr SelectTeamOptCont
SelectTeamOptRented:
	ld HL, tUnrent		; ..Else disp unrent text
	
SelectTeamOptCont:
	RCall( cArcText )
	rArcText(63, 57, tOthers)
	res TextInverse, (IY + TextFlags)
	rCopyBuffSect(56, 44, 38, 19)
	ei
	
	call SelectTeamOptCursor
	
SelectTeamOptKeyLp:
	RCall( cGetInput );b_call(_GetCSC)
	or A
	jr z, SelectTeamOptKeyLp
	
	cp gkEnter
	jr z, SelectTeamOptA
	cp gk2nd
	jr z, SelectTeamOptA
	cp gkAlpha
	jr z, SelectTeamOptRet
	cp gkClear
	jr z, SelectTeamOptRet
	cp gkMode
	jp z, PartyQuit
	cp gkUp
	jr z, SelectTeamOptUp
	cp gkDown
	jr z, SelectTeamOptDown
	
	jr SelectTeamOptKeyLp
	
SelectTeamOptUp:
	ld A, (SelectTeamOptNum)
	or A
	jr nz, SelectTeamOptUpN0
	ld A, 2
	ld (SelectTeamOptNum), A
	jp SelectTeamOptDraw
SelectTeamOptUpN0:
	dec A
	ld (SelectTeamOptNum), A
	jp SelectTeamOptDraw
	
SelectTeamOptDown:
	ld A, (SelectTeamOptNum)
	cp 2
	jr nz, SelectTeamOptDownN2
	xor A
	ld (SelectTeamOptNum), A
	jp SelectTeamOptDraw
SelectTeamOptDownN2:
	inc A
	ld (SelectTeamOptNum), A
	jp SelectTeamOptDraw
	
SelectTeamOptRet:
	;rFillBuffSect(WFill, 56, 43, 40, 21)
	;call SelectTeamDrawBR
	call SelectTeamRedraw
	jp SelectTeamKeyLp
	
SelectTeamOptA:
	bit SwapFlag, (IY + Asm_Flag1)
	jr z, SelectTeamOptARent		; else swap
	
	ld A, (SelectTeamOptNum)
	cp 2
	jr z, SelectTeamOptRet
	or A
	jp z, SelectTeamOptASum
	
	call SelectTeamScroll
	
	ld A, (SelectTeamNum)
	ld (SwapPKMN), A		; save selected player pkmn
	jp SwapTeamOpp
	
SelectTeamScroll:			; scrolling pokeball animation
	call SelectTeamRedraw	; TODO: save 24 x 25 region, then paste back?
	xor A
	rLdPoint(0, 29) \ ld (HL), A	; clear lines under the pokeballs
	rLdPoint(0, 30) \ ld (HL), A	; to not interfere w/ scrolling
	rLdPoint(88, 29) \ ld (HL), A
	rLdPoint(88, 30) \ ld (HL), A
	
	ld B, 97			; loop 97 times, once per pixel
SelectTeamOptLp:			; animation loop
	push BC
	di
	ld HL, PlotSScreen + 21 * 12; (0, 21)
	call SelectTeamOptAnim
	ld HL, SaveSScreen + 21 * 12; (0, 21)
	call SelectTeamOptAnim
	ei				; TODO: display lines under pokeballs?
	pop BC
	djnz SelectTeamOptLp
	ret
	
SelectTeamOptAnim:			; animation subroutine, scrolls one pix to left, works w/ Plot and Save Screen
	push HL			; save pt to screen buffer
	ld B, 10 * 12			; 10 full rows
	scf
	ccf				; c flag = 0
SelectTeamOptLp2:
	ld A, (HL)
	rra				; rr A, c -> MSB, LSB -> c
	ld (HL), A
	inc HL				; c flag unchanged
	djnz SelectTeamOptLp2	; c flag unchanged
	pop HL
	
	ld B, 10			; 10 rows
SelectTeamOptLp3:
	push HL
	ld A, (HL)			; take byte at left side of screen
	and %10000000			; bitmask to get only leftmost pixel
	ld DE, 12
	or A				; reset c flag
	sbc HL, DE			; HL -= 12, move up one row
	push AF
	ld A, (HL)			; A = current pixel in buffer
	and %01111111			; bitmask to remove leftmost pixel
	pop DE				; D = previous pixel
	or D				; or pixel on bottom with byte on top
	ld (HL), A			; put result into (HL)
	pop HL
	ld A, %01111111
	and (HL)
	ld (HL), A			; (HL) &= %011111111, removes leftmost pixel of bottommost byte
	ld DE, 12
	add HL, DE			; HL += 12, move down one row
	djnz SelectTeamOptLp3
	ret
	
SelectTeamOptARent:			; if renting
	ld A, (SelectTeamOptNum)
	cp 2
	jp z, SelectTeamOptRet
	or A
	jr z, SelectTeamOptASum
	call SelectTeamOptTest	; else, rent / unrent pkmn
	jr nz, SelectTeamOptA1	; If pkmn already rented
	call SelectTeamUnrent	; unrent
	
	ld A, (SelectTeamNum)
	jr SelectTeamOptA2
SelectTeamOptA1:
	call SelectTeamOptGet	; Else, rent
	ld A, (SelectTeamNum)
	ld (HL), A
	ld HL, SelectTeamQuan
	inc (HL)
SelectTeamOptA2:
	call SelectTeamOptADBall
	call SelectTeamPromptN
	
	ld A, (SelectTeamQuan)
	cp 3				; if 3 pokes selected
	jr z, SelectTeamOK		; then check if ok
	
	jp SelectTeamOptRet
	
SelectTeamOptADBall:			; subroutine - removes normal pokeball, disps darker one
	ld H, A			; Input: A = SelectTeamNum
	bit SwapFlag, (IY + Asm_Flag1)
	jr nz, SelectTeamOptADBallSwap
	
SelectTeamOptADBallRent:		; if renting
	ld L, 14			; A = (14 * Num) + 8
	call MultHL
	ld A, 8
	jr SelectTeamOptADBallCont
	
SelectTeamOptADBallSwap:		; if swapping
	ld L, 19			; A = (19 * Num) + 24
	call MultHL
	ld A, 24
	
SelectTeamOptADBallCont:
	add A, L
	push AF
	ld L, 21
	ld IX, sPokeballB
	RCall( cgsLargeSprite )	; xor out regular sprite
	pop Af
	ld L, 21
	ld IX, sPokeballD
	RCall( cgsLargeSprite )	; replace with darker sprite
	ret
	
; ---------------------------------------------------------------
; Summary/Info screen functionality
; ---------------------------------------------------------------

SelectTeamOptASum:
	xor A				; init page = Info
	ld (InfoPage), A
	
SelectTeamOptASum2:
	call SelectTeamCur
	call Info2
	
	or A
	jr z, SelectTeamOptASum0
	cp 1
	jr z, SelectTeamOptASum1	; If Mode == Down
	ld A, (SelectMax)
	ld B, A			; B = (SelectMax)
	ld A, (SelectTeamNum)	; move right
	cp B				; only if A /= (SelectMax)
	jr z, SelectTeamOptASum2
	inc A
	ld (SelectTeamNum), A
	jr SelectTeamOptASum2
	
SelectTeamOptASum1:
	ld A, (SelectTeamNum)	; Elif Mode == Up
	or A				; move left
	jr z, SelectTeamOptASum2
	dec A
	ld (SelectTeamNum), A
	jr SelectTeamOptASum2
	
SelectTeamOptASum0:
	call SelectTeamRedraw
	
	jp SelectTeamOptDraw2
	
; ---------------------------------------------------------------
; Check if team ok, then either proceed or return
; ---------------------------------------------------------------

SelectTeamOK:
	xor A					; init opt = Yes
	ld (SelectTeamOptNum), A
	
	bit SwapFlag, (IY + Asm_Flag1)	; if swapping
	jr nz, SelectTeamOKPicPKMN		; then draw one pkmn
	
	ld A, (SelectTeam1)			; else draw 3 pkmn
	ld HL, 6 * 256 + 13
	call SelectTeamOKDrawPKMN
	ld A, (SelectTeam2)
	ld HL, 35 * 256 + 13
	call SelectTeamOKDrawPKMN
	ld A, (SelectTeam3)
	ld HL, 64 * 256 + 13
	call SelectTeamOKDrawPKMN
	jr SelectTeamOKSkipPKMN
	
SelectTeamOKPicPKMN:
	ld A, (SelectTeamNum)
	ld HL, PrevOppTeam
	call SelectTeamCur3
	call SelectTeamOKDrawPKMN1
	
SelectTeamOKSkipPKMN:
SelectTeamOKDraw:
	call SelectTeamDrawMenu		; draw background for menu
	
	di
	set TextInverse, (IY + TextFlags)	; Disp prompt text
	
	bit SwapFlag, (IY + Asm_Flag1)
	jr nz, SelectTeamOKDrawSwap
SelectTeamOKDrawRent:
	rArcText(1, 49, tPKMNTeamOK)
	jr SelectTeamOKDrawCont
SelectTeamOKDrawSwap:
	rArcText(1, 49, tAcceptPKMN)
	
SelectTeamOKDrawCont:
	rCopyBuffSect(0, 50, 52, 5)
	rArcText(64, 43, tYes)		; Disp Yes/No options
	rArcText(64, 50, tNo)
	rCopyBuffSect(64, 44, 10, 12)
	res TextInverse, (IY + TextFlags)
	ei
	
	call SelectTeamOptCursor
	
	RCall( cWaitYes )		; waits for yes / no response
	jr nc, SelectTeamOKDraw	; redraw if return nc, option not selected
	
SelectTeamOKA:			; A = (SelectTeamOptNum)
	or A				; If 'No'
	jr nz, SelectTeamOKB		; then cancel and return
	
	bit SwapFlag, (IY + Asm_Flag1)
	jr nz, SelectTeamOKASwap
	
SelectTeamOKARent:			; rent - copies selected pkmn into team data
	ld HL, SelectTeam1		; Else (Yes)
	ld DE, TeamData		; Copy selected data into team data
	ld B, 3			; 3 poke in team
SelectTeamOKALp:
	push HL
	push BC
	ld A, (HL)
	call SelectTeamCur2		; preserves DE
	ld BC, 3			; 3 bytes per entry
	ldir				; DE += 3
	pop BC
	pop HL
	inc HL
	djnz SelectTeamOKALp
	ret				; return to main prog flow
	
SelectTeamOKASwap:			; swap - copies selected pkmn into swapped pkmn
	call SelectTeamCur		; HL = pt to currently selected opp pkmn
	push HL
	ld A, (SwapPKMN)
	ld HL, TeamData
	call SelectTeamCur3
	ex DE, HL			; DE = pt to previously selected player pkmn
	pop HL				; HL = pt to opp pkmn
	ld BC, 3
	ldir
	ld HL, PKMNSaveSwap
	inc (HL)			; inc swap number
	ret				; return to main prog flow
	
SelectTeamOKB:
	ld HL, SelectTeam3		; unrent last poke in queue
	call SelectTeamUnrent
	call SelectTeamRedraw
	jp SelectTeamKeyLp
	
SelectTeamOKDrawPKMN:		; subroutine - draws pkmn sprite, input: A = selection number, HL = coordinates
	di
	push HL
	call SelectTeamCur2		; HL = pt to pkmn data
	RCall( cGetPokeID )
	pop HL
	RCall( cArcPKMN )		; display pkmn front sprite
	ei
	ret
	
SelectTeamOKDrawPKMN1:		; subroutine - draws single pkmn sprite in middle of screen (35, 13), input: HL = pt to pkmn data
	di
	RCall( cGetPokeID )		; A = pkmn ID
	rArcPKMN(35, 13)		; display pkmn front sprite
	ei
	ret
	
; ---------------------------------------------------------------
; SubSubroutine that draws the background UI
; ---------------------------------------------------------------

SelectTeamDrawUI:
	di				; optimize spd
	RCall( cClearBuff )
	
	rHorizLine(0, 5, 40)		; Draws top-left graphics
	rDrawRectB(0, 6, 40, 7)
	rGSPutSprite(40, 5, sSelGradTL)
	
	rDrawRectB(49, 0, 47, 7)	; Draws top-right graphics
	rDrawRectB(61, 7, 35, 6)
	rGSPutSprite(41, 0, sSelGradTR)
	rGSPutSprite(53, 7, sSelGradTRa)
	
	ld HL, PlotSScreen + (48 / 8) + (7 * 12)
	ld (HL), %11111011
	
	rHorizLine(0, 13, 96)	; Top
	rHorizLine(0, 14, 77)
	rGSPutSprite(72, 15, sSelGradTR3)
		
	rHorizLineB(0, 15, 74)	; Top lines
	ld HL, PlotSScreen + (72 / 8) + (15 * 12)
	ld (HL), %11011100
	rHorizLine(76, 16, 20)
	rHorizLine(77, 17, 19)
	rHorizLineB(77, 18, 19)
	
	bit SwapFlag, (IY + Asm_Flag1)
	jr nz, SelectTeamPokeballSwap
	
	ld A, 8			; Pokeballs (rental)
SelectTeamLp6:			; For(A, 8, 78, 14)
	push AF			; rGSLargeSprite(A, 21, PokeballB)
	ld L, 21			; End
	ld IX, sPokeballB
	RCall( cgsLargeSprite )
	pop AF
	add A, 14
	cp 78 + 14
	jr nz, SelectTeamLp6
	jr SelectTeamPokeballSkip
	
SelectTeamPokeballSwap:
	ld A, 24			; Pokeballs (swap)
SelectTeamLp62:			; For(A, 24, 62, 19)
	push AF			; rGSLargeSprite(A, 21, PokeballB)
	ld L, 21			; End
	ld IX, sPokeballB
	RCall( cgsLargeSprite )
	pop AF
	add A, 19
	cp 62 + 19
	jr nz, SelectTeamLp62
	
	bit SwapPlayer, (IY + Asm_Flag2)
	jr z, SelectTeamPokeballSkip
	
	xor A				; if player swap, then shade in all pokeballs
	call SelectTeamOptADBall
	ld A, 1
	call SelectTeamOptADBall
	ld A, 2
	call SelectTeamOptADBall
	
SelectTeamPokeballSkip:
	di
	rLdPoint(0, 29) \ ld (HL), %11000000	; Lines under the pokeballs
	rLdPoint(0, 30) \ ld (HL), %00100000
	rLdPoint(88, 29) \ ld (HL), %00000011
	rLdPoint(88, 30) \ ld (HL), %00000100
	
	rLdPoint(0, 32) \ ld (HL), %11000000
	rLdPoint(0, 33) \ ld (HL), %00100000
	rLdPoint(88, 32) \ ld (HL), %00000011
	rLdPoint(88, 33) \ ld (HL), %00000100
	
	rHorizLineB(0, 38, 96)
	rDrawRect(0, 39, 96, 3)
	
	rHorizLine(0, 47, 55)			;  bottom left text box
	rHorizLine(0, 57, 55)
	rVertLine(55, 48, 9)
	
	rDrawRectB(0, 48, 55, 9)
	
	rLdPointG(48, 48) \ ld (HL), %11111100
	rLdPoint(48, 48) \ ld (HL), %11111011
	rLdPointG(48, 49) \ ld (HL), %11111110
	rLdPoint(48, 49) \ ld (HL), %11111101
	
	rLdPoint(48, 55) \ ld (HL), %11111101
	rLdPointG(48, 55) \ ld (HL), %11111110
	rLdPoint(48, 56) \ ld (HL), %11111011
	rLdPointG(48, 56) \ ld (HL), %11111100
	
	call SelectTeamDrawBR			; bottom right box
	
	ei	
	ret
	
; ---------------------------------------------------------------
; Other Subsubroutines
; ---------------------------------------------------------------

SelectTeamCursor:		; Subroutine - Draws cursor and lines
	call SelectTeamCursor2
	
	di
	rHorizLine(3, 31, 90)
	rHorizLine(3, 34, 90)
	ei
	ret
	
SelectTeamCursor2:		; Subroutine - Draws cursor
	di
	ld A, (SelectTeamNum); Input: (SelectTeamNum) = # (0-5)
	ld H, A
	bit SwapFlag, (IY + Asm_Flag1)
	jr nz, SelectTeamCursor2Swap
	
	ld L, 14		; if renting
	call MultHL		; A = (# * 14) + 9
	ld A, 9
	jr SelectTeamCursor2Add
	
SelectTeamCursor2Swap:
	ld L, 19		; else if swapping
	call MultHL		; A = (# * 19) + 25
	ld A, 25
	
SelectTeamCursor2Add:
	add A, L
	
	ld IX, sSelTCursor
	ld L, 31
	RCall( cgsPutSprite )
	ei
	ret

SelectTeamDrawBR:
	di
	rLdPoint(72, 43) \ ld (HL), %11110000	; bottom right
	rLdPointG(72, 43) \ ld (HL), %00001111
	ld HL, %1110000100000001 \ ld (PlotSScreen + (64 / 8) + (44 * 12)), HL
	rLdPointG(72, 44) \ ld (HL), %00011111
	
	ld HL, PlotSScreen + (64 / 8) + (45 * 12)
	ld DE, SaveSScreen + (72 / 8) + (45 * 12)
	ld A, 19
SelectTeamLp8:
	ld (HL), %00000011
	inc HL
	ld (HL), %11000011
	push AF
	ld A, %00111111
	ld (DE), A
	pop AF
	ld BC, 11
	add HL, BC
	push HL
	ld HL, 12
	add HL, DE
	ex DE, HL
	pop HL
	dec A
	jr nz, SelectTeamLp8
	
	rDrawRectB(80, 43, 16, 21)
	ei
	ret
	
SelectTeamOptGet:			; subroutine to get last poke in queue
	ld HL, SelectTeamQuan	; HL = SelectTeam1 + (SelectTeamQuan)
	ld B, 0
	ld C, (HL)
	inc HL				; move up to SelectTeam1
	add Hl, BC
	ret
	
SelectTeamOptTest:			; subroutine - ret z if cur poke already in queue
	ld HL, SelectTeamQuan	; Out: HL = location of poke in queue
	ld B, (HL)
	xor A
	cp B
	jr z, SelectTeamOpt0
	ld A, (SelectTeamNum)
	ld HL, SelectTeam1
SelectTeamOptTestLp:
	cp (HL)
	ret z
	inc HL
	djnz SelectTeamOptTestLp
SelectTeamOpt0:
	dec B				; $FF, should ret nz
	ret
	
SelectTeamPromptN:			; subroutine - disps n(st|nd|rd) at prompt txt
	ld A, (SelectTeamQuan)	; n = (SelectTeamQuan)
	cp 3
	ret nc				; 0 <= A < 3, or else ret
	
	di
	ld HL, 49 * 256 + 24
	ld (PenCol), HL
	add A, A			; A = (SelectTeamQuan)
	add A, A			; A = A * 4
	ld HL, t1st
	ld B, 0
	ld C, A
	add Hl, BC
	set TextInverse, (IY + TextFlags)
	b_call(_VPutS)
	ld A, ' '
	b_call(_VPutMap)
	ld HL, tPKMN
	b_call(_VPutS)
	ld A, ' '
	b_call(_VPutMap)
	ld A, ' '
	b_call(_VPutMap)
	res TextInverse, (IY + TextFlags)
	rCopyBuffSect(24, 50, 30, 5)
	ei
	ret
	
SelectTeamCur:			; subroutine - returns cur (or inputted) poke data
	ld A, (SelectTeamNum)
SelectTeamCur2:
	ld HL, SelectTeamArr
SelectTeamCur3:
	ld B, A			; A * 3
	add A, A
	add A, B
	ld B, 0
	ld C, A
	add HL, BC
	ret
	
SelectTeamText:			; subroutine - displays text (always the same)
	di
	set TextInverse, (IY + TextFlags)
	rTextCoord(1, 6)		; disp 'rental pkmn' | 'pkmn swap' at (1, 6)
	bit SwapFlag, (IY + Asm_Flag1)
	jr nz, SelectTeamTextSwap
	rVPutS(XX, XX, tRentalPKMN)
	jr SelectTeamTextCont
	
SelectTeamTextSwap:
	rArcText(XX, XX, tPKMNSwap)
	
SelectTeamTextCont:
	rVPutS(1, 49, txtSelect)
	ld HL, PenCol \ inc (HL)
	
	bit SwapFlag, (IY + Asm_Flag1)	; if swapflag
	jr nz, SelectTeamTextSkip		; then disp 'select pkmn'
	
	rVPutS(XX, XX, t1st)			; else disp 'select 1st pkmn'
	ld HL, PenCol \ inc (HL)
	
SelectTeamTextSkip:
	rVPutS(XX, XX, tPKMN)
	res TextInverse, (IY + TextFlags)
	
	rCopyBuffSect(0, 7, 40, 5)
	rCopyBuffSect(0, 50, 50, 5)
	ei
	ret
	
SelectTeamPokeName:			; subroutine - writes pokes name
	di
	rDrawRectB(55, 4, 41, 5)
	call SelectTeamCur
	RCall( cGetName )
	
	ld B, 10
	set ROMFlag, (IY + Asm_Flag1)
	ld A, B
	ld (TextSize), A		; max size is 10
	RCall( cArcTextDc )		; HL = pt to decoded text
	push HL
	ld B, 10
	res ROMFlag, (IY + Asm_Flag1)
	RCall( cStringLen )
	
	ld B, 0			; BC = length
	ld HL, 95			; HL = 95 - BC
	scf \ ccf
	sbc HL, BC
	ld H, 3
	ld (PenCol), HL		; (L, 3)
	pop HL
	set TextInverse, (IY + TextFlags)
	b_call(_VPutS)
	rCopyBuffSect(55, 4, 40, 5)
	res TextInverse, (IY + TextFlags)
	ei
	ret
	
SelectTeamDrawMenu:			; subroutine - draws menu background
	di
	rHorizLineB(62, 43, 34)
	rHorizLineB(61, 44, 35)	; bottom right
	rDrawRectB(60, 45, 36, 19)	
	ei
	ret
	
SelectTeamOptCursor:			; subroutine - draws menu cursor
	ld A, (SelectTeamOptNum)	; Y = (7 * OptNum) + 43
	ld H, A			; S = (SaveSScreen) + (56 / 8) + (12 * Y)
	ld L, 7 * 12			; therefore, 
	call MultHL			; S = (SaveSScreen + 56/8 + 12*43 + 12*7*OptNum)
	ld BC, SaveSScreen + 56/8 + 12*43
	add HL, BC
	ld A, 0			; white fill, so only colors are white and light gray
	ld BC, ((40/8) * 256) + 7
	RCall( cFillBuffSectG )
	ret
	
SelectTeamUnrent:			; subroutine - unrent mechanics
	bit SwapFlag, (IY + Asm_Flag1)
	ret nz				; ret if swapping
	
	inc HL				; In: HL = location of poke in queue
	ld A, (HL)			; shift from a b x d e -->
	dec HL				; 	      a b d e 0
	ld (HL), A
	inc HL
	inc HL
	ld A, (HL)
	dec HL
	ld (HL), A
	
	ld HL, SelectTeamQuan	; dec Quan
	dec (HL)
	ret
	
SelectTeamRedraw:
	call SelectTeamDrawUI	; Else
	call SelectTeamText		; redraw
	call SelectTeamPokeName	; return to loop
	call SelectTeamCursor
	call SelectTeamPromptN
	
	bit SwapFlag, (IY + Asm_Flag1)
	ret nz				; If renting and...
	ld HL, SelectTeam1
	ld A, (SelectTeamQuan)	; If Quan != 0
	ld B, A			; then...
	or A
	ret z
SelectTeamOptASum0Lp:		; For(I, 1, Quan)
	push HL
	push BC
	ld A, (HL)
	call SelectTeamOptADBall
	pop BC
	pop HL
	inc HL
	djnz SelectTeamOptASum0Lp
	ret